@@ -14,7 +14,6 @@
 #    under the License.
 
 import os
-import socket
 
 from neutron_lib.utils import runtime
 from oslo_config import cfg
@@ -45,17 +44,20 @@ get_root_helper_child_pid = utils.get_root_helper_child_pid
 pid_invoked_with_cmdline = utils.pid_invoked_with_cmdline
 
 
-def load_interface_driver(conf):
+def load_interface_driver(conf, get_networks_callback=None):
     """Load interface driver for agents like DHCP or L3 agent.
 
-    :param conf: driver configuration object
+    :param conf: Driver configuration object
+    :param get_networks_callback: A callback to get network information.
+                                  This will be passed as additional keyword
+                                  argument to the interface driver.
     :raises SystemExit of 1 if driver cannot be loaded
     """
 
     try:
         loaded_class = runtime.load_class_by_alias_or_classname(
                 INTERFACE_NAMESPACE, conf.interface_driver)
-        return loaded_class(conf)
+        return loaded_class(conf, get_networks_callback=get_networks_callback)
     except ImportError:
         LOG.error("Error loading interface driver '%s'",
                   conf.interface_driver)
@@ -65,26 +67,3 @@ def load_interface_driver(conf):
 def is_agent_down(heart_beat_time):
     return timeutils.is_older_than(heart_beat_time,
                                    cfg.CONF.agent_down_time)
-
-
-def default_rp_hypervisors(hypervisors, device_mappings):
-    """Fill config option 'resource_provider_hypervisors' with defaults.
-
-    Default hypervisor names to socket.gethostname(). Since libvirt knows
-    itself by the same name, the default is good for libvirt.
-
-    :param hypervisors: Config option 'resource_provider_hypervisors'
-        as parsed by oslo.config, that is a dict with keys of physical devices
-        and values of hypervisor names.
-    :param device_mappings: Device mappings standardized to the list-valued
-        format.
-    """
-    default_hypervisor = socket.gethostname()
-    rv = {}
-    for _physnet, devices in device_mappings.items():
-        for device in devices:
-            if device in hypervisors:
-                rv[device] = hypervisors[device]
-            else:
-                rv[device] = default_hypervisor
-    return rv
