@@ -31,6 +31,7 @@ import oslo_messaging
 from oslo_service import loopingcall
 from oslo_utils import fileutils
 from oslo_utils import importutils
+from oslo_utils import timeutils
 import six
 
 from neutron._i18n import _
@@ -39,7 +40,6 @@ from neutron.agent.linux import dhcp
 from neutron.agent.linux import external_process
 from neutron.agent.metadata import driver as metadata_driver
 from neutron.agent import rpc as agent_rpc
-from neutron.common import constants as n_const
 from neutron.common import utils
 from neutron import manager
 
@@ -50,6 +50,7 @@ DEFAULT_PRIORITY = 255
 
 DHCP_PROCESS_GREENLET_MAX = 32
 DHCP_PROCESS_GREENLET_MIN = 8
+DELETED_PORT_MAX_AGE = 86400
 
 DHCP_READY_PORTS_SYNC_MAX = 64
 
@@ -87,7 +88,6 @@ class DhcpAgent(manager.Manager):
         super(DhcpAgent, self).__init__(host=host)
         self.needs_resync_reasons = collections.defaultdict(list)
         self.dhcp_ready_ports = set()
-        self.dhcp_prio_ready_ports = set()
         self.conf = conf or cfg.CONF
         # If 'resync_throttle' is configured more than 'resync_interval' by
         # mistake, raise exception and log with message.
@@ -243,29 +243,26 @@ class DhcpAgent(manager.Manager):
     def _dhcp_ready_ports_loop(self):
         """Notifies the server of any ports that had reservations setup."""
         while True:
-            # this is just watching sets so we can do it really frequently
+            # this is just watching a set so we can do it really frequently
             eventlet.sleep(0.1)
-            prio_ports_to_send = set()
-            ports_to_send = set()
-            for port_count in range(min(len(self.dhcp_prio_ready_ports) +
-                                        len(self.dhcp_ready_ports),
-                                        DHCP_READY_PORTS_SYNC_MAX)):
-                if self.dhcp_prio_ready_ports:
-                    prio_ports_to_send.add(self.dhcp_prio_ready_ports.pop())
-                    continue
-                ports_to_send.add(self.dhcp_ready_ports.pop())
-            if prio_ports_to_send or ports_to_send:
+            if self.dhcp_ready_ports:
+                ports_to_send = set()
+                for port_count in range(min(len(self.dhcp_ready_ports),
+                                            DHCP_READY_PORTS_SYNC_MAX)):
+                    ports_to_send.add(self.dhcp_ready_ports.pop())
+
                 try:
-                    self.plugin_rpc.dhcp_ready_on_ports(prio_ports_to_send |
-                                                        ports_to_send)
+                    self.plugin_rpc.dhcp_ready_on_ports(ports_to_send)
                     LOG.info("DHCP configuration for ports %s is completed",
-                             prio_ports_to_send | ports_to_send)
+                             ports_to_send)
                     continue
+                except oslo_messaging.MessagingTimeout:
+                    LOG.error("Timeout notifying server of ports ready. "
+                              "Retrying...")
                 except Exception:
                     LOG.exception("Failure notifying DHCP server of "
                                   "ready DHCP ports. Will retry on next "
                                   "iteration.")
-                self.dhcp_prio_ready_ports |= prio_ports_to_send
                 self.dhcp_ready_ports |= ports_to_send
 
     def start_ready_ports_loop(self):
@@ -547,10 +544,9 @@ class DhcpAgent(manager.Manager):
         network = self.cache.get_network_by_id(updated_port.network_id)
         if not network:
             return
-        # treat update as a create event
-        self.reload_allocations(updated_port, network, prio=True)
+        self.reload_allocations(updated_port, network)
 
-    def reload_allocations(self, port, network, prio=False):
+    def reload_allocations(self, port, network):
         LOG.info("Trigger reload_allocations for port %s", port)
         driver_action = 'reload_allocations'
         if self._is_port_on_this_agent(port):
@@ -576,10 +572,7 @@ class DhcpAgent(manager.Manager):
                 driver_action = 'restart'
         self.cache.put_port(port)
         self.call_driver(driver_action, network)
-        if prio:
-            self.dhcp_prio_ready_ports.add(port.id)
-        else:
-            self.dhcp_ready_ports.add(port.id)
+        self.dhcp_ready_ports.add(port.id)
         self.update_isolated_metadata_proxy(network)
 
     def _is_port_on_this_agent(self, port):
@@ -616,7 +609,7 @@ class DhcpAgent(manager.Manager):
                                      "DHCP cache is out of sync",
                                      created_port.network_id)
                 return
-        self.reload_allocations(created_port, network, prio=True)
+        self.reload_allocations(created_port, network)
 
     def port_delete_end(self, context, payload):
         """Handle the port.delete.end notification event."""
@@ -637,7 +630,7 @@ class DhcpAgent(manager.Manager):
             return
         port_id = payload['port_id']
         port = self.cache.get_port_by_id(port_id)
-        self.cache.deleted_ports.add(port_id)
+        self.cache.add_to_deleted_ports(port_id)
         if not port:
             return
         network = self.cache.get_network_by_id(port.network_id)
@@ -726,14 +719,15 @@ class DhcpPluginApi(object):
         1.1 - Added get_active_networks_info, create_dhcp_port,
               and update_dhcp_port methods.
         1.5 - Added dhcp_ready_on_ports
-
+        1.7 - Added get_networks
+        1.8 - Added get_dhcp_port
     """
 
     def __init__(self, topic, host):
         self.host = host
         target = oslo_messaging.Target(
                 topic=topic,
-                namespace=n_const.RPC_NAMESPACE_DHCP_PLUGIN,
+                namespace=constants.RPC_NAMESPACE_DHCP_PLUGIN,
                 version='1.0')
         self.client = n_rpc.get_client(target)
 
@@ -784,11 +778,32 @@ class DhcpPluginApi(object):
                           network_id=network_id, device_id=device_id,
                           host=self.host)
 
+    def get_dhcp_port(self, port_id):
+        """Make a remote process call to retrieve the dhcp port."""
+        cctxt = self.client.prepare(version='1.8')
+        port = cctxt.call(self.context, 'get_dhcp_port', port_id=port_id)
+        if port:
+            return dhcp.DictModel(port)
+
     def dhcp_ready_on_ports(self, port_ids):
         """Notify the server that DHCP is configured for the port."""
         cctxt = self.client.prepare(version='1.5')
-        cctxt.cast(self.context, 'dhcp_ready_on_ports',
-                   port_ids=port_ids)
+        return cctxt.call(self.context, 'dhcp_ready_on_ports',
+                          port_ids=port_ids)
+
+    def get_networks(self, filters=None, fields=None):
+        """Get networks.
+
+        :param filters: The filters to apply.
+                        E.g {"id" : ["<uuid of a network>", ...]}
+        :param fields: A list of fields to collect, e.g ["id", "subnets"].
+        :return: A list of NetModel where each object represent a network.
+        """
+
+        cctxt = self.client.prepare(version='1.7')
+        nets = cctxt.call(self.context, 'get_networks', filters=filters,
+                          fields=fields)
+        return [dhcp.NetModel(net) for net in nets]
 
 
 class NetworkCache(object):
@@ -797,13 +812,18 @@ class NetworkCache(object):
         self.cache = {}
         self.subnet_lookup = {}
         self.port_lookup = {}
-        self.deleted_ports = set()
+        self._deleted_ports = set()
+        self._deleted_ports_ts = []
+        self.cleanup_loop = loopingcall.FixedIntervalLoopingCall(
+            self.cleanup_deleted_ports)
+        self.cleanup_loop.start(DELETED_PORT_MAX_AGE,
+                                initial_delay=DELETED_PORT_MAX_AGE)
 
     def is_port_message_stale(self, payload):
         orig = self.get_port_by_id(payload['id']) or {}
         if orig.get('revision_number', 0) > payload.get('revision_number', 0):
             return True
-        if payload['id'] in self.deleted_ports:
+        if payload['id'] in self._deleted_ports:
             return True
         return False
 
@@ -890,6 +910,29 @@ class NetworkCache(object):
                 'subnets': num_subnets,
                 'ports': num_ports}
 
+    def add_to_deleted_ports(self, port_id):
+        if port_id not in self._deleted_ports:
+            self._deleted_ports.add(port_id)
+            self._deleted_ports_ts.append((timeutils.utcnow_ts(), port_id))
+
+    def cleanup_deleted_ports(self):
+        """Cleanup the "self._deleted_ports" set based on the current TS
+
+        The variable "self._deleted_ports_ts" contains a timestamp
+        ordered list of tuples (timestamp, port_id). Every port older than the
+        current timestamp minus "timestamp_delta" will be deleted from
+        "self._deleted_ports" and "self._deleted_ports_ts".
+        """
+        timestamp_min = timeutils.utcnow_ts() - DELETED_PORT_MAX_AGE
+        idx = None
+        for idx, (ts, port_id) in enumerate(self._deleted_ports_ts):
+            if ts > timestamp_min:
+                break
+            self._deleted_ports.remove(port_id)
+
+        if idx:
+            self._deleted_ports_ts = self._deleted_ports_ts[idx:]
+
 
 class DhcpAgentWithStateReport(DhcpAgent):
     def __init__(self, host=None, conf=None):
