@@ -13,6 +13,8 @@
 #    under the License.
 #
 
+import functools
+
 import eventlet
 import netaddr
 from neutron_lib.agent import constants as agent_consts
@@ -53,7 +55,6 @@ from neutron.agent.linux import pd
 from neutron.agent.linux import utils as linux_utils
 from neutron.agent.metadata import driver as metadata_driver
 from neutron.agent import rpc as agent_rpc
-from neutron.common import constants as l3_constants
 from neutron.common import ipv6_utils
 from neutron.common import utils
 from neutron import manager
@@ -113,6 +114,8 @@ class L3PluginApi(object):
         1.9 - Added get_router_ids
         1.10 Added update_all_ha_network_port_statuses
         1.11 Added get_host_ha_router_count
+        1.12 Added get_networks
+        1.13 Removed get_external_network_id
     """
 
     def __init__(self, topic, host):
@@ -120,79 +123,128 @@ class L3PluginApi(object):
         target = oslo_messaging.Target(topic=topic, version='1.0')
         self.client = n_rpc.get_client(target)
 
+    @utils.timecost
     def get_routers(self, context, router_ids=None):
         """Make a remote process call to retrieve the sync data for routers."""
         cctxt = self.client.prepare()
         return cctxt.call(context, 'sync_routers', host=self.host,
                           router_ids=router_ids)
 
+    @utils.timecost
     def update_all_ha_network_port_statuses(self, context):
         """Make a remote process call to update HA network port status."""
         cctxt = self.client.prepare(version='1.10')
         return cctxt.call(context, 'update_all_ha_network_port_statuses',
                           host=self.host)
 
+    @utils.timecost
     def get_router_ids(self, context):
         """Make a remote process call to retrieve scheduled routers ids."""
         cctxt = self.client.prepare(version='1.9')
         return cctxt.call(context, 'get_router_ids', host=self.host)
 
-    def get_external_network_id(self, context):
-        """Make a remote process call to retrieve the external network id.
-
-        @raise oslo_messaging.RemoteError: with TooManyExternalNetworks as
-                                           exc_type if there are more than one
-                                           external network
-        """
-        cctxt = self.client.prepare()
-        return cctxt.call(context, 'get_external_network_id', host=self.host)
-
+    @utils.timecost
     def update_floatingip_statuses(self, context, router_id, fip_statuses):
         """Call the plugin update floating IPs's operational status."""
         cctxt = self.client.prepare(version='1.1')
         return cctxt.call(context, 'update_floatingip_statuses',
                           router_id=router_id, fip_statuses=fip_statuses)
 
+    @utils.timecost
     def get_ports_by_subnet(self, context, subnet_id):
         """Retrieve ports by subnet id."""
         cctxt = self.client.prepare(version='1.2')
         return cctxt.call(context, 'get_ports_by_subnet', host=self.host,
                           subnet_id=subnet_id)
 
+    @utils.timecost
     def get_agent_gateway_port(self, context, fip_net):
         """Get or create an agent_gateway_port."""
         cctxt = self.client.prepare(version='1.2')
         return cctxt.call(context, 'get_agent_gateway_port',
                           network_id=fip_net, host=self.host)
 
+    @utils.timecost
     def get_service_plugin_list(self, context):
         """Make a call to get the list of activated services."""
         cctxt = self.client.prepare(version='1.3')
         return cctxt.call(context, 'get_service_plugin_list')
 
+    @utils.timecost
     def update_ha_routers_states(self, context, states):
         """Update HA routers states."""
         cctxt = self.client.prepare(version='1.5')
         return cctxt.cast(context, 'update_ha_routers_states',
                           host=self.host, states=states)
 
+    @utils.timecost
     def process_prefix_update(self, context, prefix_update):
         """Process prefix update whenever prefixes get changed."""
         cctxt = self.client.prepare(version='1.6')
         return cctxt.call(context, 'process_prefix_update',
                           subnets=prefix_update)
 
+    @utils.timecost
     def delete_agent_gateway_port(self, context, fip_net):
         """Delete Floatingip_agent_gateway_port."""
         cctxt = self.client.prepare(version='1.7')
         return cctxt.call(context, 'delete_agent_gateway_port',
                           host=self.host, network_id=fip_net)
 
+    @utils.timecost
     def get_host_ha_router_count(self, context):
         """Make a call to get the count of HA router."""
         cctxt = self.client.prepare(version='1.11')
         return cctxt.call(context, 'get_host_ha_router_count', host=self.host)
 
+    def get_networks(self, context, filters=None, fields=None):
+        """Get networks.
+
+        :param context: Security context
+        :param filters: The filters to apply.
+                        E.g {"id" : ["<uuid of a network>", ...]}
+        :param fields: A list of fields to collect, e.g ["id", "subnets"].
+        :return: A list of dicts where each dict represent a network object.
+        """
+
+        cctxt = self.client.prepare(version='1.12')
+        return cctxt.call(
+            context, 'get_networks', filters=filters, fields=fields)
+
+
+class RouterFactory(object):
+
+    def __init__(self):
+        self._routers = {}
+
+    def register(self, features, router_cls):
+        """Register router class which implements BaseRouterInfo
+
+        Features which is a list of strings converted to frozenset internally
+        for key uniqueness.
+
+        :param features: a list of strings of router's features
+        :param router_cls: a router class which implements BaseRouterInfo
+        """
+        self._routers[frozenset(features)] = router_cls
+
+    def create(self, features, **kwargs):
+        """Create router instance with registered router class
+
+        :param features: a list of strings of router's features
+        :param kwargs: arguments for router class
+        :returns: a router instance which implements BaseRouterInfo
+        :raises: n_exc.RouterNotFoundInRouterFactory
+        """
+        try:
+            router = self._routers[frozenset(features)]
+            return router(**kwargs)
+        except KeyError:
+            exc = l3_exc.RouterNotFoundInRouterFactory(
+                router_id=kwargs['router_id'], features=features)
+            LOG.exception(exc.msg)
+            raise exc
+
 
 @profiler.trace_cls("l3-agent")
 class L3NATAgent(ha.AgentMixin,
@@ -223,6 +275,8 @@ class L3NATAgent(ha.AgentMixin,
         else:
             self.conf = cfg.CONF
         self.router_info = {}
+        self.router_factory = RouterFactory()
+        self._register_router_cls(self.router_factory)
 
         self._check_config_params()
 
@@ -230,10 +284,14 @@ class L3NATAgent(ha.AgentMixin,
             config=self.conf,
             resource_type='router')
 
-        self.driver = common_utils.load_interface_driver(self.conf)
-
         self._context = n_context.get_admin_context_without_session()
         self.plugin_rpc = L3PluginApi(topics.L3PLUGIN, host)
+
+        self.driver = common_utils.load_interface_driver(
+            self.conf,
+            get_networks_callback=functools.partial(
+                self.plugin_rpc.get_networks, self.context))
+
         self.fullsync = True
         self.sync_routers_chunk_size = SYNC_ROUTERS_MAX_CHUNK_SIZE
 
@@ -328,6 +386,21 @@ class L3NATAgent(ha.AgentMixin,
             except Exception:
                 LOG.exception('update_all_ha_network_port_statuses failed')
 
+    def _register_router_cls(self, factory):
+        factory.register([], legacy_router.LegacyRouter)
+        factory.register(['ha'], ha_router.HaRouter)
+
+        if self.conf.agent_mode == lib_const.L3_AGENT_MODE_DVR_SNAT:
+            factory.register(['distributed'],
+                             dvr_router.DvrEdgeRouter)
+            factory.register(['ha', 'distributed'],
+                             dvr_edge_ha_router.DvrEdgeHaRouter)
+        else:
+            factory.register(['distributed'],
+                             dvr_local_router.DvrLocalRouter)
+            factory.register(['ha', 'distributed'],
+                             dvr_local_router.DvrLocalRouter)
+
     def _check_config_params(self):
         """Check items in configuration files.
 
@@ -353,7 +426,6 @@ class L3NATAgent(ha.AgentMixin,
                 raise SystemExit(1)
 
     def _create_router(self, router_id, router):
-        args = []
         kwargs = {
             'agent': self,
             'router_id': router_id,
@@ -363,9 +435,15 @@ class L3NATAgent(ha.AgentMixin,
             'interface_driver': self.driver,
         }
 
+        features = []
         if router.get('distributed'):
+            features.append('distributed')
             kwargs['host'] = self.host
 
+        if router.get('ha'):
+            features.append('ha')
+            kwargs['state_change_callback'] = self.enqueue_state_change
+
         if router.get('distributed') and router.get('ha'):
             # Case 1: If the router contains information about the HA interface
             # and if the requesting agent is a DVR_SNAT agent then go ahead
@@ -376,22 +454,12 @@ class L3NATAgent(ha.AgentMixin,
             # that needs to provision a router namespace because of a DVR
             # service port (e.g. DHCP). So go ahead and create a regular DVR
             # edge router.
-            if (self.conf.agent_mode == lib_const.L3_AGENT_MODE_DVR_SNAT and
-                    router.get(lib_const.HA_INTERFACE_KEY) is not None):
-                kwargs['state_change_callback'] = self.enqueue_state_change
-                return dvr_edge_ha_router.DvrEdgeHaRouter(*args, **kwargs)
+            if (not router.get(lib_const.HA_INTERFACE_KEY) or
+                    self.conf.agent_mode != lib_const.L3_AGENT_MODE_DVR_SNAT):
+                features.remove('ha')
+                kwargs.pop('state_change_callback')
 
-        if router.get('distributed'):
-            if self.conf.agent_mode == lib_const.L3_AGENT_MODE_DVR_SNAT:
-                return dvr_router.DvrEdgeRouter(*args, **kwargs)
-            else:
-                return dvr_local_router.DvrLocalRouter(*args, **kwargs)
-
-        if router.get('ha'):
-            kwargs['state_change_callback'] = self.enqueue_state_change
-            return ha_router.HaRouter(*args, **kwargs)
-
-        return legacy_router.LegacyRouter(*args, **kwargs)
+        return self.router_factory.create(features, **kwargs)
 
     @lockutils.synchronized('resize_greenpool')
     def _resize_process_pool(self):
@@ -477,7 +545,8 @@ class L3NATAgent(ha.AgentMixin,
 
     def init_extension_manager(self, connection):
         l3_ext_manager.register_opts(self.conf)
-        self.agent_api = l3_ext_api.L3AgentExtensionAPI(self.router_info)
+        self.agent_api = l3_ext_api.L3AgentExtensionAPI(self.router_info,
+                                                        self.router_factory)
         self.l3_ext_manager = (
             l3_ext_manager.L3AgentExtensionsManager(self.conf))
         self.l3_ext_manager.initialize(
@@ -574,7 +643,7 @@ class L3NATAgent(ha.AgentMixin,
             # For HA routers check that DB state matches actual state
             if router.get('ha') and not is_dvr_only_agent and is_ha_router:
                 self.check_ha_state_for_router(
-                    router['id'], router.get(l3_constants.HA_ROUTER_STATE_KEY))
+                    router['id'], router.get(lib_const.HA_ROUTER_STATE_KEY))
             ri.router = router
             registry.notify(resources.ROUTER, events.BEFORE_UPDATE,
                             self, router=ri)
@@ -589,6 +658,9 @@ class L3NATAgent(ha.AgentMixin,
         if router_update.hit_retry_limit():
             LOG.warning("Hit retry limit with router update for %s, action %s",
                         router_update.id, router_update.action)
+            if router_update.action != DELETE_ROUTER:
+                LOG.debug("Deleting router %s", router_update.id)
+                self._safe_router_removed(router_update.id)
             return
         router_update.timestamp = timeutils.utcnow()
         router_update.priority = priority
@@ -844,11 +916,8 @@ class L3NATAgentWithStateReport(L3NATAgent):
                 'agent_mode': self.conf.agent_mode,
                 'handle_internal_only_routers':
                 self.conf.handle_internal_only_routers,
-                'gateway_external_network_id':
-                self.conf.gateway_external_network_id,
                 'interface_driver': self.conf.interface_driver,
-                'log_agent_heartbeats': self.conf.AGENT.log_agent_heartbeats,
-                'extensions': self.l3_ext_manager.names()},
+                'log_agent_heartbeats': self.conf.AGENT.log_agent_heartbeats},
             'start_flag': True,
             'agent_type': lib_const.AGENT_TYPE_L3}
         report_interval = self.conf.AGENT.report_interval
