@@ -17,7 +17,6 @@ import collections
 
 import netaddr
 from neutron_lib import constants as lib_constants
-from neutron_lib import exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
 import six
@@ -25,8 +24,8 @@ import six
 from neutron.agent.l3 import dvr_fip_ns
 from neutron.agent.l3 import dvr_router_base
 from neutron.agent.linux import ip_lib
-from neutron.common import constants as n_const
 from neutron.common import utils as common_utils
+from neutron.privileged.agent.linux import ip_lib as priv_ip_lib
 
 LOG = logging.getLogger(__name__)
 # xor-folding mask used for IPv6 rule index
@@ -48,27 +47,6 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         self.fip_ns = None
         self._pending_arp_set = set()
 
-        self.load_used_fip_information()
-
-    def load_used_fip_information(self):
-        """Some information needed to remove a floating ip e.g. it's associated
-        ip rule priorities, are stored in memory to avoid extra db lookups.
-        Since this is lost on agent restart we need to reload them.
-        """
-        ex_gw_port = self.get_ex_gw_port()
-        if ex_gw_port:
-            fip_ns = self.agent.get_fip_ns(ex_gw_port['network_id'])
-
-            for fip in self.get_floating_ips():
-                floating_ip = fip['floating_ip_address']
-                fixed_ip = fip['fixed_ip_address']
-                rule_pr = fip_ns.lookup_rule_priority(floating_ip)
-                if rule_pr:
-                    self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
-                else:
-                    LOG.error("Rule priority not found for floating ip %s",
-                              floating_ip)
-
     def migrate_centralized_floating_ip(self, fip, interface_name, device):
         # Remove the centralized fip first and then add fip to the host
         ip_cidr = common_utils.ip_to_cidr(fip['floating_ip_address'])
@@ -182,11 +160,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                                   table=dvr_fip_ns.FIP_RT_TBL,
                                   priority=int(str(rule_pr)))
             self.fip_ns.deallocate_rule_priority(floating_ip)
-        else:
-            LOG.error("Unable to find necessary information to complete "
-                      "removal of floating ip rules for %s - will require "
-                      "manual cleanup (see LP 1891673 for details).",
-                      floating_ip)
+            # TODO(rajeev): Handle else case - exception/log?
 
     def floating_ip_removed_dist(self, fip_cidr):
         """Remove floating IP from FIP namespace."""
@@ -206,7 +180,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
             device = ip_lib.IPDevice(fip_2_rtr_name, namespace=fip_ns_name)
 
-            device.route.delete_route(fip_cidr, str(rtr_2_fip.ip))
+            device.route.delete_route(fip_cidr, via=str(rtr_2_fip.ip))
             return device
 
     def floating_ip_moved_dist(self, fip):
@@ -348,7 +322,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                                          snat_idx):
         try:
             ns_ip_device.route.delete_gateway(gw_ip_addr, table=snat_idx)
-        except exceptions.DeviceNotFoundError:
+        except priv_ip_lib.NetworkInterfaceNotFound:
             pass
 
     def _stale_ip_rule_cleanup(self, namespace, ns_ipd, ip_version):
@@ -501,7 +475,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def get_floating_agent_gw_interface(self, ext_net_id):
         """Filter Floating Agent GW port for the external network."""
-        fip_ports = self.router.get(n_const.FLOATINGIP_AGENT_INTF_KEY, [])
+        fip_ports = self.router.get(
+            lib_constants.FLOATINGIP_AGENT_INTF_KEY, [])
         return next(
             (p for p in fip_ports if p['network_id'] == ext_net_id), None)
 
@@ -647,7 +622,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def process_external(self):
         if self.agent_conf.agent_mode != (
-            lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL):
+                lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL):
             ex_gw_port = self.get_ex_gw_port()
             if ex_gw_port:
                 self.create_dvr_external_gateway_on_agent(ex_gw_port)
@@ -701,7 +676,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                 return
             for subnet in router_port['subnets']:
                 rtr_port_cidr = subnet['cidr']
-                device.route.delete_route(rtr_port_cidr, str(rtr_2_fip_ip))
+                device.route.delete_route(rtr_port_cidr, via=str(rtr_2_fip_ip))
 
     def _add_interface_route_to_fip_ns(self, router_port):
         rtr_2_fip_ip, fip_2_rtr_name = self.get_rtr_fip_ip_and_interface_name()
@@ -781,8 +756,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                     operation, route, fip_ns_name, tbl_index)
         super(DvrLocalRouter, self).update_routing_table(operation, route)
 
-    def _update_fip_route_table_with_next_hop_routes(
-        self, operation, route, fip_ns_name, tbl_index):
+    def _update_fip_route_table_with_next_hop_routes(self, operation, route,
+                                                     fip_ns_name, tbl_index):
         cmd = ['ip', 'route', operation, 'to', route['destination'],
                'via', route['nexthop'], 'table', tbl_index]
         ip_wrapper = ip_lib.IPWrapper(namespace=fip_ns_name)
@@ -793,8 +768,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                       "router %(id)s",
                       {'ns': fip_ns_name, 'id': self.router_id})
 
-    def _check_if_route_applicable_to_fip_namespace(
-        self, route, agent_gateway_port):
+    def _check_if_route_applicable_to_fip_namespace(self, route,
+                                                    agent_gateway_port):
         ip_cidrs = common_utils.fixed_ip_cidrs(agent_gateway_port['fixed_ips'])
         nexthop_cidr = netaddr.IPAddress(route['nexthop'])
         for gw_cidr in ip_cidrs:
