@@ -27,11 +27,9 @@ from neutron.agent.l3 import router_info as router
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
-from neutron.common import constants as const
 from neutron.common import utils as common_utils
 from neutron.extensions import revisions
 from neutron.extensions import timestamp
-from neutron.ipam import utils as ipam_utils
 
 LOG = logging.getLogger(__name__)
 HA_DEV_PREFIX = 'ha-'
@@ -40,6 +38,9 @@ SIGTERM_TIMEOUT = 10
 KEEPALIVED_STATE_CHANGE_MONITOR_SERVICE_NAME = (
     "neutron-keepalived-state-change-monitor")
 
+# TODO(liuyulong): move to neutron-lib?
+STATE_CHANGE_PROC_NAME = 'neutron-keepalived-state-change'
+
 # The multiplier is used to compensate execution time of function sending
 # SIGHUP to keepalived process. The constant multiplies ha_vrrp_advert_int
 # config option and the result is the throttle delay.
@@ -93,15 +94,6 @@ class HaRouter(router.RouterInfo):
     def ha_vr_id(self):
         return self.router.get('ha_vr_id')
 
-    def _check_and_set_real_state(self):
-        # When the physical host was down/up, the 'master' router may still
-        # have its original state in the _ha_state_path file. We directly
-        # reset it to 'backup'.
-        if (not self.keepalived_manager.check_processes() and
-                os.path.exists(self.ha_state_path) and
-                self.ha_state == 'master'):
-            self.ha_state = 'backup'
-
     @property
     def ha_state(self):
         if self._ha_state:
@@ -147,7 +139,6 @@ class HaRouter(router.RouterInfo):
 
         self.set_ha_port()
         self._init_keepalived_manager(process_monitor)
-        self._check_and_set_real_state()
         self.ha_network_added()
         self.update_initial_state(self.state_change_callback)
         self.spawn_state_change_monitor(process_monitor)
@@ -198,10 +189,7 @@ class HaRouter(router.RouterInfo):
             return
         self.keepalived_manager.disable()
         conf_dir = self.keepalived_manager.get_conf_dir()
-        try:
-            shutil.rmtree(conf_dir)
-        except FileNotFoundError:
-            pass
+        shutil.rmtree(conf_dir)
 
     def _get_keepalived_instance(self):
         return self.keepalived_manager.config.get_instance(self.ha_vr_id)
@@ -273,14 +261,6 @@ class HaRouter(router.RouterInfo):
 
         default_gw_rts = []
         instance = self._get_keepalived_instance()
-        for subnet in ex_gw_port.get('subnets', []):
-            is_gateway_not_in_subnet = (subnet['gateway_ip'] and
-                                        not ipam_utils.check_subnet_ip(
-                                            subnet['cidr'],
-                                            subnet['gateway_ip']))
-            if is_gateway_not_in_subnet:
-                default_gw_rts.append(keepalived.KeepalivedVirtualRoute(
-                    subnet['gateway_ip'], None, interface_name, scope='link'))
         for gw_ip in gateway_ips:
             # TODO(Carl) This is repeated everywhere.  A method would
             # be nice.
@@ -325,11 +305,12 @@ class HaRouter(router.RouterInfo):
 
         if self._should_delete_ipv6_lladdr(ipv6_lladdr):
             self.driver.configure_ipv6_ra(self.ha_namespace, interface_name,
-                                          const.ACCEPT_RA_DISABLED)
+                                          n_consts.ACCEPT_RA_DISABLED)
             device.addr.flush(n_consts.IP_VERSION_6)
         else:
-            self.driver.configure_ipv6_ra(self.ha_namespace, interface_name,
-                                          const.ACCEPT_RA_WITHOUT_FORWARDING)
+            self.driver.configure_ipv6_ra(
+                self.ha_namespace, interface_name,
+                n_consts.ACCEPT_RA_WITHOUT_FORWARDING)
 
         self._remove_vip(ipv6_lladdr)
         self._add_vip(ipv6_lladdr, interface_name, scope='link')
@@ -394,13 +375,17 @@ class HaRouter(router.RouterInfo):
     def _get_state_change_monitor_callback(self):
         ha_device = self.get_ha_device_name()
         ha_cidr = self._get_primary_vip()
+        config_dir = self.keepalived_manager.get_conf_dir()
+        state_change_log = (
+            "%s/neutron-keepalived-state-change.log") % config_dir
 
         def callback(pid_file):
             cmd = [
-                'neutron-keepalived-state-change',
+                STATE_CHANGE_PROC_NAME,
                 '--router_id=%s' % self.router_id,
                 '--namespace=%s' % self.ha_namespace,
-                '--conf_dir=%s' % self.keepalived_manager.get_conf_dir(),
+                '--conf_dir=%s' % config_dir,
+                '--log-file=%s' % state_change_log,
                 '--monitor_interface=%s' % ha_device,
                 '--monitor_cidr=%s' % ha_cidr,
                 '--pid_file=%s' % pid_file,
@@ -416,6 +401,10 @@ class HaRouter(router.RouterInfo):
         pm.enable()
         process_monitor.register(
             self.router_id, IP_MONITOR_PROCESS_SERVICE, pm)
+        LOG.debug("Router %(router_id)s %(process)s pid %(pid)d",
+                  {"router_id": self.router_id,
+                   "process": KEEPALIVED_STATE_CHANGE_MONITOR_SERVICE_NAME,
+                   "pid": pm.pid})
 
     def destroy_state_change_monitor(self, process_monitor):
         if not self.ha_port:
@@ -453,9 +442,7 @@ class HaRouter(router.RouterInfo):
         return port1_filtered == port2_filtered
 
     def external_gateway_added(self, ex_gw_port, interface_name):
-        link_up = self.external_gateway_link_up()
-        self._plug_external_gateway(ex_gw_port, interface_name,
-                                    self.ns_name, link_up=link_up)
+        self._plug_external_gateway(ex_gw_port, interface_name, self.ns_name)
         self._add_gateway_vip(ex_gw_port, interface_name)
         self._disable_ipv6_addressing_on_interface(interface_name)
 
@@ -519,27 +506,3 @@ class HaRouter(router.RouterInfo):
         if (self.keepalived_manager.get_process().active and
                 self.ha_state == 'master'):
             super(HaRouter, self).enable_radvd(internal_ports)
-
-    def external_gateway_link_up(self):
-        # Check HA router ha_state for its gateway port link state.
-        # 'backup' instance will not link up the gateway port.
-        return self.ha_state == 'master'
-
-    def set_external_gw_port_link_status(self, link_up, set_gw=False):
-        link_state = "up" if link_up else "down"
-        LOG.info('Set router %s gateway device link state to %s.',
-                 self.router_id, link_state)
-
-        ex_gw_port = self.get_ex_gw_port()
-        ex_gw_port_id = (ex_gw_port and ex_gw_port['id'] or
-                         self.ex_gw_port and self.ex_gw_port['id'])
-        if ex_gw_port_id:
-            interface_name = self.get_external_device_name(ex_gw_port_id)
-            ns_name = self.get_gw_ns_name()
-            self.driver.set_link_status(interface_name, ns_name,
-                                        link_up=link_up)
-            if link_up and set_gw:
-                preserve_ips = self.get_router_preserve_ips()
-                self._external_gateway_settings(ex_gw_port, interface_name,
-                                                ns_name, preserve_ips)
-                self.routes_updated([], self.routes)
