@@ -37,7 +37,6 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.cmd import runtime_checks as checks
-from neutron.common import constants as n_const
 from neutron.common import ipv6_utils
 from neutron.common import utils as common_utils
 from neutron.ipam import utils as ipam_utils
@@ -215,7 +214,7 @@ class DhcpLocalProcess(DhcpBase):
     def enable(self):
         """Enables DHCP for this network by spawning a local process."""
         try:
-            common_utils.wait_until_true(self._enable, timeout=300)
+            common_utils.wait_until_true(self._enable)
         except common_utils.WaitTimeout:
             LOG.error("Failed to start DHCP process for network %s",
                       self.network.id)
@@ -313,8 +312,7 @@ class Dnsmasq(DhcpLocalProcess):
              [(UDP, DNS_PORT), (TCP, DNS_PORT), (UDP, DHCPV6_PORT)],
              }
 
-    _SUBNET_TAG_PREFIX = 'subnet-%s'
-    _PORT_TAG_PREFIX = 'port-%s'
+    _TAG_PREFIX = 'tag%d'
 
     _ID = 'id:'
 
@@ -362,7 +360,7 @@ class Dnsmasq(DhcpLocalProcess):
             ]
 
         possible_leases = 0
-        for subnet in self._get_all_subnets(self.network):
+        for i, subnet in enumerate(self._get_all_subnets(self.network)):
             mode = None
             # if a subnet is specified to have dhcp disabled
             if not subnet.enable_dhcp:
@@ -390,7 +388,7 @@ class Dnsmasq(DhcpLocalProcess):
             if mode:
                 if subnet.ip_version == 4:
                     cmd.append('--dhcp-range=%s%s,%s,%s,%s,%s' %
-                               ('set:', self._SUBNET_TAG_PREFIX % subnet.id,
+                               ('set:', self._TAG_PREFIX % i,
                                 cidr.network, mode, cidr.netmask, lease))
                 else:
                     if cidr.prefixlen < 64:
@@ -399,7 +397,7 @@ class Dnsmasq(DhcpLocalProcess):
                                   {'subnet': subnet.id, 'cidr': cidr})
                         continue
                     cmd.append('--dhcp-range=%s%s,%s,%s,%d,%s' %
-                               ('set:', self._SUBNET_TAG_PREFIX % subnet.id,
+                               ('set:', self._TAG_PREFIX % i,
                                 cidr.network, mode,
                                 cidr.prefixlen, lease))
                 possible_leases += cidr.size
@@ -468,7 +466,7 @@ class Dnsmasq(DhcpLocalProcess):
         pm = self._get_process_manager(
             cmd_callback=self._build_cmdline_callback)
 
-        pm.enable(reload_cfg=reload_with_HUP, ensure_active=True)
+        pm.enable(reload_cfg=reload_with_HUP)
 
         self.process_monitor.register(uuid=self.network.id,
                                       service_name=DNSMASQ_SERVICE_NAME,
@@ -707,9 +705,8 @@ class Dnsmasq(DhcpLocalProcess):
             port, alloc, hostname, name, no_dhcp, no_opts = host_tuple
             if no_dhcp:
                 if not no_opts and self._get_port_extra_dhcp_opts(port):
-                    buf.write('%s,%s%s\n' % (
-                        port.mac_address,
-                        'set:', self._PORT_TAG_PREFIX % port.id))
+                    buf.write('%s,%s%s\n' %
+                              (port.mac_address, 'set:', port.id))
                 continue
 
             # don't write ip address which belongs to a dhcp disabled subnet.
@@ -723,8 +720,7 @@ class Dnsmasq(DhcpLocalProcess):
                 if client_id and len(port.extra_dhcp_opts) > 1:
                     buf.write('%s,%s%s,%s,%s,%s%s\n' %
                               (port.mac_address, self._ID, client_id, name,
-                               ip_address, 'set:',
-                               self._PORT_TAG_PREFIX % port.id))
+                               ip_address, 'set:', port.id))
                 elif client_id and len(port.extra_dhcp_opts) == 1:
                     buf.write('%s,%s%s,%s,%s\n' %
                               (port.mac_address, self._ID, client_id, name,
@@ -732,7 +728,7 @@ class Dnsmasq(DhcpLocalProcess):
                 else:
                     buf.write('%s,%s,%s,%s%s\n' %
                               (port.mac_address, name, ip_address,
-                               'set:', self._PORT_TAG_PREFIX % port.id))
+                               'set:', port.id))
             else:
                 buf.write('%s,%s,%s\n' %
                           (port.mac_address, name, ip_address))
@@ -951,11 +947,11 @@ class Dnsmasq(DhcpLocalProcess):
 
     def _generate_opts_per_subnet(self):
         options = []
-        subnets_without_nameservers = set()
+        subnet_index_map = {}
         if self.conf.enable_isolated_metadata or self.conf.force_metadata:
             subnet_to_interface_ip = self._make_subnet_interface_ip_map()
         isolated_subnets = self.get_isolated_subnets(self.network)
-        for subnet in self._get_all_subnets(self.network):
+        for i, subnet in enumerate(self._get_all_subnets(self.network)):
             addr_mode = getattr(subnet, 'ipv6_address_mode', None)
             segment_id = getattr(subnet, 'segment_id', None)
             if (not subnet.enable_dhcp or
@@ -970,30 +966,23 @@ class Dnsmasq(DhcpLocalProcess):
                     # Special case: Do not announce DNS servers
                     options.append(
                         self._format_option(
-                            subnet.ip_version,
-                            self._SUBNET_TAG_PREFIX % subnet.id,
-                            'dns-server'))
+                            subnet.ip_version, i, 'dns-server'))
                 else:
                     options.append(
                         self._format_option(
-                            subnet.ip_version,
-                            self._SUBNET_TAG_PREFIX % subnet.id,
-                            'dns-server', ','.join(
+                            subnet.ip_version, i, 'dns-server',
+                            ','.join(
                                 Dnsmasq._convert_to_literal_addrs(
                                     subnet.ip_version,
                                     subnet.dns_nameservers))))
             else:
                 # use the dnsmasq ip as nameservers only if there is no
                 # dns-server submitted by the server
-                # Here is something to check still
-                subnets_without_nameservers.add(subnet.id)
+                subnet_index_map[subnet.id] = i
 
             if self.conf.dns_domain and subnet.ip_version == 6:
-                # This should be change also
-                options.append(
-                    self._format_option(
-                        subnet.ip_version, self._SUBNET_TAG_PREFIX % subnet.id,
-                        "domain-search", ''.join(self.conf.dns_domain)))
+                options.append('tag:tag%s,option6:domain-search,%s' %
+                               (i, ''.join(self.conf.dns_domain)))
 
             gateway = subnet.gateway_ip
             host_routes = []
@@ -1026,36 +1015,31 @@ class Dnsmasq(DhcpLocalProcess):
                     if (s.ip_version == 4 and
                             s.cidr != subnet.cidr and
                             sub_segment_id == segment_id):
-                        host_routes.append("%s,0.0.0.0" % s.cidr)
+                        host_routes.insert(0, "%s,0.0.0.0" % s.cidr)
 
                 if host_routes:
                     if gateway:
                         host_routes.append("%s,%s" % (constants.IPv4_ANY,
                                                       gateway))
                     options.append(
-                        self._format_option(
-                            subnet.ip_version,
-                            self._SUBNET_TAG_PREFIX % subnet.id,
-                            'classless-static-route',
-                            ','.join(host_routes)))
+                        self._format_option(subnet.ip_version, i,
+                                            'classless-static-route',
+                                            ','.join(host_routes)))
                     options.append(
-                        self._format_option(
-                            subnet.ip_version,
-                            self._SUBNET_TAG_PREFIX % subnet.id,
-                            WIN2k3_STATIC_DNS,
-                            ','.join(host_routes)))
+                        self._format_option(subnet.ip_version, i,
+                                            WIN2k3_STATIC_DNS,
+                                            ','.join(host_routes)))
 
                 if gateway:
-                    options.append(self._format_option(
-                        subnet.ip_version, self._SUBNET_TAG_PREFIX % subnet.id,
-                        'router', gateway))
+                    options.append(self._format_option(subnet.ip_version,
+                                                       i, 'router',
+                                                       gateway))
                 else:
-                    options.append(self._format_option(
-                        subnet.ip_version, self._SUBNET_TAG_PREFIX % subnet.id,
-                        'router'))
-        return options, subnets_without_nameservers
+                    options.append(self._format_option(subnet.ip_version,
+                                                       i, 'router'))
+        return options, subnet_index_map
 
-    def _generate_opts_per_port(self, subnets_without_nameservers):
+    def _generate_opts_per_port(self, subnet_index_map):
         options = []
         dhcp_ips = collections.defaultdict(list)
         for port in self.network.ports:
@@ -1071,10 +1055,8 @@ class Dnsmasq(DhcpLocalProcess):
                     opt_ip_version = opt.ip_version
                     if opt_ip_version in port_ip_versions:
                         options.append(
-                            self._format_option(
-                                opt_ip_version,
-                                self._PORT_TAG_PREFIX % port.id,
-                                opt.opt_name, opt.opt_value))
+                            self._format_option(opt_ip_version, port.id,
+                                                opt.opt_name, opt.opt_value))
                     else:
                         LOG.info("Cannot apply dhcp option %(opt)s "
                                  "because it's ip_version %(version)d "
@@ -1087,19 +1069,19 @@ class Dnsmasq(DhcpLocalProcess):
             # by the server
             if port.device_owner == constants.DEVICE_OWNER_DHCP:
                 for ip in port.fixed_ips:
-                    if ip.subnet_id not in subnets_without_nameservers:
+                    i = subnet_index_map.get(ip.subnet_id)
+                    if i is None:
                         continue
-                    dhcp_ips[ip.subnet_id].append(ip.ip_address)
+                    dhcp_ips[i].append(ip.ip_address)
 
-        for subnet_id, ips in dhcp_ips.items():
+        for i, ips in dhcp_ips.items():
             for ip_version in (4, 6):
                 vx_ips = [ip for ip in ips
                           if netaddr.IPAddress(ip).version == ip_version]
                 if len(vx_ips) > 1:
                     options.append(
                         self._format_option(
-                            ip_version, self._SUBNET_TAG_PREFIX % subnet_id,
-                            'dns-server',
+                            ip_version, i, 'dns-server',
                             ','.join(
                                 Dnsmasq._convert_to_literal_addrs(ip_version,
                                                                   vx_ips))))
@@ -1130,6 +1112,9 @@ class Dnsmasq(DhcpLocalProcess):
         extra_tag = matches.groups()[0]
         option = matches.groups()[2]
 
+        if isinstance(tag, int):
+            tag = self._TAG_PREFIX % tag
+
         # NOTE(TheJulia): prepending option6 to any DHCPv6 option is
         # indicated as required in the dnsmasq man page for version 2.79.
         # Testing reveals that the man page is correct, option is not
@@ -1224,7 +1209,9 @@ class DeviceManager(object):
     def __init__(self, conf, plugin):
         self.conf = conf
         self.plugin = plugin
-        self.driver = agent_common_utils.load_interface_driver(conf)
+        self.driver = agent_common_utils.load_interface_driver(
+            conf,
+            get_networks_callback=self.plugin.get_networks)
 
     def get_interface_name(self, network, port):
         """Return interface(device) name for use by the DHCP process."""
@@ -1404,6 +1391,37 @@ class DeviceManager(object):
             fixed_ips=unique_ip_subnets)
         return self.plugin.create_dhcp_port({'port': port_dict})
 
+    def _check_dhcp_port_subnet(self, dhcp_port, dhcp_subnets, network):
+        """Check if DHCP port IPs are in the range of the DHCP subnets
+
+        FIXME(kevinbenton): ensure we have the IPs we actually need.
+        can be removed once bug/1627480 is fixed
+        """
+        if self.driver.use_gateway_ips:
+            return
+
+        expected = set(dhcp_subnets)
+        actual = {fip.subnet_id for fip in dhcp_port.fixed_ips}
+        missing = expected - actual
+        if not missing:
+            return
+
+        LOG.debug('Requested DHCP port with IPs on subnets %(expected)s '
+                  'but only got IPs on subnets %(actual)s.',
+                  {'expected': expected, 'actual': actual})
+        updated_dhcp_port = self.plugin.get_dhcp_port(dhcp_port.id)
+        actual = {fip.subnet_id for fip in updated_dhcp_port.fixed_ips}
+        missing = expected - actual
+        if missing:
+            raise exceptions.SubnetMismatchForPort(
+                port_id=updated_dhcp_port.id, subnet_id=list(missing)[0])
+
+        self._update_dhcp_port(network, updated_dhcp_port)
+        LOG.debug('Previous DHCP port information: %(dhcp_port)s. Updated '
+                  'DHCP port information: %(updated_dhcp_port)s.',
+                  {'dhcp_port': dhcp_port,
+                   'updated_dhcp_port': updated_dhcp_port})
+
     def setup_dhcp_port(self, network):
         """Create/update DHCP port for the host if needed and return port."""
 
@@ -1429,19 +1447,8 @@ class DeviceManager(object):
         else:
             raise exceptions.Conflict()
 
-        # FIXME(kevinbenton): ensure we have the IPs we actually need.
-        # can be removed once bug/1627480 is fixed
-        if not self.driver.use_gateway_ips:
-            expected = set(dhcp_subnets)
-            actual = {fip.subnet_id for fip in dhcp_port.fixed_ips}
-            missing = expected - actual
-            if missing:
-                LOG.debug("Requested DHCP port with IPs on subnets "
-                          "%(expected)s but only got IPs on subnets "
-                          "%(actual)s.", {'expected': expected,
-                                          'actual': actual})
-                raise exceptions.SubnetMismatchForPort(
-                    port_id=dhcp_port.id, subnet_id=list(missing)[0])
+        self._check_dhcp_port_subnet(dhcp_port, dhcp_subnets, network)
+
         # Convert subnet_id to subnet dict
         fixed_ips = [dict(subnet_id=fixed_ip.subnet_id,
                           ip_address=fixed_ip.ip_address,
@@ -1518,7 +1525,7 @@ class DeviceManager(object):
                                                       root_namespace=True)
         if ipv6_utils.is_enabled_and_bind_by_default():
             self.driver.configure_ipv6_ra(network.namespace, 'default',
-                                          n_const.ACCEPT_RA_DISABLED)
+                                          constants.ACCEPT_RA_DISABLED)
 
         if ip_lib.ensure_device_is_ready(interface_name,
                                          namespace=network.namespace):
@@ -1591,6 +1598,7 @@ class DeviceManager(object):
     def fill_dhcp_udp_checksums(self, namespace):
         """Ensure DHCP reply packets always have correct UDP checksums."""
         iptables_mgr = iptables_manager.IptablesManager(use_ipv6=True,
+                                                        nat=False,
                                                         namespace=namespace)
         ipv4_rule = ('-p udp -m udp --dport %d -j CHECKSUM --checksum-fill'
                      % constants.DHCP_RESPONSE_PORT)
