@@ -24,13 +24,8 @@ from oslo_utils import excutils
 import six
 
 from neutron.agent.common import ovs_lib
-from neutron.agent.linux import ethtool
 from neutron.agent.linux import ip_lib
-from neutron.common import constants as n_const
 from neutron.common import utils
-from neutron.conf.plugins.ml2.drivers import ovs_conf
-from neutron.plugins.ml2.drivers.openvswitch.agent.common \
-    import constants as ovs_const
 
 LOG = logging.getLogger(__name__)
 
@@ -43,10 +38,10 @@ def _get_veth(name1, name2, namespace2):
 @six.add_metaclass(abc.ABCMeta)
 class LinuxInterfaceDriver(object):
 
-    DEV_NAME_LEN = n_const.LINUX_DEV_LEN
+    DEV_NAME_LEN = constants.LINUX_DEV_LEN
     DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
 
-    def __init__(self, conf):
+    def __init__(self, conf, **kwargs):
         self.conf = conf
         self._mtu_update_warn_logged = False
 
@@ -260,17 +255,15 @@ class LinuxInterfaceDriver(object):
 
     @abc.abstractmethod
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None,
-                 link_up=True):
+                 bridge=None, namespace=None, prefix=None, mtu=None):
         """Plug in the interface only for new devices that don't exist yet."""
 
     def plug(self, network_id, port_id, device_name, mac_address,
-             bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
+             bridge=None, namespace=None, prefix=None, mtu=None):
         if not ip_lib.device_exists(device_name,
                                     namespace=namespace):
-            self._safe_plug_new(
-                network_id, port_id, device_name, mac_address, bridge,
-                namespace, prefix, mtu, link_up)
+            self.plug_new(network_id, port_id, device_name, mac_address,
+                          bridge, namespace, prefix, mtu)
         else:
             LOG.info("Device %s already exists", device_name)
             if mtu:
@@ -279,22 +272,6 @@ class LinuxInterfaceDriver(object):
             else:
                 LOG.warning("No MTU configured for port %s", port_id)
 
-    def _safe_plug_new(self, network_id, port_id, device_name, mac_address,
-            bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
-        try:
-            self.plug_new(
-                network_id, port_id, device_name, mac_address, bridge,
-                namespace, prefix, mtu, link_up)
-        except TypeError:
-            LOG.warning("Interface driver's plug_new() method should now "
-                        "accept additional optional parameter 'link_up'. "
-                        "Usage of plug_new() method which takes from 5 to 9 "
-                        "positional arguments is now deprecated and will not "
-                        "be possible in W release.")
-            self.plug_new(
-                network_id, port_id, device_name, mac_address, bridge,
-                namespace, prefix, mtu)
-
     @abc.abstractmethod
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
@@ -322,21 +299,10 @@ class LinuxInterfaceDriver(object):
             LOG.warning("Interface driver cannot update MTU for ports")
             self._mtu_update_warn_logged = True
 
-    def set_link_status(self, device_name, namespace=None, link_up=True):
-        ns_dev = ip_lib.IPWrapper(namespace=namespace).device(device_name)
-        if not ns_dev.exists():
-            LOG.debug("Device %s may concurrently be deleted.", device_name)
-            return
-        if link_up:
-            ns_dev.link.set_up()
-        else:
-            ns_dev.link.set_down()
-
 
 class NullDriver(LinuxInterfaceDriver):
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None,
-                 link_up=True):
+                 bridge=None, namespace=None, prefix=None, mtu=None):
         pass
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
@@ -348,9 +314,8 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
 
     DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
 
-    def __init__(self, conf):
-        super(OVSInterfaceDriver, self).__init__(conf)
-        ovs_conf.register_ovs_agent_opts(self.conf)
+    def __init__(self, conf, **kwargs):
+        super(OVSInterfaceDriver, self).__init__(conf, **kwargs)
         if self.conf.ovs_use_veth:
             self.DEV_NAME_PREFIX = 'ns-'
 
@@ -372,8 +337,7 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         ovs.replace_port(device_name, *attrs)
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None,
-                 link_up=True):
+                 bridge=None, namespace=None, prefix=None, mtu=None):
         """Plug in the interface."""
         if not bridge:
             bridge = self.conf.ovs_integration_bridge
@@ -437,14 +401,8 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         else:
             LOG.warning("No MTU configured for port %s", port_id)
 
-        if link_up:
-            ns_dev.link.set_up()
+        ns_dev.link.set_up()
         if self.conf.ovs_use_veth:
-            # ovs-dpdk does not do checksum calculations for veth interface
-            # (bug 1832021)
-            if self.conf.OVS.datapath_type == ovs_const.OVS_DATAPATH_NETDEV:
-                ethtool.Ethtool.offload(ns_dev.name, rx=False, tx=False,
-                                        namespace=namespace)
             root_dev.link.set_up()
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
@@ -483,8 +441,7 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
     DEV_NAME_PREFIX = 'ns-'
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None,
-                 link_up=True):
+                 bridge=None, namespace=None, prefix=None, mtu=None):
         """Plugin the interface."""
         ip = ip_lib.IPWrapper()
 
@@ -503,8 +460,7 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
             LOG.warning("No MTU configured for port %s", port_id)
 
         root_veth.link.set_up()
-        if link_up:
-            ns_veth.link.set_up()
+        ns_veth.link.set_up()
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
