@@ -17,6 +17,7 @@ import itertools
 import os
 
 import netaddr
+from neutron_lib import constants
 from neutron_lib import exceptions
 from neutron_lib.utils import file as file_utils
 from oslo_config import cfg
@@ -25,7 +26,6 @@ from oslo_utils import fileutils
 
 from neutron._i18n import _
 from neutron.agent.linux import external_process
-from neutron.common import constants
 from neutron.common import utils
 
 VALID_STATES = ['MASTER', 'BACKUP']
@@ -85,28 +85,24 @@ class InvalidAuthenticationTypeException(exceptions.NeutronException):
 class KeepalivedVipAddress(object):
     """A virtual address entry of a keepalived configuration."""
 
-    def __init__(self, ip_address, interface_name, scope=None, track=True):
+    def __init__(self, ip_address, interface_name, scope=None):
         self.ip_address = ip_address
         self.interface_name = interface_name
         self.scope = scope
-        self.track = track
 
     def __eq__(self, other):
         return (isinstance(other, KeepalivedVipAddress) and
                 self.ip_address == other.ip_address)
 
     def __str__(self):
-        return '[%s, %s, %s, %s]' % (self.ip_address,
-                                     self.interface_name,
-                                     self.scope,
-                                     self.track)
+        return '[%s, %s, %s]' % (self.ip_address,
+                                 self.interface_name,
+                                 self.scope)
 
     def build_config(self):
         result = '%s dev %s' % (self.ip_address, self.interface_name)
         if self.scope:
             result += ' scope %s' % self.scope
-        if cfg.CONF.keepalived_use_no_track and not self.track:
-            result += ' no_track'
         return result
 
 
@@ -128,8 +124,6 @@ class KeepalivedVirtualRoute(object):
             output += ' dev %s' % self.interface_name
         if self.scope:
             output += ' scope %s' % self.scope
-        if cfg.CONF.keepalived_use_no_track:
-            output += ' no_track'
         return output
 
 
@@ -206,8 +200,7 @@ class KeepalivedInstance(object):
         self.authentication = (auth_type, password)
 
     def add_vip(self, ip_cidr, interface_name, scope):
-        track = interface_name in self.track_interfaces
-        vip = KeepalivedVipAddress(ip_cidr, interface_name, scope, track=track)
+        vip = KeepalivedVipAddress(ip_cidr, interface_name, scope)
         if vip not in self.vips:
             self.vips.append(vip)
         else:
@@ -459,12 +452,6 @@ class KeepalivedManager(object):
         pm = self.get_process()
         pm.disable(sig='15')
 
-    def check_processes(self):
-        keepalived_pm = self.get_process()
-        vrrp_pm = self._get_vrrp_process(
-            self.get_vrrp_pid_file_name(keepalived_pm.get_pid_file_name()))
-        return keepalived_pm.active and vrrp_pm.active
-
     def get_process(self):
         return external_process.ProcessManager(
             cfg.CONF,
