@@ -23,20 +23,16 @@ from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
 from neutron_lib import constants as lib_const
-from neutron_lib.plugins import utils as p_utils
-from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
 
 from neutron._i18n import _
-from neutron.agent.common import ovs_lib
 from neutron.agent import firewall
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import iptables
 from neutron.agent.linux.openvswitch_firewall import rules
-from neutron.common import constants
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
 
@@ -86,26 +82,6 @@ def get_segmentation_id_from_other_config(bridge, port_name):
         pass
 
 
-def get_network_type_from_other_config(bridge, port_name):
-    """Return network_type stored in OVSDB other_config metadata.
-
-    :param bridge: OVSBridge instance where port is.
-    :param port_name: Name of the port.
-    """
-    other_config = bridge.db_get_val('Port', port_name, 'other_config')
-    return other_config.get('network_type')
-
-
-def get_physical_network_from_other_config(bridge, port_name):
-    """Return physical_network stored in OVSDB other_config metadata.
-
-    :param bridge: OVSBridge instance where port is.
-    :param port_name: Name of the port.
-    """
-    other_config = bridge.db_get_val('Port', port_name, 'other_config')
-    return other_config.get('physical_network')
-
-
 def get_tag_from_other_config(bridge, port_name):
     """Return tag stored in OVSDB other_config metadata.
 
@@ -159,8 +135,7 @@ class SecurityGroup(object):
 
 
 class OFPort(object):
-    def __init__(self, port_dict, ovs_port, vlan_tag, segment_id=None,
-                 network_type=None, physical_network=None):
+    def __init__(self, port_dict, ovs_port, vlan_tag, segment_id=None):
         self.id = port_dict['device']
         self.vlan_tag = vlan_tag
         self.segment_id = segment_id
@@ -173,8 +148,6 @@ class OFPort(object):
         self.neutron_port_dict = port_dict.copy()
         self.allowed_pairs_v4 = self._get_allowed_pairs(port_dict, version=4)
         self.allowed_pairs_v6 = self._get_allowed_pairs(port_dict, version=6)
-        self.network_type = network_type
-        self.physical_network = physical_network
 
     @staticmethod
     def _get_allowed_pairs(port_dict, version):
@@ -269,10 +242,6 @@ class ConjIdMap(object):
 
     def __init__(self):
         self.id_map = collections.defaultdict(self._conj_id_factory)
-        # Stores the set of conjuntion IDs used for each unique tuple
-        # (sg_id, remote_sg_id, direction, ethertype). Each tuple can have up
-        # to 8 conjuntion IDs (see ConjIPFlowManager.add()).
-        self.id_map_group = collections.defaultdict(set)
         self.id_free = collections.deque()
         self.max_id = 0
 
@@ -303,22 +272,13 @@ class ConjIdMap(object):
         return a list of (remote_sg_id, conj_id), which are no longer
         in use.
         """
-        result = set([])
+        result = []
         for k in list(self.id_map.keys()):
             if sg_id in k[0:2]:
                 conj_id = self.id_map.pop(k)
-                result.add((k[1], conj_id))
+                result.append((k[1], conj_id))
                 self.id_free.append(conj_id)
 
-        # If the remote_sg_id is removed, the tuple (sg_id, remote_sg_id,
-        # direction, ethertype) no longer exists; the conjunction IDs assigned
-        # to this tuple should be removed too.
-        for k in list(self.id_map_group.keys()):
-            if sg_id in k[0:2]:
-                conj_id_groups = self.id_map_group.pop(k, [])
-                for conj_id in conj_id_groups:
-                    result.add((k[1], conj_id))
-
         return result
 
 
@@ -350,7 +310,7 @@ class ConjIPFlowManager(object):
         addr_to_conj = collections.defaultdict(list)
         for remote_id, conj_id_set in sg_conj_id_map.items():
             remote_group = self.driver.sg_port_map.get_sg(remote_id)
-            if not remote_group or not remote_group.members:
+            if not remote_group:
                 LOG.debug('No member for SG %s', remote_id)
                 continue
             for addr in remote_group.get_ethertype_filtered_addresses(
@@ -360,22 +320,12 @@ class ConjIPFlowManager(object):
         return addr_to_conj
 
     def _update_flows_for_vlan_subr(self, direction, ethertype, vlan_tag,
-                                    flow_state, addr_to_conj,
-                                    conj_id_to_remove):
+                                    flow_state, addr_to_conj):
         """Do the actual flow updates for given direction and ethertype."""
-        conj_id_to_remove = conj_id_to_remove or []
-        # Delete any current flow related to any deleted IP address, before
-        # creating the flows for the current IPs.
-        self.driver.delete_flows_for_flow_state(
-            flow_state, addr_to_conj, direction, ethertype, vlan_tag)
-        for conj_id_set in conj_id_to_remove:
-            # Remove any remaining flow with remote SG ID conj_id_to_remove
-            for current_ip, conj_ids in flow_state.items():
-                conj_ids_to_remove = conj_id_set & set(conj_ids)
-                self.driver.delete_flow_for_ip(
-                    current_ip, direction, ethertype, vlan_tag,
-                    conj_ids_to_remove)
-
+        current_ips = set(flow_state.keys())
+        self.driver.delete_flows_for_ip_addresses(
+            current_ips - set(addr_to_conj.keys()),
+            direction, ethertype, vlan_tag)
         for addr, conj_ids in addr_to_conj.items():
             conj_ids.sort()
             if flow_state.get(addr) == conj_ids:
@@ -384,7 +334,7 @@ class ConjIPFlowManager(object):
                     addr, direction, ethertype, vlan_tag, conj_ids):
                 self.driver._add_flow(**flow)
 
-    def update_flows_for_vlan(self, vlan_tag, conj_id_to_remove=None):
+    def update_flows_for_vlan(self, vlan_tag):
         """Install action=conjunction(conj_id, 1/2) flows,
         which depend on IP addresses of remote_group_id.
         """
@@ -397,7 +347,7 @@ class ConjIPFlowManager(object):
             self._update_flows_for_vlan_subr(
                 direction, ethertype, vlan_tag,
                 self.flow_state[vlan_tag][(direction, ethertype)],
-                addr_to_conj, conj_id_to_remove)
+                addr_to_conj)
             self.flow_state[vlan_tag][(direction, ethertype)] = addr_to_conj
 
     def add(self, vlan_tag, sg_id, remote_sg_id, direction, ethertype,
@@ -418,43 +368,32 @@ class ConjIPFlowManager(object):
                 collections.defaultdict(set))
         self.conj_ids[vlan_tag][(direction, ethertype)][remote_sg_id].add(
             conj_id)
-
-        conj_id_tuple = (sg_id, remote_sg_id, direction, ethertype)
-        self.conj_id_map.id_map_group[conj_id_tuple].add(conj_id)
         return conj_id
 
     def sg_removed(self, sg_id):
         """Handle SG removal events.
 
-        Free all conj_ids associated with the sg_id removed and clean up
+        Free all conj_ids associated with the sg_id and clean up
         obsolete entries from the self.conj_ids map.  Unlike the add
         method, it also updates flows.
-        If a SG is removed, both sg_id and remote_sg_id should be removed from
-        the "vlan_conj_id_map".
         """
-        id_set = self.conj_id_map.delete_sg(sg_id)
+        id_list = self.conj_id_map.delete_sg(sg_id)
         unused_dict = collections.defaultdict(set)
-        for remote_sg_id, conj_id in id_set:
+        for remote_sg_id, conj_id in id_list:
             unused_dict[remote_sg_id].add(conj_id)
 
         for vlan_tag, vlan_conj_id_map in self.conj_ids.items():
             update = False
-            conj_id_to_remove = []
             for sg_conj_id_map in vlan_conj_id_map.values():
                 for remote_sg_id, unused in unused_dict.items():
                     if (remote_sg_id in sg_conj_id_map and
                             sg_conj_id_map[remote_sg_id] & unused):
-                        if remote_sg_id == sg_id:
-                            conj_id_to_remove.append(
-                                sg_conj_id_map[remote_sg_id] & unused)
                         sg_conj_id_map[remote_sg_id] -= unused
                         if not sg_conj_id_map[remote_sg_id]:
                             del sg_conj_id_map[remote_sg_id]
                         update = True
-
             if update:
-                self.update_flows_for_vlan(vlan_tag,
-                                           conj_id_to_remove=conj_id_to_remove)
+                self.update_flows_for_vlan(vlan_tag)
 
 
 class OVSFirewallDriver(firewall.FirewallDriver):
@@ -491,7 +430,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             callbacks_resources.AGENT,
             callbacks_events.OVS_RESTARTED)
 
-    def _init_firewall_callback(self, resource, event, trigger, **kwargs):
+    def _init_firewall_callback(self, resource, event, trigger, payload=None):
         LOG.info("Reinitialize Openvswitch firewall after OVS restart.")
         self._initialize_firewall()
 
@@ -535,8 +474,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def _delete_flows(self, **kwargs):
         create_reg_numbers(kwargs)
-        deferred = kwargs.pop('deferred', self._deferred)
-        if deferred:
+        if self._deferred:
             self.int_br.delete_flows(**kwargs)
         else:
             self.int_br.br.delete_flows(**kwargs)
@@ -600,14 +538,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         return get_segmentation_id_from_other_config(
             self.int_br.br, port_name)
 
-    def _get_port_network_type(self, port_name):
-        return get_network_type_from_other_config(
-            self.int_br.br, port_name)
-
-    def _get_port_physical_network(self, port_name):
-        return get_physical_network_from_other_config(
-            self.int_br.br, port_name)
-
     def get_ofport(self, port):
         port_id = port['device']
         return self.sg_port_map.ports.get(port_id)
@@ -625,13 +555,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             port_vlan_id = self._get_port_vlan_tag(ovs_port.port_name)
             segment_id = self._get_port_segmentation_id(
                 ovs_port.port_name)
-            network_type = self._get_port_network_type(
-                ovs_port.port_name)
-            physical_network = self._get_port_physical_network(
-                ovs_port.port_name)
             of_port = OFPort(port, ovs_port, port_vlan_id,
-                             segment_id,
-                             network_type, physical_network)
+                             segment_id)
             self.sg_port_map.create_port(of_port, port)
         else:
             if of_port.ofport != ovs_port.ofport:
@@ -653,23 +578,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             self._initialize_egress_no_port_security(port['device'])
             return
 
-        try:
-            old_of_port = self.get_ofport(port)
-            of_port = self.get_or_create_ofport(port)
-            if old_of_port:
-                LOG.info("Initializing port %s that was already initialized.",
-                         port['device'])
-                self._update_flows_for_port(of_port, old_of_port)
-            else:
-                self._set_port_filters(of_port)
-        except exceptions.OVSFWPortNotFound as not_found_error:
-            LOG.info("port %(port_id)s does not exist in ovsdb: %(err)s.",
-                     {'port_id': port['device'],
-                      'err': not_found_error})
-        except exceptions.OVSFWTagNotFound as tag_not_found:
-            LOG.info("Tag was not found for port %(port_id)s: %(err)s.",
-                     {'port_id': port['device'],
-                      'err': tag_not_found})
+        old_of_port = self.get_ofport(port)
+        of_port = self.get_or_create_ofport(port)
+        if old_of_port:
+            LOG.info("Initializing port %s that was already initialized.",
+                     port['device'])
+            self._update_flows_for_port(of_port, old_of_port)
+        else:
+            self._set_port_filters(of_port)
 
     def update_port_filter(self, port):
         """Update rules for given port
@@ -705,9 +621,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             LOG.info("port %(port_id)s does not exist in ovsdb: %(err)s.",
                      {'port_id': port['device'],
                       'err': not_found_error})
-            # If port doesn't exist in ovsdb, lets ensure that there are no
-            # leftovers
-            self.remove_port_filter(port)
         except exceptions.OVSFWTagNotFound as tag_not_found:
             LOG.info("Tag was not found for port %(port_id)s: %(err)s.",
                      {'port_id': port['device'],
@@ -885,7 +798,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 priority=95,
                 in_port=port.ofport,
                 reg_port=port.ofport,
-                dl_type=constants.ETHERTYPE_IPV6,
+                dl_type=lib_const.ETHERTYPE_IPV6,
                 nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
                 icmp_type=icmp_type,
                 actions='resubmit(,%d)' % (
@@ -910,7 +823,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                       {'port_id': port_id,
                        'err': not_found_e})
             return
-        self.sg_port_map.unfiltered[port_id] = (ovs_port, vlan_tag)
+        self.sg_port_map.unfiltered[port_id] = ovs_port.ofport
         self._add_flow(
             table=ovs_consts.TRANSIENT_TABLE,
             priority=100,
@@ -932,33 +845,20 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
         )
 
-        tunnel_direct_info = {
-            "network_type": self._get_port_network_type(ovs_port.port_name),
-            "physical_network": self._get_port_physical_network(
-                ovs_port.port_name)
-        }
-        self.install_accepted_egress_direct_flow(
-            ovs_port.vif_mac, vlan_tag, ovs_port.ofport,
-            tunnel_direct_info=tunnel_direct_info)
-
     def _remove_egress_no_port_security(self, port_id):
         try:
-            ovs_port, vlan_tag = self.sg_port_map.unfiltered[port_id]
+            ofport = self.sg_port_map.unfiltered[port_id]
         except KeyError:
             raise exceptions.OVSFWPortNotHandled(port_id=port_id)
 
         self._delete_flows(
             table=ovs_consts.TRANSIENT_TABLE,
-            in_port=ovs_port.ofport
+            in_port=ofport
         )
         self._delete_flows(
             table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
-            reg_port=ovs_port.ofport
+            reg_port=ofport
         )
-
-        self.delete_accepted_egress_direct_flow(
-            ovs_port.vif_mac, vlan_tag)
-
         del self.sg_port_map.unfiltered[port_id]
 
     def _initialize_egress(self, port):
@@ -975,7 +875,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 in_port=port.ofport,
                 reg_port=port.ofport,
                 dl_src=mac_addr,
-                dl_type=constants.ETHERTYPE_ARP,
+                dl_type=lib_const.ETHERTYPE_ARP,
                 arp_spa=ip_addr,
                 actions='resubmit(,%d)' % (
                     ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
@@ -984,7 +884,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=65,
                 reg_port=port.ofport,
-                dl_type=constants.ETHERTYPE_IP,
+                dl_type=lib_const.ETHERTYPE_IP,
                 in_port=port.ofport,
                 dl_src=mac_addr,
                 nw_src=ip_addr,
@@ -1002,7 +902,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 priority=65,
                 reg_port=port.ofport,
                 in_port=port.ofport,
-                dl_type=constants.ETHERTYPE_IPV6,
+                dl_type=lib_const.ETHERTYPE_IPV6,
                 dl_src=mac_addr,
                 ipv6_src=ip_addr,
                 actions='ct(table={:d},zone=NXM_NX_REG{:d}[0..15])'.format(
@@ -1012,8 +912,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
         # DHCP discovery
         for dl_type, src_port, dst_port in (
-                (constants.ETHERTYPE_IP, 68, 67),
-                (constants.ETHERTYPE_IPV6, 546, 547)):
+                (lib_const.ETHERTYPE_IP, 68, 67),
+                (lib_const.ETHERTYPE_IPV6, 546, 547)):
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=80,
@@ -1028,8 +928,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
         # Ban dhcp service running on an instance
         for dl_type, src_port, dst_port in (
-                (constants.ETHERTYPE_IP, 67, 68),
-                (constants.ETHERTYPE_IPV6, 547, 546)):
+                (lib_const.ETHERTYPE_IP, 67, 68),
+                (lib_const.ETHERTYPE_IPV6, 547, 546)):
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=70,
@@ -1048,7 +948,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             priority=70,
             in_port=port.ofport,
             reg_port=port.ofport,
-            dl_type=constants.ETHERTYPE_IPV6,
+            dl_type=lib_const.ETHERTYPE_IPV6,
             nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
             icmp_type=lib_const.ICMPV6_TYPE_RA,
             actions='resubmit(,%d)' % ovs_consts.DROPPED_TRAFFIC_TABLE
@@ -1098,7 +998,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     ovsfw_consts.REG_PORT,
                     ovs_consts.BASE_INGRESS_TABLE),
             )
-        for ethertype in [constants.ETHERTYPE_IP, constants.ETHERTYPE_IPV6]:
+        for ethertype in [lib_const.ETHERTYPE_IP, lib_const.ETHERTYPE_IPV6]:
             self._add_flow(
                 table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                 priority=90,
@@ -1118,72 +1018,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
         )
 
-        tunnel_direct_info = {"network_type": port.network_type,
-                              "physical_network": port.physical_network}
-        self.install_accepted_egress_direct_flow(
-            port.mac, port.vlan_tag, port.ofport,
-            tunnel_direct_info=tunnel_direct_info)
-
-    def install_accepted_egress_direct_flow(self, mac, vlan_tag, dst_port,
-                                            tunnel_direct_info=None):
-        if not cfg.CONF.AGENT.explicitly_egress_direct:
-            return
-
-        # Prevent flood for accepted egress traffic
-        self._add_flow(
-            table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
-            priority=12,
-            dl_dst=mac,
-            reg_net=vlan_tag,
-            actions='output:{:d}'.format(dst_port)
-        )
-
-        # The former flow may not match, that means the destination port is
-        # not in this host. So, we direct the packet to mapped bridge(s).
-        if tunnel_direct_info:
-            patch_ofport = ovs_lib.INVALID_OFPORT
-            if tunnel_direct_info["network_type"] in (
-                    lib_const.TYPE_VXLAN, lib_const.TYPE_GRE,
-                    lib_const.TYPE_GENEVE):
-                # Some ports like router internal gateway will not install
-                # the l2pop related flows, so we will transmit the ARP request
-                # packet to tunnel bridge use NORMAL action as usual.
-                port_name = cfg.CONF.OVS.int_peer_patch_port
-                patch_ofport = self.int_br.br.get_port_ofport(port_name)
-            elif tunnel_direct_info["network_type"] == lib_const.TYPE_VLAN:
-                physical_network = tunnel_direct_info["physical_network"]
-                if not physical_network:
-                    return
-                bridge_mappings = helpers.parse_mappings(
-                    cfg.CONF.OVS.bridge_mappings)
-                bridge = bridge_mappings.get(physical_network)
-                port_name = p_utils.get_interface_name(
-                    bridge, prefix=ovs_consts.PEER_INTEGRATION_PREFIX)
-                patch_ofport = self.int_br.br.get_port_ofport(port_name)
-
-            if patch_ofport is not ovs_lib.INVALID_OFPORT:
-                self._add_flow(
-                    table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
-                    priority=10,
-                    dl_src=mac,
-                    dl_dst="00:00:00:00:00:00/01:00:00:00:00:00",
-                    reg_net=vlan_tag,
-                    actions='mod_vlan_vid:{:d},output:{:d}'.format(
-                        vlan_tag,
-                        patch_ofport)
-                )
-
-    def delete_accepted_egress_direct_flow(self, mac, vlan_tag):
-        self._delete_flows(
-            table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
-            dl_dst=mac,
-            reg_net=vlan_tag)
-
-        self._delete_flows(
-            table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
-            dl_src=mac,
-            reg_net=vlan_tag)
-
     def _initialize_tracked_egress(self, port):
         # Drop invalid packets
         self._add_flow(
@@ -1222,7 +1056,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             ct_state=ovsfw_consts.OF_STATE_NOT_ESTABLISHED,
             actions='resubmit(,%d)' % ovs_consts.DROPPED_TRAFFIC_TABLE
         )
-        for ethertype in [constants.ETHERTYPE_IP, constants.ETHERTYPE_IPV6]:
+        for ethertype in [lib_const.ETHERTYPE_IP, lib_const.ETHERTYPE_IPV6]:
             self._add_flow(
                 table=ovs_consts.RULES_EGRESS_TABLE,
                 priority=40,
@@ -1241,7 +1075,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 table=ovs_consts.BASE_INGRESS_TABLE,
                 priority=100,
                 reg_port=port.ofport,
-                dl_type=constants.ETHERTYPE_IPV6,
+                dl_type=lib_const.ETHERTYPE_IPV6,
                 nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
                 icmp_type=icmp_type,
                 actions='output:{:d}'.format(port.ofport)
@@ -1252,7 +1086,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         self._add_flow(
             table=ovs_consts.BASE_INGRESS_TABLE,
             priority=100,
-            dl_type=constants.ETHERTYPE_ARP,
+            dl_type=lib_const.ETHERTYPE_ARP,
             reg_port=port.ofport,
             actions='output:{:d}'.format(port.ofport)
         )
@@ -1261,8 +1095,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
         # DHCP offers
         for dl_type, src_port, dst_port in (
-                (constants.ETHERTYPE_IP, 67, 68),
-                (constants.ETHERTYPE_IPV6, 547, 546)):
+                (lib_const.ETHERTYPE_IP, 67, 68),
+                (lib_const.ETHERTYPE_IPV6, 547, 546)):
             self._add_flow(
                 table=ovs_consts.BASE_INGRESS_TABLE,
                 priority=95,
@@ -1275,7 +1109,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # Track untracked
-        for dl_type in (constants.ETHERTYPE_IP, constants.ETHERTYPE_IPV6):
+        for dl_type in (lib_const.ETHERTYPE_IP, lib_const.ETHERTYPE_IPV6):
             self._add_flow(
                 table=ovs_consts.BASE_INGRESS_TABLE,
                 priority=90,
@@ -1311,18 +1145,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             actions='resubmit(,%d)' % ovs_consts.DROPPED_TRAFFIC_TABLE
         )
 
-        # NOTE: The OUTPUT action is used instead of NORMAL action to reduce
-        # cpu utilization, but it causes the datapath rule to be flood rule.
-        # This is due to mac learning not happened on ingress traffic.
-        # While this is ok for no offload case, in ovs offload flood rule
-        # is not offloaded. Therefore, we change the action to be NORMAL in
-        # offload case. In case the explicitly_egress_direct is used the
-        # pipeline don't contain action NORMAL so we don't have flood rule
-        # issue.
-        actions = 'output:{:d}'.format(port.ofport)
-        if (self.int_br.br.is_hw_offload_enabled and
-                not cfg.CONF.AGENT.explicitly_egress_direct):
-            actions = 'mod_vlan_vid:{:d},normal'.format(port.vlan_tag)
         # Allow established and related connections
         for state in (ovsfw_consts.OF_STATE_ESTABLISHED_REPLY,
                       ovsfw_consts.OF_STATE_RELATED):
@@ -1333,7 +1155,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 ct_state=state,
                 ct_mark=ovsfw_consts.CT_MARK_NORMAL,
                 ct_zone=port.vlan_tag,
-                actions=actions
+                actions='output:{:d}'.format(port.ofport)
             )
         self._add_flow(
             table=ovs_consts.RULES_INGRESS_TABLE,
@@ -1342,7 +1164,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             ct_state=ovsfw_consts.OF_STATE_NOT_ESTABLISHED,
             actions='resubmit(,%d)' % ovs_consts.DROPPED_TRAFFIC_TABLE
         )
-        for ethertype in [constants.ETHERTYPE_IP, constants.ETHERTYPE_IPV6]:
+        for ethertype in [lib_const.ETHERTYPE_IP, lib_const.ETHERTYPE_IPV6]:
             self._add_flow(
                 table=ovs_consts.RULES_INGRESS_TABLE,
                 priority=40,
@@ -1374,13 +1196,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                                                rule['remote_group_id'],
                                                direction, ethertype,
                                                priority_offset)
-            LOG.debug("Created conjunction %(conj_id)s for SG %(sg_id)s "
-                      "referencing remote SG ID %(remote_sg_id)s on port "
-                      "%(port_id)s.",
-                      {'conj_id': conj_id,
-                       'sg_id': sec_group_id,
-                       'remote_sg_id': rule['remote_group_id'],
-                       'port_id': port.id})
 
             rule1 = rule.copy()
             del rule1['remote_group_id']
@@ -1457,39 +1272,22 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             self.delete_vlan_direct_flow(mac_addr, port.segment_id)
             self._delete_flows(table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                                dl_dst=mac_addr, reg_net=port.vlan_tag)
-
-        self.delete_accepted_egress_direct_flow(
-            port.mac, port.vlan_tag)
         self._strict_delete_flow(priority=100,
                                  table=ovs_consts.TRANSIENT_TABLE,
                                  in_port=port.ofport)
         self._delete_flows(reg_port=port.ofport)
 
-    def delete_flows_for_flow_state(
-            self, flow_state, addr_to_conj, direction, ethertype, vlan_tag):
-        # Remove rules for deleted IPs and action=conjunction(conj_id, 1/2)
-        removed_ips = set(flow_state.keys()) - set(addr_to_conj.keys())
-        for removed_ip in removed_ips:
-            conj_ids = flow_state[removed_ip]
-            self.delete_flow_for_ip(removed_ip, direction, ethertype, vlan_tag,
-                                    conj_ids)
-
-        if not cfg.CONF.AGENT.explicitly_egress_direct:
-            return
-
-        for ip_addr in removed_ips:
+    def delete_flows_for_ip_addresses(
+            self, ip_addresses, direction, ethertype, vlan_tag):
+        for ip_addr in ip_addresses:
             # Generate deletion template with bogus conj_id.
-            self.delete_flow_for_ip(ip_addr, direction, ethertype, vlan_tag,
-                                    [0])
-
-    def delete_flow_for_ip(self, ip_address, direction, ethertype,
-                           vlan_tag, conj_ids):
-        for flow in rules.create_flows_for_ip_address(
-                ip_address, direction, ethertype, vlan_tag, conj_ids):
-            # The following del statements are partly for
-            # complying the OpenFlow spec. It forbids the use of
-            # these field in non-strict delete flow messages, and
-            # the actions field is bogus anyway.
-            del flow['actions']
-            del flow['priority']
-            self._delete_flows(deferred=False, **flow)
+            flows = rules.create_flows_for_ip_address(
+                ip_addr, direction, ethertype, vlan_tag, [0])
+            for f in flows:
+                # The following del statements are partly for
+                # complying the OpenFlow spec. It forbids the use of
+                # these field in non-strict delete flow messages, and
+                # the actions field is bogus anyway.
+                del f['actions']
+                del f['priority']
+                self._delete_flows(**f)
