@@ -21,6 +21,7 @@ import pwd
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
+from neutron_lib import constants
 from neutron_lib import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -29,7 +30,6 @@ from neutron._i18n import _
 from neutron.agent.l3 import ha_router
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import external_process
-from neutron.common import constants
 
 
 LOG = logging.getLogger(__name__)
@@ -65,8 +65,7 @@ defaults
 listen listener
     bind %(host)s:%(port)s
     server metadata %(unix_socket_path)s
-    http-request del-header X-Neutron-%(res_type_del)s-ID
-    http-request set-header X-Neutron-%(res_type)s-ID %(res_id)s
+    http-request add-header X-Neutron-%(res_type)s-ID %(res_id)s
 """
 
 
@@ -129,16 +128,12 @@ class HaproxyConfigurator(object):
             'log_level': self.log_level,
             'log_tag': self.log_tag
         }
-        # If using the network ID, delete any spurious router ID that might
-        # have been in the request, same for network ID when using router ID.
         if self.network_id:
             cfg_info['res_type'] = 'Network'
             cfg_info['res_id'] = self.network_id
-            cfg_info['res_type_del'] = 'Router'
         else:
             cfg_info['res_type'] = 'Router'
             cfg_info['res_id'] = self.router_id
-            cfg_info['res_type_del'] = 'Network'
 
         haproxy_cfg = _HAPROXY_CONFIG_TEMPLATE % cfg_info
         LOG.debug("haproxy_cfg = %s", haproxy_cfg)
