@@ -31,7 +31,7 @@ from oslo_utils import uuidutils
 
 from neutron.agent import resource_cache
 from neutron.api.rpc.callbacks import resources
-from neutron.common import constants as n_const
+from neutron.common import _constants as n_const
 from neutron import objects
 
 LOG = logging.getLogger(__name__)
@@ -78,7 +78,7 @@ class PluginReportStateAPI(object):
     """
     def __init__(self, topic):
         target = oslo_messaging.Target(topic=topic, version='1.2',
-                                       namespace=n_const.RPC_NAMESPACE_STATE)
+                                       namespace=constants.RPC_NAMESPACE_STATE)
         self.client = lib_rpc.get_client(target)
 
     def has_alive_neutron_server(self, context, **kwargs):
@@ -112,6 +112,8 @@ class PluginApi(object):
         1.4 - tunnel_sync rpc signature upgrade to obtain 'host'
         1.5 - Support update_device_list and
               get_devices_details_list_and_failed_devices
+        1.6 - Support get_network_details
+        1.7 - Support get_ports_by_vnic_type_and_host
     '''
 
     def __init__(self, topic):
@@ -143,6 +145,11 @@ class PluginApi(object):
             'get_devices_details_list_and_failed_devices',
             devices=devices, agent_id=agent_id, host=host)
 
+    def get_network_details(self, context, network, agent_id, host=None):
+        cctxt = self.client.prepare(version='1.6')
+        return cctxt.call(context, 'get_network_details', network=network,
+                          agent_id=agent_id, host=host)
+
     def update_device_down(self, context, device, agent_id, host=None):
         cctxt = self.client.prepare()
         return cctxt.call(context, 'update_device_down', device=device,
@@ -187,28 +194,24 @@ class PluginApi(object):
         return cctxt.call(context, 'tunnel_sync', tunnel_ip=tunnel_ip,
                           tunnel_type=tunnel_type, host=host)
 
-
-def create_cache_for_l2_agent():
-    """Create a push-notifications cache for L2 agent related resources."""
-
-    objects.register_objects()
-    resource_types = [
-        resources.PORT,
-        resources.SECURITYGROUP,
-        resources.SECURITYGROUPRULE,
-        resources.NETWORK,
-        resources.SUBNET
-    ]
-    rcache = resource_cache.RemoteResourceCache(resource_types)
-    rcache.start_watcher()
-    return rcache
+    def get_ports_by_vnic_type_and_host(self, context, vnic_type, host):
+        cctxt = self.client.prepare(version='1.7')
+        return cctxt.call(context, 'get_ports_by_vnic_type_and_host',
+                          vnic_type=vnic_type, host=host)
 
 
 class CacheBackedPluginApi(PluginApi):
 
+    RESOURCE_TYPES = [resources.PORT,
+                      resources.SECURITYGROUP,
+                      resources.SECURITYGROUPRULE,
+                      resources.NETWORK,
+                      resources.SUBNET]
+
     def __init__(self, *args, **kwargs):
         super(CacheBackedPluginApi, self).__init__(*args, **kwargs)
-        self.remote_resource_cache = create_cache_for_l2_agent()
+        self.remote_resource_cache = None
+        self._create_cache_for_l2_agent()
 
     def register_legacy_notification_callbacks(self, legacy_interface):
         """Emulates the server-side notifications from ml2 AgentNotifierApi.
@@ -329,7 +332,7 @@ class CacheBackedPluginApi(PluginApi):
             LOG.debug("Device %s has no active binding in this host",
                       port_obj)
             return {'device': device,
-                    n_const.NO_ACTIVE_BINDING: True}
+                    constants.NO_ACTIVE_BINDING: True}
         net = self.remote_resource_cache.get_resource_by_id(
             resources.NETWORK, port_obj.network_id)
         net_qos_policy_id = net.qos_policy_id
@@ -338,6 +341,7 @@ class CacheBackedPluginApi(PluginApi):
                                    dialect=netaddr.mac_unix_expanded))
         entry = {
             'device': device,
+            'device_id': port_obj.device_id,
             'network_id': port_obj.network_id,
             'port_id': port_obj.id,
             'mac_address': mac_addr,
@@ -357,6 +361,8 @@ class CacheBackedPluginApi(PluginApi):
             'qos_policy_id': port_obj.qos_policy_id,
             'network_qos_policy_id': net_qos_policy_id,
             'profile': binding.profile,
+            'vif_type': binding.vif_type,
+            'vnic_type': binding.vnic_type,
             'security_groups': list(port_obj.security_group_ids)
         }
         LOG.debug("Returning: %s", entry)
@@ -365,3 +371,10 @@ class CacheBackedPluginApi(PluginApi):
     def get_devices_details_list(self, context, devices, agent_id, host=None):
         return [self.get_device_details(context, device, agent_id, host)
                 for device in devices]
+
+    def _create_cache_for_l2_agent(self):
+        """Create a push-notifications cache for L2 agent related resources."""
+        objects.register_objects()
+        rcache = resource_cache.RemoteResourceCache(self.RESOURCE_TYPES)
+        rcache.start_watcher()
+        self.remote_resource_cache = rcache
