@@ -34,8 +34,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from sqlalchemy.orm import exc
 
-from neutron.common import constants as n_const
-from neutron.db import common_db_mixin
 from neutron.db import models_v2
 from neutron.objects import base as base_obj
 from neutron.objects import ports as port_obj
@@ -79,7 +77,22 @@ def filter_fields(f):
     return inner_filter
 
 
-class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
+def make_result_with_fields(f):
+    @functools.wraps(f)
+    def inner(*args, **kwargs):
+        fields = kwargs.get('fields')
+        result = f(*args, **kwargs)
+        if fields is None:
+            return result
+        elif isinstance(result, list):
+            return [db_utils.resource_fields(r, fields) for r in result]
+        else:
+            return db_utils.resource_fields(result, fields)
+
+    return inner
+
+
+class DbBasePluginCommon(object):
     """Stores getters and helper methods for db_base_plugin_v2
 
     All private getters and simple helpers like _make_*_dict were moved from
@@ -307,7 +320,7 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                'name': network['name'],
                'tenant_id': network['tenant_id'],
                'admin_state_up': network['admin_state_up'],
-               'mtu': network.get('mtu', n_const.DEFAULT_NETWORK_MTU),
+               'mtu': network.get('mtu', constants.DEFAULT_NETWORK_MTU),
                'status': network['status'],
                'subnets': [subnet['id']
                            for subnet in network['subnets']]}
