@@ -23,7 +23,6 @@ from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
-from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from sqlalchemy import or_
 
@@ -86,7 +85,8 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
     """
 
     def dvr_handle_new_service_port(self, context, port,
-                                    dest_host=None, unbound_migrate=False):
+                                    dest_host=None, unbound_migrate=False,
+                                    router_id=None):
         """Handle new dvr service port creation.
 
         When a new dvr service port is created, this function will
@@ -103,15 +103,20 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         if not l3_agent_on_host:
             return
 
-        if dest_host:
+        if dest_host and router_id is not None:
             # Make sure we create the floatingip agent gateway port
             # for the destination node if fip is associated with this
             # fixed port
             l3plugin = directory.get_plugin(plugin_constants.L3)
-            (
-                l3plugin.
-                check_for_fip_and_create_agent_gw_port_on_host_if_not_exists(
-                    context, port, dest_host))
+            router = l3plugin._get_router(context, router_id)
+            if l3_dvr_db.is_distributed_router(router):
+                (l3plugin.
+                 check_for_fip_and_create_agent_gw_port_on_host_if_not_exists(
+                     context, port, dest_host))
+            else:
+                LOG.debug("Port-in-Migration: Floating IP has a non-"
+                          "distributed router %(router_id)s",
+                          {'router_id': router_id})
 
         subnet_ids = [ip['subnet_id'] for ip in port['fixed_ips']]
         router_ids = self.get_dvr_routers_by_subnet_ids(context, subnet_ids)
@@ -168,18 +173,12 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             router_ids.add(subnet_port['device_id'])
         return router_ids
 
-    def get_subnet_ids_on_router(self, context, router_id,
-                                 keep_gateway_port=True):
+    def get_subnet_ids_on_router(self, context, router_id):
         """Return subnet IDs for interfaces attached to the given router."""
         subnet_ids = set()
         filter_rtr = {'device_id': [router_id]}
         int_ports = self._core_plugin.get_ports(context, filters=filter_rtr)
-
         for int_port in int_ports:
-            if (not keep_gateway_port and
-                    int_port['device_owner'] ==
-                    n_const.DEVICE_OWNER_ROUTER_GW):
-                continue
             int_ips = int_port['fixed_ips']
             if int_ips:
                 int_subnet = int_ips[0]['subnet_id']
@@ -411,9 +410,6 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             context, agent_db, router_ids, with_dvr))
         if not with_dvr:
             return result_set
-        LOG.debug("Routers %(router_ids)s bound to L3 agent in host %(host)s",
-                  {'router_ids': result_set,
-                   'host': agent_db['host']})
         router_ids = set(router_ids or [])
         if router_ids and result_set == router_ids:
             # no need for extra dvr checks if requested routers are
@@ -433,32 +429,20 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             else:
                 for router_id in (router_ids - result_set):
                     subnet_ids = self.get_subnet_ids_on_router(
-                        context, router_id, keep_gateway_port=False)
-                    if (subnet_ids and (
+                        context, router_id)
+                    if (subnet_ids and
                             self._check_dvr_serviceable_ports_on_host(
                                     context, agent_db['host'],
-                                    list(subnet_ids)) or
-                            self._is_router_related_to_dvr_routers(
-                                    context, router_id, dvr_routers))):
+                                    list(subnet_ids))):
                         result_set.add(router_id)
 
-            LOG.debug("Routers %(router_ids)s are scheduled or have "
-                      "serviceable ports in host %(host)s",
-                      {'router_ids': result_set,
-                       'host': agent_db['host']})
-            related_routers = set()
-            for router_id in result_set:
-                related_routers |= set(
+            for dvr_router in dvr_routers:
+                result_set |= set(
                     self._get_other_dvr_router_ids_connected_router(
-                        context, router_id))
-            result_set |= related_routers
+                        context, dvr_router))
 
-        LOG.debug("Router IDs %(router_ids)s for agent in host %(host)s",
-                  {'router_ids': result_set,
-                   'host': agent_db['host']})
         return list(result_set)
 
-    @log_helpers.log_method_call
     def _check_dvr_serviceable_ports_on_host(self, context, host, subnet_ids):
         """Check for existence of dvr serviceable ports on host
 
@@ -504,13 +488,6 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         query = query.filter(host_filter)
         return query.first() is not None
 
-    @log_helpers.log_method_call
-    def _is_router_related_to_dvr_routers(self, context, router_id,
-                                          dvr_routers):
-        related_routers = self._get_other_dvr_router_ids_connected_router(
-            context, router_id)
-        return any([r in dvr_routers for r in related_routers])
-
 
 def _dvr_handle_unbound_allowed_addr_pair_add(
         plugin, context, port, allowed_address_pair):
@@ -576,10 +553,25 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
     if new_port and original_port:
         l3plugin = directory.get_plugin(plugin_constants.L3)
         context = kwargs['context']
+        is_new_port_binding_changed = (
+            new_port[portbindings.HOST_ID] and
+            new_port[portbindings.HOST_ID] !=
+            original_port[portbindings.HOST_ID])
         is_bound_port_moved = (
             original_port[portbindings.HOST_ID] and
             original_port[portbindings.HOST_ID] !=
             new_port[portbindings.HOST_ID])
+        fip_router_id = None
+        dest_host = None
+        new_port_profile = new_port.get(portbindings.PROFILE)
+        if new_port_profile:
+            dest_host = new_port_profile.get('migrating_to')
+        if is_new_port_binding_changed or is_bound_port_moved or dest_host:
+            fips = l3plugin._get_floatingips_by_port_id(
+                    context, port_id=original_port['id'])
+            fip = fips[0] if fips else None
+            if fip:
+                fip_router_id = fip['router_id']
         if is_bound_port_moved:
             removed_routers = l3plugin.get_dvr_routers_to_remove(
                 context,
@@ -594,38 +586,23 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
                 }
                 _notify_port_delete(
                     event, resource, trigger, **removed_router_args)
-            fips = l3plugin._get_floatingips_by_port_id(
-                context, port_id=original_port['id'])
-            fip = fips[0] if fips else None
 
             def _should_notify_on_fip_update():
-                if not fip:
+                if not fip_router_id:
                     return False
                 for info in removed_routers:
-                    if info['router_id'] == fip['router_id']:
+                    if info['router_id'] == fip_router_id:
                         return False
                 try:
-                    router = l3plugin._get_router(context, fip['router_id'])
+                    router = l3plugin._get_router(context, fip_router_id)
                 except l3_exc.RouterNotFound:
                     return False
                 return l3_dvr_db.is_distributed_router(router)
 
             if _should_notify_on_fip_update():
                 l3plugin.l3_rpc_notifier.routers_updated_on_host(
-                    context, [fip['router_id']],
+                    context, [fip_router_id],
                     original_port[portbindings.HOST_ID])
-        is_new_port_binding_changed = (
-            new_port[portbindings.HOST_ID] and
-            (original_port[portbindings.HOST_ID] !=
-                new_port[portbindings.HOST_ID]))
-        dest_host = None
-        new_port_profile = new_port.get(portbindings.PROFILE)
-        if new_port_profile:
-            dest_host = new_port_profile.get('migrating_to')
-            # This check is required to prevent an arp update
-            # of the allowed_address_pair port.
-            if new_port_profile.get('original_owner'):
-                return
         # If dest_host is set, then the port profile has changed
         # and this port is in migration. The call below will
         # pre-create the router on the new host
@@ -637,8 +614,10 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
                 l3plugin.dvr_handle_new_service_port(context, new_port,
                                                      unbound_migrate=True)
             else:
-                l3plugin.dvr_handle_new_service_port(context, new_port,
-                                                     dest_host=dest_host)
+                l3plugin.dvr_handle_new_service_port(
+                    context, new_port,
+                    dest_host=dest_host,
+                    router_id=fip_router_id)
             l3plugin.update_arp_entry_for_dvr_service_port(
                 context, new_port)
             return
