@@ -26,27 +26,24 @@ from neutron_lib.db import model_query
 from neutron_lib.db import resource_extend
 from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as n_exc
-from neutron_lib.objects import exceptions as obj_exc
 from neutron_lib.utils import helpers
 from neutron_lib.utils import net
+from oslo_log import log as logging
 from oslo_utils import uuidutils
 import six
 from sqlalchemy.orm import scoped_session
 
 from neutron._i18n import _
 from neutron.common import _constants as const
-from neutron.common import constants as n_const
-from neutron.common import utils
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import rbac_db_mixin as rbac_mixin
 from neutron.extensions import securitygroup as ext_sg
 from neutron.objects import base as base_obj
 from neutron.objects import ports as port_obj
 from neutron.objects import securitygroup as sg_obj
-from neutron import quota
 
 
-DEFAULT_SG_DESCRIPTION = _('Default security group')
+LOG = logging.getLogger(__name__)
 
 
 @resource_extend.has_resource_extenders
@@ -114,12 +111,6 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 name=s['name'], is_default=default_sg)
             sg.create()
 
-            delta = len(ext_sg.sg_supported_ethertypes)
-            delta = delta * 2 if default_sg else delta
-            reservation = quota.QUOTAS.make_reservation(
-                context, tenant_id, {'security_group_rule': delta},
-                self)
-
             for ethertype in ext_sg.sg_supported_ethertypes:
                 if default_sg:
                     # Allow intercommunication
@@ -139,9 +130,6 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 sg.rules.append(egress_rule)
             sg.obj_reset_changes(['rules'])
 
-            quota.QUOTAS.commit_reservation(context,
-                                            reservation.reservation_id)
-
             # fetch sg from db to load the sg rules with sg model.
             sg = sg_obj.SecurityGroup.get_object(context, id=sg.id)
             secgroup_dict = self._make_security_group_dict(sg)
@@ -398,7 +386,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         rule_dict = security_group_rule['security_group_rule']
         remote_ip_prefix = rule_dict.get('remote_ip_prefix')
         if remote_ip_prefix:
-            remote_ip_prefix = utils.AuthenticIPNetwork(remote_ip_prefix)
+            remote_ip_prefix = net.AuthenticIPNetwork(remote_ip_prefix)
 
         protocol = rule_dict.get('protocol')
         if protocol:
@@ -458,18 +446,25 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         # problems with comparing int and string in PostgreSQL. Here this
         # string is converted to int to give an opportunity to use it as
         # before.
-        if protocol in n_const.IP_PROTOCOL_NAME_ALIASES:
-            protocol = n_const.IP_PROTOCOL_NAME_ALIASES[protocol]
+        if protocol in constants.IP_PROTOCOL_NAME_ALIASES:
+            protocol = constants.IP_PROTOCOL_NAME_ALIASES[protocol]
         return int(constants.IP_PROTOCOL_MAP.get(protocol, protocol))
 
-    def _get_ip_proto_name_and_num(self, protocol):
+    def _get_ip_proto_name_and_num(self, protocol, ethertype=None):
         if protocol is None:
             return
         protocol = str(protocol)
+        # Force all legacy IPv6 ICMP protocol names to be 'ipv6-icmp', and
+        # protocol number 1 to be 58
+        if ethertype == constants.IPv6:
+            if protocol in const.IPV6_ICMP_LEGACY_PROTO_LIST:
+                protocol = constants.PROTO_NAME_IPV6_ICMP
+            elif protocol == str(constants.PROTO_NUM_ICMP):
+                protocol = str(constants.PROTO_NUM_IPV6_ICMP)
         if protocol in constants.IP_PROTOCOL_MAP:
             return [protocol, str(constants.IP_PROTOCOL_MAP.get(protocol))]
-        elif protocol in n_const.IP_PROTOCOL_NUM_TO_NAME_MAP:
-            return [n_const.IP_PROTOCOL_NUM_TO_NAME_MAP.get(protocol),
+        elif protocol in constants.IP_PROTOCOL_NUM_TO_NAME_MAP:
+            return [constants.IP_PROTOCOL_NUM_TO_NAME_MAP.get(protocol),
                     protocol]
         return [protocol, protocol]
 
@@ -557,8 +552,30 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 raise ext_sg.SecurityGroupRulesNotSingleTenant()
         return sg_groups.pop()
 
+    def _make_canonical_ipv6_icmp_protocol(self, rule):
+        if rule.get('ethertype') == constants.IPv6:
+            if rule.get('protocol') in const.IPV6_ICMP_LEGACY_PROTO_LIST:
+                LOG.info('Project %(project)s added a security group rule '
+                         'with legacy IPv6 ICMP protocol name %(protocol)s, '
+                         '%(new_protocol)s should be used instead. It was '
+                         'automatically converted.',
+                         {'project': rule['tenant_id'],
+                          'protocol': rule['protocol'],
+                          'new_protocol': constants.PROTO_NAME_IPV6_ICMP})
+                rule['protocol'] = constants.PROTO_NAME_IPV6_ICMP
+            elif rule.get('protocol') == str(constants.PROTO_NUM_ICMP):
+                LOG.info('Project %(project)s added a security group rule '
+                         'with legacy IPv6 ICMP protocol number %(protocol)s, '
+                         '%(new_protocol)s should be used instead. It was '
+                         'automatically converted.',
+                         {'project': rule['tenant_id'],
+                          'protocol': rule['protocol'],
+                          'new_protocol': str(constants.PROTO_NUM_IPV6_ICMP)})
+                rule['protocol'] = str(constants.PROTO_NUM_IPV6_ICMP)
+
     def _validate_security_group_rule(self, context, security_group_rule):
         rule = security_group_rule['security_group_rule']
+        self._make_canonical_ipv6_icmp_protocol(rule)
         self._validate_port_range(rule)
         self._validate_ip_prefix(rule)
         self._validate_ethertype_and_protocol(rule)
@@ -602,24 +619,6 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                                     security_group_rule)
         return db_utils.resource_fields(res, fields)
 
-    def _make_security_group_rule_filter_dict(self, security_group_rule):
-        sgr = security_group_rule['security_group_rule']
-        res = {'tenant_id': [sgr['tenant_id']],
-               'security_group_id': [sgr['security_group_id']],
-               'direction': [sgr['direction']]}
-
-        include_if_present = ['protocol', 'port_range_max', 'port_range_min',
-                              'ethertype', 'remote_group_id']
-        for key in include_if_present:
-            value = sgr.get(key)
-            if value:
-                res[key] = [value]
-        # protocol field will get corresponding name and number
-        value = sgr.get('protocol')
-        if value:
-            res['protocol'] = self._get_ip_proto_name_and_num(value)
-        return res
-
     def _rule_to_key(self, rule):
         def _normalize_rule_value(key, value):
             # This string is used as a placeholder for str(None), but shorter.
@@ -632,7 +631,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             elif value is None:
                 return none_char
             elif key == 'protocol':
-                return str(self._get_ip_proto_name_and_num(value))
+                return str(self._get_ip_proto_name_and_num(
+                               value, ethertype=rule.get('ethertype')))
             return str(value)
 
         comparison_keys = [
@@ -695,26 +695,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         pager = base_obj.Pager(
             sorts=sorts, marker=marker, limit=limit, page_reverse=page_reverse)
 
-        project_id = filters.get('project_id') or filters.get('tenant_id')
-        if project_id:
-            project_id = project_id[0]
-        else:
-            project_id = context.project_id
-        if project_id:
-            self._ensure_default_security_group(context, project_id)
-
-        if not filters and context.project_id and not context.is_admin:
-            rule_ids = sg_obj.SecurityGroupRule.get_security_group_rule_ids(
-                context.project_id)
-            filters = {'id': rule_ids}
-
-        # NOTE(slaweq): use admin context here to be able to get all rules
-        # which fits filters' criteria. Later in policy engine rules will be
-        # filtered and only those which are allowed according to policy will
-        # be returned
         rule_objs = sg_obj.SecurityGroupRule.get_objects(
-            context_lib.get_admin_context(), _pager=pager,
-            validate_filters=False, **filters
+            context, _pager=pager, validate_filters=False, **filters
         )
         return [
             self._make_security_group_rule_dict(obj.db_obj, fields)
@@ -729,12 +711,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
     @db_api.retry_if_session_inactive()
     def get_security_group_rule(self, context, id, fields=None):
-        # NOTE(slaweq): use admin context here to be able to get all rules
-        # which fits filters' criteria. Later in policy engine rules will be
-        # filtered and only those which are allowed according to policy will
-        # be returned
-        security_group_rule = self._get_security_group_rule(
-            context_lib.get_admin_context(), id)
+        security_group_rule = self._get_security_group_rule(context, id)
         return self._make_security_group_rule_dict(
             security_group_rule.db_obj, fields)
 
@@ -825,13 +802,10 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             'security_group':
                 {'name': 'default',
                  'tenant_id': tenant_id,
-                 'description': DEFAULT_SG_DESCRIPTION}
+                 'description': _('Default security group')}
         }
-        try:
-            return self.create_security_group(context, security_group,
-                                              default_sg=True)['id']
-        except obj_exc.NeutronDbObjectDuplicateEntry:
-            return self._get_default_sg_id(context, tenant_id)
+        return self.create_security_group(context, security_group,
+                                          default_sg=True)['id']
 
     def _get_security_groups_on_port(self, context, port):
         """Check that all security groups on port belong to tenant.
@@ -851,8 +825,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
         valid_groups = set(
             g.id for g in sg_objs
-            if (context.is_admin or not tenant_id or
-                g.tenant_id == tenant_id or
+            if (not tenant_id or g.tenant_id == tenant_id or
                 sg_obj.SecurityGroup.is_shared_with_tenant(
                     context, g.id, tenant_id))
         )
