@@ -13,11 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from debtcollector import removals
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib.db import api as db_api
 from neutron_lib.plugins import directory
 from oslo_db import exception as db_exc
 from oslo_log import log
@@ -27,9 +29,9 @@ from sqlalchemy import or_
 from sqlalchemy.orm import exc
 
 from neutron._i18n import _
-from neutron.db import api as db_api
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import models_v2
+from neutron.objects import base as objects_base
 from neutron.objects import ports as port_obj
 from neutron.plugins.ml2 import models
 from neutron.services.segments import exceptions as seg_exc
@@ -40,7 +42,7 @@ LOG = log.getLogger(__name__)
 MAX_PORTS_PER_QUERY = 500
 
 
-@db_api.context_manager.writer
+@db_api.CONTEXT_WRITER
 def add_port_binding(context, port_id):
     record = models.PortBinding(
         port_id=port_id,
@@ -49,11 +51,11 @@ def add_port_binding(context, port_id):
     return record
 
 
-@db_api.context_manager.writer
+@db_api.CONTEXT_WRITER
 def set_binding_levels(context, levels):
     if levels:
         for level in levels:
-            level.persist_state_to_session(context.session)
+            level.create()
         LOG.debug("For port %(port_id)s, host %(host)s, "
                   "set binding levels %(levels)s",
                   {'port_id': levels[0].port_id,
@@ -63,7 +65,11 @@ def set_binding_levels(context, levels):
         LOG.debug("Attempted to set empty binding levels")
 
 
-@db_api.context_manager.reader
+@removals.remove(
+    version="Stein", removal_version="T",
+    message="Function get_binding_levels is deprecated. Please use "
+            "get_binding_level_objs instead as it makes use of OVOs.")
+@db_api.CONTEXT_READER
 def get_binding_levels(context, port_id, host):
     if host:
         result = (context.session.query(models.PortBindingLevel).
@@ -78,12 +84,25 @@ def get_binding_levels(context, port_id, host):
         return result
 
 
-@db_api.context_manager.writer
+@db_api.CONTEXT_READER
+def get_binding_level_objs(context, port_id, host):
+    if host:
+        pager = objects_base.Pager(sorts=[('level', True)])
+        port_bl_objs = port_obj.PortBindingLevel.get_objects(
+            context, _pager=pager, port_id=port_id, host=host)
+        LOG.debug("For port %(port_id)s, host %(host)s, "
+                  "got binding levels %(levels)s",
+                  {'port_id': port_id,
+                   'host': host,
+                   'levels': port_bl_objs})
+        return port_bl_objs
+
+
+@db_api.CONTEXT_WRITER
 def clear_binding_levels(context, port_id, host):
     if host:
-        for l in (context.session.query(models.PortBindingLevel).
-                  filter_by(port_id=port_id, host=host)):
-            context.session.delete(l)
+        port_obj.PortBindingLevel.delete_objects(
+            context, port_id=port_id, host=host)
         LOG.debug("For port %(port_id)s, host %(host)s, "
                   "cleared binding levels",
                   {'port_id': port_id,
@@ -91,14 +110,14 @@ def clear_binding_levels(context, port_id, host):
 
 
 def ensure_distributed_port_binding(context, port_id, host, router_id=None):
-    with db_api.context_manager.reader.using(context):
+    with db_api.CONTEXT_READER.using(context):
         record = (context.session.query(models.DistributedPortBinding).
                   filter_by(port_id=port_id, host=host).first())
     if record:
         return record
 
     try:
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             record = models.DistributedPortBinding(
                 port_id=port_id,
                 host=host,
@@ -110,14 +129,14 @@ def ensure_distributed_port_binding(context, port_id, host, router_id=None):
             return record
     except db_exc.DBDuplicateEntry:
         LOG.debug("Distributed Port %s already bound", port_id)
-        with db_api.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             return (context.session.query(models.DistributedPortBinding).
                     filter_by(port_id=port_id, host=host).one())
 
 
 def delete_distributed_port_binding_if_stale(context, binding):
     if not binding.router_id and binding.status == n_const.PORT_STATUS_DOWN:
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             LOG.debug("Distributed port: Deleting binding %s", binding)
             context.session.delete(binding)
 
@@ -125,7 +144,7 @@ def delete_distributed_port_binding_if_stale(context, binding):
 def get_port(context, port_id):
     """Get port record for update within transaction."""
 
-    with db_api.context_manager.reader.using(context):
+    with db_api.CONTEXT_READER.using(context):
         try:
             # Set enable_eagerloads to True, so that lazy load can be
             # proceed later.
@@ -142,7 +161,7 @@ def get_port(context, port_id):
             return
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_port_from_device_mac(context, device_mac):
     LOG.debug("get_port_from_device_mac() called for mac %s", device_mac)
     ports = port_obj.Port.get_objects(context, mac_address=device_mac)
@@ -174,7 +193,7 @@ def get_sg_ids_grouped_by_port(context, port_ids):
     sg_ids_grouped_by_port = {}
     sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
 
-    with db_api.context_manager.reader.using(context):
+    with db_api.CONTEXT_READER.using(context):
         # partial UUIDs must be individually matched with startswith.
         # full UUIDs may be matched directly in an IN statement
         partial_uuids = set(port_id for port_id in port_ids
@@ -213,8 +232,8 @@ def make_port_dict_with_security_groups(port, sec_groups):
 
 def get_port_binding_host(context, port_id):
     try:
-        with db_api.context_manager.reader.using(context):
-            query = (context.session.query(models.PortBinding).
+        with db_api.CONTEXT_READER.using(context):
+            query = (context.session.query(models.PortBinding.host).
                      filter(models.PortBinding.port_id.startswith(port_id)))
             query = query.filter(
                 models.PortBinding.status == n_const.ACTIVE).one()
@@ -229,11 +248,11 @@ def get_port_binding_host(context, port_id):
     return query.host
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def generate_distributed_port_status(context, port_id):
     # an OR'ed value of status assigned to parent port from the
     # distributedportbinding bucket
-    query = context.session.query(models.DistributedPortBinding)
+    query = context.session.query(models.DistributedPortBinding.status)
     final_status = n_const.PORT_STATUS_BUILD
     for bind in query.filter(models.DistributedPortBinding.port_id == port_id):
         if bind.status == n_const.PORT_STATUS_ACTIVE:
@@ -244,8 +263,9 @@ def generate_distributed_port_status(context, port_id):
 
 
 def get_distributed_port_binding_by_host(context, port_id, host):
-    with db_api.context_manager.reader.using(context):
-        binding = (context.session.query(models.DistributedPortBinding).
+    with db_api.CONTEXT_READER.using(context):
+        binding = (
+            context.session.query(models.DistributedPortBinding).
             filter(models.DistributedPortBinding.port_id.startswith(port_id),
                    models.DistributedPortBinding.host == host).first())
     if not binding:
@@ -255,7 +275,7 @@ def get_distributed_port_binding_by_host(context, port_id, host):
 
 
 def get_distributed_port_bindings(context, port_id):
-    with db_api.context_manager.reader.using(context):
+    with db_api.CONTEXT_READER.using(context):
         bindings = (context.session.query(models.DistributedPortBinding).
                     filter(models.DistributedPortBinding.port_id.startswith(
                            port_id)).all())
@@ -264,7 +284,7 @@ def get_distributed_port_bindings(context, port_id):
     return bindings
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def partial_port_ids_to_full_ids(context, partial_ids):
     """Takes a list of the start of port IDs and returns full IDs.
 
@@ -289,7 +309,7 @@ def partial_port_ids_to_full_ids(context, partial_ids):
     return result
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_port_db_objects(context, port_ids):
     """Takes a list of port_ids and returns matching port db objects.
 
@@ -304,38 +324,33 @@ def get_port_db_objects(context, port_ids):
     return result
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def is_dhcp_active_on_any_subnet(context, subnet_ids):
     if not subnet_ids:
         return False
-    return bool(context.session.query(models_v2.Subnet).
+    return bool(context.session.query(models_v2.Subnet.id).
                 enable_eagerloads(False).filter_by(enable_dhcp=True).
                 filter(models_v2.Subnet.id.in_(subnet_ids)).count())
 
 
 def _prevent_segment_delete_with_port_bound(resource, event, trigger,
-                                            context, segment,
-                                            for_net_delete=False):
+                                            payload=None):
     """Raise exception if there are any ports bound with segment_id."""
-    if for_net_delete:
+    if payload.metadata.get('for_net_delete'):
         # don't check for network deletes
         return
 
-    with db_api.context_manager.reader.using(context):
-        segment_id = segment['id']
-        query = context.session.query(models_v2.Port)
-        query = query.join(
-            models.PortBindingLevel,
-            models.PortBindingLevel.port_id == models_v2.Port.id)
-        query = query.filter(models.PortBindingLevel.segment_id == segment_id)
-        port_ids = [p.id for p in query]
+    with db_api.CONTEXT_READER.using(payload.context):
+        port_ids = port_obj.Port.get_port_ids_filter_by_segment_id(
+            payload.context, segment_id=payload.resource_id)
 
     # There are still some ports in the segment, segment should not be deleted
     # TODO(xiaohhui): Should we delete the dhcp port automatically here?
     if port_ids:
         reason = _("The segment is still bound with port(s) "
                    "%s") % ", ".join(port_ids)
-        raise seg_exc.SegmentInUse(segment_id=segment_id, reason=reason)
+        raise seg_exc.SegmentInUse(segment_id=payload.resource_id,
+                                   reason=reason)
 
 
 def subscribe():
