@@ -24,6 +24,7 @@ from neutron_lib.agent import topics
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_constants
 from neutron_lib import context
+from neutron_lib.placement import utils as place_utils
 from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -40,6 +41,7 @@ from neutron.api.rpc.callbacks import resources
 from neutron.api.rpc.handlers import securitygroups_rpc as sg_rpc
 from neutron.common import config as common_config
 from neutron.common import profiler as setup_profiler
+from neutron.common import utils as n_utils
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common import config
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common \
     import exceptions as exc
@@ -58,8 +60,9 @@ class SriovNicSwitchRpcCallbacks(sg_rpc.SecurityGroupAgentRpcCallbackMixin):
     #   1.3 Added param devices_to_update to security_groups_provider_updated
     #       (works with NoopFirewallDriver)
     #   1.4 Added support for network_update
+    #   1.5 Added support for binding_activate and binding_deactivate
 
-    target = oslo_messaging.Target(version='1.4')
+    target = oslo_messaging.Target(version='1.5')
 
     def __init__(self, context, agent, sg_agent):
         super(SriovNicSwitchRpcCallbacks, self).__init__()
@@ -105,11 +108,32 @@ class SriovNicSwitchRpcCallbacks(sg_rpc.SecurityGroupAgentRpcCallbackMixin):
         for port_data in self.agent.network_ports[network_id]:
             self.agent.updated_devices.add(port_data['device'])
 
+    def binding_activate(self, context, **kwargs):
+        if kwargs.get('host') != self.agent.conf.host:
+            return
+        LOG.debug("binding activate for port %s", kwargs.get('port_id'))
+        device_details = self.agent.get_device_details_from_port_id(
+            kwargs.get('port_id'))
+        mac = device_details.get('mac_address')
+        binding_profile = device_details.get('profile')
+        if binding_profile:
+            pci_slot = binding_profile.get('pci_slot')
+            self.agent.activated_bindings.add((mac, pci_slot))
+        else:
+            LOG.warning("binding_profile not found for port %s.",
+                        kwargs.get('port_id'))
+
+    def binding_deactivate(self, context, **kwargs):
+        if kwargs.get('host') != self.agent.conf.host:
+            return
+        LOG.debug("binding deactivate for port %s. NOOP.",
+                  kwargs.get('port_id'))
+
 
 @profiler.trace_cls("rpc")
 class SriovNicSwitchAgent(object):
     def __init__(self, physical_devices_mappings, exclude_devices,
-                 polling_interval):
+                 polling_interval, rp_bandwidths, rp_inventory_defaults):
 
         self.polling_interval = polling_interval
         self.network_ports = collections.defaultdict(list)
@@ -121,17 +145,23 @@ class SriovNicSwitchAgent(object):
 
         # Stores port update notifications for processing in the main loop
         self.updated_devices = set()
+        # Stores <mac, pci_slot> pairs for ports whose binding has been
+        # activated.
+        self.activated_bindings = set()
 
         self.context = context.get_admin_context_without_session()
         self.plugin_rpc = agent_rpc.PluginApi(topics.PLUGIN)
         self.sg_plugin_rpc = sg_rpc.SecurityGroupServerRpcApi(topics.PLUGIN)
-        self.sg_agent = agent_sg_rpc.SecurityGroupAgentRpc(self.context,
-                self.sg_plugin_rpc)
+        self.sg_agent = agent_sg_rpc.SecurityGroupAgentRpc(
+            self.context, self.sg_plugin_rpc)
         self._setup_rpc()
         self.ext_manager = self._create_agent_extension_manager(
             self.connection)
 
         configurations = {'device_mappings': physical_devices_mappings,
+                          n_constants.RP_BANDWIDTHS: rp_bandwidths,
+                          n_constants.RP_INVENTORY_DEFAULTS:
+                              rp_inventory_defaults,
                           'extensions': self.ext_manager.names()}
 
         # TODO(mangelajo): optimize resource_versions (see ovs agent)
@@ -152,8 +182,8 @@ class SriovNicSwitchAgent(object):
     def _setup_rpc(self):
         self.agent_id = 'nic-switch-agent.%s' % socket.gethostname()
         LOG.info("RPC agent_id: %s", self.agent_id)
-
         self.topic = topics.AGENT
+        self.failed_report_state = False
         self.state_rpc = agent_rpc.PluginReportStateAPI(topics.REPORTS)
         # RPC network init
         # Handle updates from service
@@ -162,7 +192,9 @@ class SriovNicSwitchAgent(object):
         # Define the listening consumers for the agent
         consumers = [[topics.PORT, topics.UPDATE],
                      [topics.NETWORK, topics.UPDATE],
-                     [topics.SECURITY_GROUP, topics.UPDATE]]
+                     [topics.SECURITY_GROUP, topics.UPDATE],
+                     [topics.PORT_BINDING, topics.DEACTIVATE],
+                     [topics.PORT_BINDING, topics.ACTIVATE]]
         self.connection = agent_rpc.create_consumers(self.endpoints,
                                                      self.topic,
                                                      consumers,
@@ -183,7 +215,12 @@ class SriovNicSwitchAgent(object):
             self.agent_state.pop('resource_versions', None)
             self.agent_state.pop('start_flag', None)
         except Exception:
+            self.failed_report_state = True
             LOG.exception("Failed reporting state!")
+            return
+        if self.failed_report_state:
+            self.failed_report_state = False
+            LOG.info("Successfully reported state after a previous failure.")
 
     def _create_agent_extension_manager(self, connection):
         ext_manager.register_opts(self.conf)
@@ -235,7 +272,8 @@ class SriovNicSwitchAgent(object):
         # If one of the above operations fails => resync with plugin
         return (resync_a | resync_b)
 
-    def treat_device(self, device, pci_slot, admin_state_up, spoofcheck=True):
+    def treat_device(self, device, pci_slot, admin_state_up, spoofcheck=True,
+                     propagate_uplink_state=False):
         if self.eswitch_mgr.device_exists(device, pci_slot):
             try:
                 self.eswitch_mgr.set_device_spoofcheck(device, pci_slot,
@@ -248,7 +286,8 @@ class SriovNicSwitchAgent(object):
 
             try:
                 self.eswitch_mgr.set_device_state(device, pci_slot,
-                                                  admin_state_up)
+                                                  admin_state_up,
+                                                  propagate_uplink_state)
             except exc.IpCommandOperationNotSupportedError:
                 LOG.warning("Device %s does not support state change",
                             device)
@@ -300,10 +339,12 @@ class SriovNicSwitchAgent(object):
                 port_id = device_details['port_id']
                 profile = device_details['profile']
                 spoofcheck = device_details.get('port_security_enabled', True)
-                if self.treat_device(device,
-                                     profile.get('pci_slot'),
-                                     device_details['admin_state_up'],
-                                     spoofcheck):
+                if self.treat_device(
+                        device,
+                        profile.get('pci_slot'),
+                        device_details['admin_state_up'],
+                        spoofcheck,
+                        device_details['propagate_uplink_status']):
                     if device_details['admin_state_up']:
                         devices_up.add(device)
                     else:
@@ -314,6 +355,12 @@ class SriovNicSwitchAgent(object):
                                            port_id,
                                            (device, profile.get('pci_slot')))
                 self.ext_manager.handle_port(self.context, device_details)
+            elif n_constants.NO_ACTIVE_BINDING in device_details:
+                # Port was added but its binding in this agent
+                # hasn't been activated yet. It will be treated as
+                # added when binding is activated
+                LOG.info("Device with MAC %s has no active binding in host",
+                         device)
             else:
                 LOG.info("Device with MAC %s not defined on plugin",
                          device)
@@ -362,6 +409,36 @@ class SriovNicSwitchAgent(object):
                           {'mac': mac, 'pci_slot': pci_slot})
         return resync
 
+    def process_activated_bindings(self, device_info, activated_bindings_copy):
+        """Process activated bindings.
+
+        Add activated bindings to the 'added' set in device info.
+
+        :param device_info: A dict that contains the set of 'current', 'added',
+                            'removed' and 'updated' ports.
+        :param activated_bindings_copy: A set of activated port bindings.
+        :return: None
+        """
+        LOG.debug("Processing activated bindings: %s", activated_bindings_copy)
+        # Compute which ports for activated bindings are already present
+        activated_bindings_copy &= device_info['current']
+        # Treat them as just added
+        device_info['added'] |= activated_bindings_copy
+
+    def get_device_details_from_port_id(self, port_id):
+        """Get device details from server
+
+        :param port_id: Port identifier (UUID).
+        :return: A dict containing various port attributes if the port is
+                 bound to the host. In case the port is not bound to the host
+                 then the method will return A dict with a minimal set of
+                 attributes e.g  {'device': port_id}.
+        """
+        return self.plugin_rpc.get_device_details(self.context,
+                                                  port_id,
+                                                  self.agent_id,
+                                                  host=cfg.CONF.host)
+
     def daemon_loop(self):
         sync = True
         devices = set()
@@ -383,10 +460,15 @@ class SriovNicSwitchAgent(object):
             # two statements, this will should be thread-safe.
             updated_devices_copy = self.updated_devices
             self.updated_devices = set()
+            activated_bindings_copy = self.activated_bindings
+            self.activated_bindings = set()
             try:
                 self.eswitch_mgr.discover_devices(self.device_mappings,
                                                   self.exclude_devices)
                 device_info = self.scan_devices(devices, updated_devices_copy)
+                if activated_bindings_copy:
+                    self.process_activated_bindings(device_info,
+                                                    activated_bindings_copy)
                 if self._device_info_has_changes(device_info):
                     LOG.debug("Agent loop found changes! %s", device_info)
                     # If treat devices fails - indicates must resync with
@@ -400,6 +482,7 @@ class SriovNicSwitchAgent(object):
                 # Restore devices that were removed from this set earlier
                 # without overwriting ones that may have arrived since.
                 self.updated_devices |= updated_devices_copy
+                self.activated_bindings |= activated_bindings_copy
 
             # sleep till end of polling interval
             elapsed = (time.time() - start)
@@ -427,21 +510,29 @@ class SriovNicAgentConfigParser(object):
             cfg.CONF.SRIOV_NIC.physical_device_mappings, unique_keys=False)
         self.exclude_devices = config.parse_exclude_devices(
             cfg.CONF.SRIOV_NIC.exclude_devices)
+        self.rp_bandwidths = place_utils.parse_rp_bandwidths(
+            cfg.CONF.SRIOV_NIC.resource_provider_bandwidths)
+        self.rp_inventory_defaults = place_utils.parse_rp_inventory_defaults(
+            cfg.CONF.SRIOV_NIC.resource_provider_inventory_defaults)
         self._validate()
 
     def _validate(self):
         """Validate configuration.
 
         Validate that network_device in excluded_device
-        exists in device mappings
+        exists in device mappings.
+        Validate that network_device in resource_provider_bandwidths
+        exists in device mappings.
         """
         dev_net_set = set(itertools.chain.from_iterable(
                           six.itervalues(self.device_mappings)))
         for dev_name in self.exclude_devices.keys():
             if dev_name not in dev_net_set:
-                raise ValueError(_("Device name %(dev_name)s is missing from "
-                                   "physical_device_mappings") % {'dev_name':
-                                                                  dev_name})
+                raise ValueError(_(
+                    "Invalid exclude_devices: "
+                    "Device name %(dev_name)s is missing from "
+                    "physical_device_mappings") % {'dev_name': dev_name})
+        n_utils.validate_rp_bandwidth(self.rp_bandwidths, dev_net_set)
 
 
 def main():
@@ -453,6 +544,8 @@ def main():
         config_parser.parse()
         device_mappings = config_parser.device_mappings
         exclude_devices = config_parser.exclude_devices
+        rp_bandwidths = config_parser.rp_bandwidths
+        rp_inventory_defaults = config_parser.rp_inventory_defaults
 
     except ValueError:
         LOG.exception("Failed on Agent configuration parse. "
@@ -465,7 +558,9 @@ def main():
     try:
         agent = SriovNicSwitchAgent(device_mappings,
                                     exclude_devices,
-                                    polling_interval)
+                                    polling_interval,
+                                    rp_bandwidths,
+                                    rp_inventory_defaults)
     except exc.SriovNicError:
         LOG.exception("Agent Initialization Failed")
         raise SystemExit(1)
