@@ -56,8 +56,6 @@ MAC_SPOOF_TABLE = 25
 
 # Table to decide whether further filtering is needed
 TRANSIENT_TABLE = 60
-LOCAL_MAC_DIRECT = 61
-TRANSIENT_EGRESS_TABLE = 62
 
 # Tables used for ovs firewall
 BASE_EGRESS_TABLE = 71
@@ -87,9 +85,7 @@ INT_BR_ALL_TABLES = (
     CANARY_TABLE,
     ARP_SPOOF_TABLE,
     MAC_SPOOF_TABLE,
-    LOCAL_MAC_DIRECT,
     TRANSIENT_TABLE,
-    TRANSIENT_EGRESS_TABLE,
     BASE_EGRESS_TABLE,
     RULES_EGRESS_TABLE,
     ACCEPT_OR_INGRESS_TABLE,
