@@ -17,6 +17,7 @@
 import functools
 import random
 
+import debtcollector
 import eventlet
 import netaddr
 from neutron_lib import exceptions
@@ -70,13 +71,14 @@ class OpenFlowSwitchMixin(object):
             dp = ofctl_api.get_datapath(self._app, dpid_int)
             if dp is not None:
                 break
-            # The switch has not established a connection to us; retry again
-            # until timeout.
+            # The switch has not established a connection to us.
+            # Wait for a little.
             if timeutils.now() > start_time + timeout_sec:
                 m = _("Switch connection timeout")
                 LOG.error(m)
                 # NOTE(yamamoto): use RuntimeError for compat with ovs_lib
                 raise RuntimeError(m)
+            eventlet.sleep(1)
         return dp
 
     def _send_msg(self, msg, reply_cls=None, reply_multi=False,
@@ -219,9 +221,8 @@ class OpenFlowSwitchMixin(object):
         (dp, ofp, ofpp) = self._get_dp()
         match = self._match(ofp, ofpp, match, **match_kwargs)
         if isinstance(instructions, six.string_types):
-            # NOTE: instructions must be str for the ofctl of_interface.
-            # After the ofctl driver is removed, a deprecation warning
-            # could be added here.
+            debtcollector.deprecate("Use of string instruction is "
+                "deprecated", removal_version='U')
             jsonlist = ofctl_string.ofp_instruction_from_str(
                 ofp, instructions)
             instructions = ofproto_parser.ofp_instruction_from_jsondict(
