@@ -17,13 +17,11 @@ import sys
 
 import netaddr
 from neutron_lib import constants as n_const
-from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_utils import excutils
 from osprofiler import profiler
 
-from neutron.agent.linux.openvswitch_firewall import firewall as ovs_firewall
 from neutron.common import utils as n_utils
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 
@@ -129,34 +127,28 @@ class OVSDVRNeutronAgent(object):
         self.enable_tunneling = enable_tunneling
         self.enable_distributed_routing = enable_distributed_routing
         self.bridge_mappings = bridge_mappings
+        self.phys_brs = phys_brs
         self.int_ofports = int_ofports
         self.phys_ofports = phys_ofports
-        self.reset_ovs_parameters(integ_br, tun_br, phys_brs,
+        self.reset_ovs_parameters(integ_br, tun_br,
                                   patch_int_ofport, patch_tun_ofport)
         self.reset_dvr_parameters()
         self.dvr_mac_address = None
         self.arp_responder_enabled = arp_responder_enabled
         if self.enable_distributed_routing:
             self.get_dvr_mac_address()
-        self.conf = cfg.CONF
-        self.firewall = None
 
-    def set_firewall(self, firewall=None):
-        self.firewall = firewall
-
-    def setup_dvr_flows(self, bridge_mappings=None):
-        bridge_mappings = bridge_mappings or self.bridge_mappings
+    def setup_dvr_flows(self):
         self.setup_dvr_flows_on_integ_br()
         self.setup_dvr_flows_on_tun_br()
-        self.setup_dvr_flows_on_phys_br(bridge_mappings)
+        self.setup_dvr_flows_on_phys_br()
         self.setup_dvr_mac_flows_on_all_brs()
 
-    def reset_ovs_parameters(self, integ_br, tun_br, phys_brs,
+    def reset_ovs_parameters(self, integ_br, tun_br,
                              patch_int_ofport, patch_tun_ofport):
         '''Reset the openvswitch parameters'''
         self.int_br = integ_br
         self.tun_br = tun_br
-        self.phys_brs = phys_brs
         self.patch_int_ofport = patch_int_ofport
         self.patch_tun_ofport = patch_tun_ofport
 
@@ -167,15 +159,6 @@ class OVSDVRNeutronAgent(object):
         self.local_ports = {}
         self.registered_dvr_macs = set()
 
-    def reset_dvr_flows(self, integ_br, tun_br, phys_brs,
-                        patch_int_ofport, patch_tun_ofport,
-                        bridge_mappings=None):
-        '''Reset the openvswitch and DVR parameters and DVR flows'''
-        self.reset_ovs_parameters(
-            integ_br, tun_br, phys_brs, patch_int_ofport, patch_tun_ofport)
-        self.reset_dvr_parameters()
-        self.setup_dvr_flows(bridge_mappings)
-
     def get_dvr_mac_address(self):
         try:
             self.get_dvr_mac_address_with_retry()
@@ -251,10 +234,10 @@ class OVSDVRNeutronAgent(object):
         self.tun_br.install_goto(table_id=constants.DVR_PROCESS,
                                  dest_table_id=constants.PATCH_LV_TO_TUN)
 
-    def setup_dvr_flows_on_phys_br(self, bridge_mappings=None):
+    def setup_dvr_flows_on_phys_br(self):
         '''Setup up initial dvr flows into br-phys'''
-        bridge_mappings = bridge_mappings or self.bridge_mappings
-        for physical_network in bridge_mappings:
+
+        for physical_network in self.bridge_mappings:
             self.phys_brs[physical_network].install_goto(
                 in_port=self.phys_ofports[physical_network],
                 priority=2,
@@ -409,15 +392,6 @@ class OVSDVRNeutronAgent(object):
 
         subnet_info = ldm.get_subnet_info()
         ip_version = subnet_info['ip_version']
-
-        if self.firewall and isinstance(self.firewall,
-                                        ovs_firewall.OVSFirewallDriver):
-            tunnel_direct_info = {"network_type": lvm.network_type,
-                                  "physical_network": lvm.physical_network}
-            self.firewall.install_accepted_egress_direct_flow(
-                subnet_info['gateway_mac'], lvm.vlan, port.ofport,
-                tunnel_direct_info=tunnel_direct_info)
-
         local_compute_ports = (
             self.plugin_rpc.get_ports_on_host_by_subnet(
                 self.context, self.host, subnet_uuid))
@@ -683,11 +657,6 @@ class OVSDVRNeutronAgent(object):
                     vlan_tag=lvm.vlan, gateway_mac=subnet_info['gateway_mac'])
             ovsport.remove_subnet(sub_uuid)
 
-            if self.firewall and isinstance(self.firewall,
-                                            ovs_firewall.OVSFirewallDriver):
-                self.firewall.delete_accepted_egress_direct_flow(
-                    subnet_info['gateway_mac'], lvm.vlan)
-
         if lvm.network_type == n_const.TYPE_VLAN:
             br = self.phys_brs[physical_network]
         if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
