@@ -200,6 +200,10 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         segments_db.add_network_segment(
             context, network_id, segment, segment_index)
 
+    def _update_network_segment(self, context, network_id, segmentation_id):
+        segments_db.update_network_segment(
+            context, network_id, segmentation_id)
+
     def create_network_segments(self, context, network, tenant_id):
         """Call type drivers to create network segments."""
         segments = self._process_provider_create(network)
@@ -222,6 +226,30 @@ class TypeManager(stevedore.named.NamedExtensionManager):
                     context, filters=filters)
                 self._add_network_segment(context, network_id, segment)
 
+    def update_network_segment(self, context, network, net_data, segment):
+        """Call type drivers to update a network segment.
+
+        Update operation is currently only supported for VLAN type segments,
+        and only the SEGMENTATION_ID field can be changed.
+        """
+        segmentation_id = net_data.get(provider.SEGMENTATION_ID)
+        network_type = segment[api.NETWORK_TYPE]
+        if network_type != constants.TYPE_VLAN:
+            msg = (_('Only VLAN type networks can be updated.'))
+            raise exc.InvalidInput(error_message=msg)
+        elif not segmentation_id:
+            msg = (_('Only %s field can be updated in VLAN type networks') %
+                   api.SEGMENTATION_ID)
+            raise exc.InvalidInput(error_message=msg)
+
+        new_segment = {api.NETWORK_TYPE: segment[api.NETWORK_TYPE],
+                       api.PHYSICAL_NETWORK: segment[api.PHYSICAL_NETWORK],
+                       api.SEGMENTATION_ID: segmentation_id}
+        self.validate_provider_segment(new_segment)
+        self.reserve_provider_segment(context, new_segment)
+        self._update_network_segment(context, segment['id'], segmentation_id)
+        self.release_network_segment(context, segment)
+
     def reserve_network_segment(self, context, segment_data):
         """Call type drivers to reserve a network segment."""
         # Validate the data of segment
@@ -499,7 +527,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def update_network_precommit(self, context):
         """Notify all mechanism drivers during network update.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if update_network_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver update_network_precommit call fails.
@@ -529,7 +557,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def delete_network_precommit(self, context):
         """Notify all mechanism drivers during network deletion.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if delete_network_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver delete_network_precommit call fails.
@@ -563,7 +591,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def create_subnet_precommit(self, context):
         """Notify all mechanism drivers during subnet creation.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if create_subnet_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver create_subnet_precommit call fails.
@@ -593,7 +621,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def update_subnet_precommit(self, context):
         """Notify all mechanism drivers during subnet update.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if update_subnet_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver update_subnet_precommit call fails.
@@ -623,7 +651,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def delete_subnet_precommit(self, context):
         """Notify all mechanism drivers during subnet deletion.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if delete_subnet_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver delete_subnet_precommit call fails.
@@ -657,7 +685,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def create_port_precommit(self, context):
         """Notify all mechanism drivers during port creation.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if create_port_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver create_port_precommit call fails.
@@ -687,7 +715,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def update_port_precommit(self, context):
         """Notify all mechanism drivers during port update.
 
-        :raises: DB retriable error if create_network_precommit raises them
+        :raises: DB retriable error if update_port_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver update_port_precommit call fails.
@@ -717,7 +745,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
     def delete_port_precommit(self, context):
         """Notify all mechanism drivers during port deletion.
 
-        :raises:DB retriable error if create_network_precommit raises them
+        :raises:DB retriable error if delete_port_precommit raises them
         See neutron_lib.db.api.is_retriable for what db exception is retriable
         or neutron.plugins.ml2.common.MechanismDriverError
         if any mechanism driver delete_port_precommit call fails.
@@ -798,10 +826,19 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                        'host': context.host})
             return False
 
+        drivers = self._check_drivers_connectivity(drivers, context)
+        if not drivers:
+            LOG.error("Port %(port)s does not have an IP address assigned and "
+                      "there are no driver with 'connectivity' = 'l2'. The "
+                      "port cannot be bound.",
+                      {'port': context.current['id']})
+            return False
+
         for driver in drivers:
             if not self._check_driver_to_bind(driver, segments_to_bind,
                                               context._binding_levels):
                 continue
+
             try:
                 context._prepare_to_bind(segments_to_bind)
                 driver.obj.bind_port(context)
@@ -837,7 +874,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                         # level that represents the closest physical interface
                         # to the nova server." Link to spec:
                         #
-                        # https://review.openstack.org/#/c/508149/14/specs\
+                        # https://review.opendev.org/#/c/508149/14/specs\
                         #        /rocky/minimum-bandwidth-\
                         #        allocation-placement-api.rst@582
                         #
@@ -1000,6 +1037,21 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                 return False
         return True
 
+    def _check_drivers_connectivity(self, drivers, port_context):
+        """If port does not have an IP address, driver connectivity must be l2
+
+        A port without an IP address can be bound only to a mech driver with
+        "connectivity" = "l2". "legacy" or "l3" (e.g.: Calico) drivers cannot
+        have a port bound without an IP allocated.
+        """
+        if port_context.current.get('fixed_ips'):
+            return drivers
+
+        return [d for d in drivers if
+                getattr(d.obj, 'vif_details', {}).get(
+                    portbindings.VIF_DETAILS_CONNECTIVITY) ==
+                portbindings.CONNECTIVITY_L2]
+
     def get_workers(self):
         workers = []
         for driver in self.ordered_mech_drivers:
