@@ -10,8 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import ctypes
-from ctypes import util as ctypes_util
 import errno
 import functools
 import os
@@ -19,7 +17,6 @@ import socket
 
 from neutron_lib import constants
 from oslo_concurrency import lockutils
-from oslo_log import log as logging
 import pyroute2
 from pyroute2 import netlink
 from pyroute2.netlink import exceptions as netlink_exceptions
@@ -34,27 +31,10 @@ from neutron._i18n import _
 from neutron import privileged
 
 
-LOG = logging.getLogger(__name__)
-
 _IP_VERSION_FAMILY_MAP = {4: socket.AF_INET, 6: socket.AF_INET6}
 
 NETNS_RUN_DIR = '/var/run/netns'
 
-_CDLL = None
-
-
-def _get_cdll():
-    global _CDLL
-    if not _CDLL:
-        # NOTE(ralonsoh): from https://docs.python.org/3.6/library/
-        # ctypes.html#ctypes.PyDLL: "Instances of this class behave like CDLL
-        # instances, except that the Python GIL is not released during the
-        # function call, and after the function execution the Python error
-        # flag is checked."
-        # Check https://bugs.launchpad.net/neutron/+bug/1870352
-        _CDLL = ctypes.PyDLL(ctypes_util.find_library('c'), use_errno=True)
-    return _CDLL
-
 
 @lockutils.synchronized("privileged-ip-lib")
 # NOTE(slaweq): Because of issue with pyroute2.NetNS objects running in threads
@@ -260,17 +240,12 @@ def _translate_ip_device_exception(e, device=None, namespace=None):
                                              namespace=namespace)
 
 
-def get_link_id(device, namespace, raise_exception=True):
-    with get_iproute(namespace) as ip:
-        link_id = ip.link_lookup(ifname=device)
-    if not link_id or len(link_id) < 1:
-        if raise_exception:
-            raise NetworkInterfaceNotFound(device=device, namespace=namespace)
-        else:
-            LOG.debug('Interface %(dev)s not found in namespace %(namespace)s',
-                      {'dev': device, 'namespace': namespace})
-            return None
-    return link_id[0]
+def get_link_id(device, namespace):
+    try:
+        with get_iproute(namespace) as ip:
+            return ip.link_lookup(ifname=device)[0]
+    except IndexError:
+        raise NetworkInterfaceNotFound(device=device, namespace=namespace)
 
 
 def _run_iproute_link(command, device, namespace=None, **kwargs):
@@ -401,8 +376,10 @@ def delete_interface(ifname, namespace, **kwargs):
 @privileged.default.entrypoint
 def interface_exists(ifname, namespace):
     try:
-        idx = get_link_id(ifname, namespace, raise_exception=False)
+        idx = get_link_id(ifname, namespace)
         return bool(idx)
+    except NetworkInterfaceNotFound:
+        return False
     except OSError as e:
         if e.errno == errno.ENOENT:
             return False
@@ -435,7 +412,8 @@ def get_link_attributes(device, namespace):
         'brd': link.get_attr('IFLA_BROADCAST'),
         'link/ether': link.get_attr('IFLA_ADDRESS'),
         'alias': link.get_attr('IFLA_IFALIAS'),
-        'allmulticast': bool(link['flags'] & ifinfmsg.IFF_ALLMULTI)
+        'allmulticast': bool(link['flags'] & ifinfmsg.IFF_ALLMULTI),
+        'link_kind': link.get_nested('IFLA_LINKINFO', 'IFLA_INFO_KIND')
     }
 
 
@@ -519,44 +497,32 @@ def dump_neigh_entries(ip_version, device, namespace, **kwargs):
 
 
 @privileged.default.entrypoint
-# NOTE(slaweq): Because of issue with pyroute2.NetNS objects running in threads
-# we need to lock this function to workaround this issue.
-# For details please check https://bugs.launchpad.net/neutron/+bug/1811515
-@lockutils.synchronized("privileged-ip-lib")
 def create_netns(name, **kwargs):
     """Create a network namespace.
 
     :param name: The name of the namespace to create
     """
     try:
-        netns.create(name, libc=_get_cdll())
+        netns.create(name, **kwargs)
     except OSError as e:
         if e.errno != errno.EEXIST:
             raise
 
 
 @privileged.default.entrypoint
-# NOTE(slaweq): Because of issue with pyroute2.NetNS objects running in threads
-# we need to lock this function to workaround this issue.
-# For details please check https://bugs.launchpad.net/neutron/+bug/1811515
-@lockutils.synchronized("privileged-ip-lib")
 def remove_netns(name, **kwargs):
     """Remove a network namespace.
 
     :param name: The name of the namespace to remove
     """
     try:
-        netns.remove(name, libc=_get_cdll())
+        netns.remove(name, **kwargs)
     except OSError as e:
         if e.errno != errno.ENOENT:
             raise
 
 
 @privileged.default.entrypoint
-# NOTE(slaweq): Because of issue with pyroute2.NetNS objects running in threads
-# we need to lock this function to workaround this issue.
-# For details please check https://bugs.launchpad.net/neutron/+bug/1811515
-@lockutils.synchronized("privileged-ip-lib")
 def list_netns(**kwargs):
     """List network namespaces.
 
@@ -571,24 +537,15 @@ def make_serializable(value):
     This function converts 'netlink.nla_slot' object (key, value) in a list
     of two elements.
     """
-    def _ensure_string(value):
-        # NOTE(ralonsoh): once support for PY2 is dropped, the str()
-        # conversion will be no needed and six.binary_type --> bytes.
-        return (str(value.decode('utf-8'))
-                if isinstance(value, six.binary_type) else value)
-
     if isinstance(value, list):
         return [make_serializable(item) for item in value]
-    elif isinstance(value, netlink.nla_slot):
-        return [_ensure_string(value[0]), make_serializable(value[1])]
-    elif isinstance(value, netlink.nla_base) and six.PY3:
-        return make_serializable(value.dump())
     elif isinstance(value, dict):
-        return {_ensure_string(key): make_serializable(data)
-                for key, data in value.items()}
+        return {key: make_serializable(data) for key, data in value.items()}
+    elif isinstance(value, netlink.nla_slot):
+        return [value[0], make_serializable(value[1])]
     elif isinstance(value, tuple):
         return tuple(make_serializable(item) for item in value)
-    return _ensure_string(value)
+    return value
 
 
 @_sync
@@ -644,9 +601,8 @@ def list_ip_rules(namespace, ip_version, match=None, **kwargs):
     """List all IP rules"""
     try:
         with get_iproute(namespace) as ip:
-            rules = make_serializable(ip.get_rules(
-                family=_IP_VERSION_FAMILY_MAP[ip_version],
-                match=match, **kwargs))
+            rules = ip.get_rules(family=_IP_VERSION_FAMILY_MAP[ip_version],
+                                 match=match, **kwargs)
             for rule in rules:
                 rule['attrs'] = {
                     key: value for key, value
@@ -687,3 +643,87 @@ def delete_ip_rule(namespace, **kwargs):
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
         raise
+
+
+def _make_pyroute2_route_args(namespace, ip_version, cidr, device, via, table,
+                              metric, scope, protocol):
+    """Returns a dictionary of arguments to be used in pyroute route commands
+
+    :param namespace: (string) name of the namespace
+    :param ip_version: (int) [4, 6]
+    :param cidr: (string) source IP or CIDR address (IPv4, IPv6)
+    :param device: (string) input interface name
+    :param via: (string) gateway IP address
+    :param table: (string, int) table number or name
+    :param metric: (int) route metric
+    :param scope: (int) route scope
+    :param protocol: (string) protocol name (pyroute2.netlink.rtnl.rt_proto)
+    :return: a dictionary with the kwargs needed in pyroute rule commands
+    """
+    args = {'family': _IP_VERSION_FAMILY_MAP[ip_version]}
+    if not scope:
+        scope = 'global' if via else 'link'
+    scope = _get_scope_name(scope)
+    if scope:
+        args['scope'] = scope
+    if cidr:
+        args['dst'] = cidr
+    if device:
+        args['oif'] = get_link_id(device, namespace)
+    if via:
+        args['gateway'] = via
+    if table:
+        args['table'] = int(table)
+    if metric:
+        args['priority'] = int(metric)
+    if protocol:
+        args['proto'] = protocol
+    return args
+
+
+@_sync
+@privileged.default.entrypoint
+def add_ip_route(namespace, cidr, ip_version, device=None, via=None,
+                 table=None, metric=None, scope=None, **kwargs):
+    """Add an IP route"""
+    kwargs.update(_make_pyroute2_route_args(
+        namespace, ip_version, cidr, device, via, table, metric, scope,
+        'static'))
+    try:
+        with get_iproute(namespace) as ip:
+            ip.route('replace', **kwargs)
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            raise NetworkNamespaceNotFound(netns_name=namespace)
+        raise
+
+
+@_sync
+@privileged.default.entrypoint
+def list_ip_routes(namespace, ip_version, device=None, table=None, **kwargs):
+    """List IP routes"""
+    kwargs.update(_make_pyroute2_route_args(
+        namespace, ip_version, None, device, None, table, None, None, None))
+    try:
+        with get_iproute(namespace) as ip:
+            return make_serializable(ip.route('show', **kwargs))
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            raise NetworkNamespaceNotFound(netns_name=namespace)
+        raise
+
+
+@_sync
+@privileged.default.entrypoint
+def delete_ip_route(namespace, cidr, ip_version, device=None, via=None,
+                    table=None, scope=None, **kwargs):
+    """Delete an IP route"""
+    kwargs.update(_make_pyroute2_route_args(
+        namespace, ip_version, cidr, device, via, table, None, scope, None))
+    try:
+        with get_iproute(namespace) as ip:
+            ip.route('del', **kwargs)
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            raise NetworkNamespaceNotFound(netns_name=namespace)
+        raise
