@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -21,7 +22,6 @@ from neutron.agent.l3 import dvr_snat_ns
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
-from neutron.common import constants
 from neutron.common import ipv6_utils
 from neutron.conf.agent import common as config
 from neutron.services.metering.drivers import abstract_driver
@@ -346,8 +346,8 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
         rm.iptables_manager.ipv4['filter'].add_rule(
             label_chain, '', wrap=False)
 
-    def _process_metering_rule_action_based_on_ns(
-        self, router, action, ext_dev, im):
+    def _process_metering_rule_action_based_on_ns(self, router, action,
+                                                  ext_dev, im):
         '''Process metering rule actions based specific namespaces.'''
         rm = self.routers.get(router['id'])
         with IptablesManagerTransaction(im):
