@@ -17,7 +17,10 @@ from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api.definitions import port_resource_request
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import qos as qos_apidef
+from neutron_lib.api.definitions import qos_bw_limit_direction
 from neutron_lib.api.definitions import qos_bw_minimum_ingress
+from neutron_lib.api.definitions import qos_default
+from neutron_lib.api.definitions import qos_rule_type_details
 from neutron_lib.api.definitions import qos_rules_alias
 from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
@@ -53,9 +56,9 @@ class QoSPlugin(qos.QoSPluginBase):
 
     """
     supported_extension_aliases = [qos_apidef.ALIAS,
-                                   'qos-bw-limit-direction',
-                                   'qos-default',
-                                   'qos-rule-type-details',
+                                   qos_bw_limit_direction.ALIAS,
+                                   qos_default.ALIAS,
+                                   qos_rule_type_details.ALIAS,
                                    port_resource_request.ALIAS,
                                    qos_bw_minimum_ingress.ALIAS,
                                    qos_rules_alias.ALIAS]
@@ -123,7 +126,7 @@ class QoSPlugin(qos.QoSPluginBase):
 
         # TODO(lajoskatona): Change to handle all segments when any traits
         # support will be available. See Placement spec:
-        # https://review.openstack.org/565730
+        # https://review.opendev.org/565730
         first_segment = network_object.NetworkSegment.get_objects(
             context.get_admin_context(), network_id=port_db.network_id)[0]
 
@@ -158,10 +161,8 @@ class QoSPlugin(qos.QoSPluginBase):
         context = kwargs['context']
         port_id = kwargs['port']['id']
         port = ports_object.Port.get_object(context, id=port_id)
-        network = network_object.Network.get_object(context,
-                                                    id=port.network_id)
 
-        policy_id = port.qos_policy_id or network.qos_policy_id
+        policy_id = port.qos_policy_id or port.qos_network_policy_id
         if policy_id is None:
             return
 
@@ -398,7 +399,7 @@ class QoSPlugin(qos.QoSPluginBase):
         rule_type = rule_cls.rule_type
         rule_data = rule_data[rule_type + '_rule']
 
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_WRITER.using(context):
             # Ensure that we have access to the policy.
             policy = policy_object.QosPolicy.get_policy_obj(context, policy_id)
             checker.check_bandwidth_rule_conflict(policy, rule_data)
@@ -437,7 +438,7 @@ class QoSPlugin(qos.QoSPluginBase):
         rule_type = rule_cls.rule_type
         rule_data = rule_data[rule_type + '_rule']
 
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_WRITER.using(context):
             # Ensure we have access to the policy.
             policy = policy_object.QosPolicy.get_policy_obj(context, policy_id)
             # Ensure the rule belongs to the policy.
@@ -502,7 +503,7 @@ class QoSPlugin(qos.QoSPluginBase):
 
         :returns: None
         """
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_WRITER.using(context):
             # Ensure we have access to the policy.
             policy = policy_object.QosPolicy.get_policy_obj(context, policy_id)
             rule = policy.get_rule_by_id(rule_id)
@@ -551,7 +552,7 @@ class QoSPlugin(qos.QoSPluginBase):
         :returns: a QoS policy rule object
         :raises: qos_exc.QosRuleNotFound
         """
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_READER.using(context):
             # Ensure we have access to the policy.
             policy_object.QosPolicy.get_policy_obj(context, policy_id)
             rule = rule_cls.get_object(context, id=rule_id)
@@ -594,7 +595,7 @@ class QoSPlugin(qos.QoSPluginBase):
 
         :returns: QoS policy rule objects meeting the search criteria
         """
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_READER.using(context):
             # Ensure we have access to the policy.
             policy_object.QosPolicy.get_policy_obj(context, policy_id)
             filters = filters or dict()
