@@ -23,6 +23,12 @@ class SegmentNotFound(exceptions.NotFound):
     message = _("Segment %(segment_id)s could not be found.")
 
 
+class NoUpdateSubnetWhenMultipleSegmentsOnNetwork(exceptions.BadRequest):
+    message = _("The network '%(network_id)s' has multiple segments, it is "
+                "only possible to associate an existing subnet with a segment "
+                "on networks with a single segment.")
+
+
 class SubnetsNotAllAssociatedWithSegments(exceptions.BadRequest):
     message = _("All of the subnets on network '%(network_id)s' must either "
                 "all be associated with segments or all not associated with "
@@ -33,6 +39,11 @@ class SubnetCantAssociateToDynamicSegment(exceptions.BadRequest):
     message = _("A subnet cannot be associated with a dynamic segment.")
 
 
+class SubnetSegmentAssociationChangeNotAllowed(exceptions.BadRequest):
+    message = _("A subnet that is already associated with a segment cannot "
+                "have its segment association changed.")
+
+
 class NetworkIdsDontMatch(exceptions.BadRequest):
     message = _("The subnet's network id, '%(subnet_network)s', doesn't match "
                 "the network_id of segment '%(segment_id)s'")
