@@ -28,7 +28,6 @@ import time
 import fixtures
 import netaddr
 from neutron_lib import constants as n_const
-from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
@@ -111,7 +110,7 @@ def assert_ping(src_namespace, dst_ip, timeout=1, count=3):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', count,
+    ns_ip_wrapper.netns.execute([ping_command, '-c', count, '-W', timeout,
                                  dst_ip])
 
 
@@ -124,7 +123,7 @@ def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
     # cannot be used and it needs to be done using the following workaround.
     for _index in range(count):
         start_time = time.time()
-        ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', '1',
+        ns_ip_wrapper.netns.execute([ping_command, '-c', '1', '-W', timeout,
                                      dst_ip])
         end_time = time.time()
         diff = end_time - start_time
@@ -229,6 +228,18 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
     return get_unused_port(used_ports, start, end)
 
 
+def set_local_port_range(start, end):
+    utils.execute(
+        ['sysctl', '-w', 'net.ipv4.ip_local_port_range=%d %d' % (start, end)],
+        run_as_root=True)
+    utils.execute(['sysctl', '-p'], run_as_root=True)
+    # verify
+    port_range = utils.execute(
+        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+    assert int(port_range.split()[0]) == start
+    assert int(port_range.split()[1]) == end
+
+
 def create_patch_ports(source, destination):
     """Hook up two OVS bridges.
 
@@ -390,12 +401,11 @@ class Pinger(object):
             raise RuntimeError("This pinger has already a running process")
         ip_version = common_utils.get_ip_version(self.address)
         ping_exec = 'ping' if ip_version == n_const.IP_VERSION_4 else 'ping6'
-        cmd = [ping_exec, '-W', str(self.timeout)]
+        cmd = [ping_exec, self.address, '-W', str(self.timeout)]
         if self.count:
             cmd.extend(['-c', str(self.count)])
         if self.interval:
             cmd.extend(['-i', str(self.interval)])
-        cmd.append(self.address)
         self.proc = RootHelperProcess(cmd, namespace=self.namespace)
 
     def stop(self):
@@ -544,22 +554,13 @@ class NetcatTester(object):
         proc = RootHelperProcess(cmd, namespace=namespace)
         return proc
 
-    def stop_processes(self, skip_errors=None):
-        skip_errors = (['No such process'] if skip_errors is None
-                       else skip_errors)
+    def stop_processes(self):
         for proc_attr in ('_client_process', '_server_process'):
             proc = getattr(self, proc_attr)
             if proc:
-                try:
-                    if proc.poll() is None:
-                        proc.kill()
-                        proc.wait()
-                except n_exc.ProcessExecutionError as exc:
-                    for skip_error in skip_errors:
-                        if skip_error in str(exc):
-                            break
-                    else:
-                        raise exc
+                if proc.poll() is None:
+                    proc.kill()
+                    proc.wait()
                 setattr(self, proc_attr, None)
 
 
