@@ -18,19 +18,28 @@ import tempfile
 
 from neutron_lib import constants
 
-from neutron.common import constants as c_const
 from neutron.common import utils
 from neutron.plugins.ml2.extensions import qos as qos_ext
 from neutron.tests import base
 from neutron.tests.common import config_fixtures
 from neutron.tests.common.exclusive_resources import port
 from neutron.tests.common import helpers as c_helpers
+from neutron.tests.common import net_helpers
 from neutron.tests.fullstack import base as fullstack_base
 
 PHYSICAL_NETWORK_NAME = "physnet1"
 MINIMUM_BANDWIDTH_INGRESS_KBPS = 1000
 MINIMUM_BANDWIDTH_EGRESS_KBPS = 1000
 
+NEUTRON_SERVER_PORT_START = 10000
+NEUTRON_SERVER_PORT_END = 20000
+
+OVS_OF_PORT_LISTEN_START = 20001
+OVS_OF_PORT_LISTEN_END = 30000
+
+CLIENT_CONN_PORT_START = 30001
+CLIENT_CONN_PORT_END = 65000
+
 
 class ConfigFixture(config_fixtures.ConfigFileFixture):
     """A fixture that holds an actual Neutron configuration.
@@ -83,7 +92,7 @@ class NeutronConfigFixture(ConfigFixture):
                 'lock_path': '$state_path/lock',
             },
             'agent': {
-                'report_interval': str(env_desc.agent_down_time / 2.0),
+                'report_interval': str(env_desc.agent_down_time // 2),
                 'log_agent_heartbeats': 'True',
             },
         })
@@ -101,11 +110,28 @@ class NeutronConfigFixture(ConfigFixture):
         if env_desc.router_scheduler:
             self.config['DEFAULT']['router_scheduler_driver'] = (
                 env_desc.router_scheduler)
+        if env_desc.has_placement:
+            service_plugins = self.config['DEFAULT']['service_plugins']
+            self.config['DEFAULT']['service_plugins'] = (
+                '%s,%s' % (service_plugins, 'placement')
+            )
+            self.config.update({
+                'placement': {
+                    'auth_type': 'noauth',
+                    'auth_section': 'http://127.0.0.1:%s/placement' %
+                    env_desc.placement_port
+                }
+            })
+
+        net_helpers.set_local_port_range(CLIENT_CONN_PORT_START,
+                                         CLIENT_CONN_PORT_END)
 
     def _setUp(self):
         self.config['DEFAULT'].update({
             'bind_port': self.useFixture(
-                port.ExclusivePort(constants.PROTO_NAME_TCP)).port
+                port.ExclusivePort(constants.PROTO_NAME_TCP,
+                                   start=NEUTRON_SERVER_PORT_START,
+                                   end=NEUTRON_SERVER_PORT_END)).port
         })
         super(NeutronConfigFixture, self)._setUp()
 
@@ -172,7 +198,6 @@ class OVSConfigFixture(ConfigFixture):
             'ovs': {
                 'local_ip': local_ip,
                 'integration_bridge': self._generate_integration_bridge(),
-                'of_interface': host_desc.of_interface,
                 'bridge_mappings': '%s:%s' % (PHYSICAL_NETWORK_NAME, ext_dev),
                 'of_inactivity_probe': '0',
             },
@@ -196,7 +221,7 @@ class OVSConfigFixture(ConfigFixture):
                 'tun_peer_patch_port': self._generate_tun_peer()})
         else:
             if env_desc.report_bandwidths:
-                self.config['ovs'][c_const.RP_BANDWIDTHS] = \
+                self.config['ovs'][constants.RP_BANDWIDTHS] = \
                     '%s:%s:%s' % (ext_dev, MINIMUM_BANDWIDTH_EGRESS_KBPS,
                                   MINIMUM_BANDWIDTH_INGRESS_KBPS)
 
@@ -213,11 +238,12 @@ class OVSConfigFixture(ConfigFixture):
             })
 
     def _setUp(self):
-        if self.config['ovs']['of_interface'] == 'native':
-            self.config['ovs'].update({
-                'of_listen_port': self.useFixture(
-                    port.ExclusivePort(constants.PROTO_NAME_TCP)).port
-            })
+        self.config['ovs'].update({
+            'of_listen_port': self.useFixture(
+                port.ExclusivePort(constants.PROTO_NAME_TCP,
+                                   start=OVS_OF_PORT_LISTEN_START,
+                                   end=OVS_OF_PORT_LISTEN_END)).port
+        })
         super(OVSConfigFixture, self)._setUp()
 
     def _generate_integration_bridge(self):
@@ -276,6 +302,22 @@ class SRIOVConfigFixture(ConfigFixture):
         super(SRIOVConfigFixture, self)._setUp()
 
 
+class PlacementConfigFixture(ConfigFixture):
+
+    def __init__(self, env_desc, host_desc, temp_dir):
+        super(PlacementConfigFixture, self).__init__(
+            env_desc, host_desc, temp_dir, base_filename='placement.ini')
+        self.config.update({
+            'DEFAULT': {
+                'debug': 'True',
+                'placement_port': self.env_desc.placement_port
+            }
+        })
+
+    def _setUp(self):
+        super(PlacementConfigFixture, self)._setUp()
+
+
 class LinuxBridgeConfigFixture(ConfigFixture):
 
     def __init__(self, env_desc, host_desc, temp_dir, local_ip,
@@ -349,6 +391,10 @@ class L3ConfigFixture(ConfigFixture):
             self.config['agent'].update({
                 'availability_zone': host_desc.availability_zone
             })
+        if host_desc.l3_agent_extensions:
+            self.config['agent'].update({
+                'extensions': host_desc.l3_agent_extensions
+            })
 
     def _prepare_config_with_ovs_agent(self, integration_bridge):
         self.config.update({
