@@ -18,7 +18,6 @@ from neutron_lib import constants
 from neutron_lib.services.qos import constants as qos_consts
 from neutronclient.common import exceptions
 from oslo_utils import uuidutils
-import testscenarios
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import tc_lib
@@ -28,7 +27,6 @@ from neutron.tests.fullstack import base
 from neutron.tests.fullstack.resources import config as fullstack_config
 from neutron.tests.fullstack.resources import environment
 from neutron.tests.fullstack.resources import machine
-from neutron.tests.fullstack import utils as fullstack_utils
 from neutron.tests.unit import testlib_api
 
 from neutron.conf.plugins.ml2.drivers import linuxbridge as \
@@ -48,7 +46,6 @@ DSCP_MARK = 16
 
 
 class BaseQoSRuleTestCase(object):
-    of_interface = None
     number_of_hosts = 1
     physical_network = None
 
@@ -63,7 +60,6 @@ class BaseQoSRuleTestCase(object):
         host_desc = [
             environment.HostDescription(
                 l3_agent=False,
-                of_interface=self.of_interface,
                 l2_agent_type=self.l2_agent_type
             ) for _ in range(self.number_of_hosts)]
         env_desc = environment.EnvironmentDescription(
@@ -344,12 +340,10 @@ class _TestBwLimitQoS(BaseQoSRuleTestCase):
 
 class TestBwLimitQoSOvs(_TestBwLimitQoS, base.BaseFullStackTestCase):
     l2_agent_type = constants.AGENT_TYPE_OVS
-    direction_scenarios = [
+    scenarios = [
         ('ingress', {'direction': constants.INGRESS_DIRECTION}),
         ('egress', {'direction': constants.EGRESS_DIRECTION})
     ]
-    scenarios = testscenarios.multiply_scenarios(
-        direction_scenarios, fullstack_utils.get_ovs_interface_scenarios())
 
     @staticmethod
     def _get_expected_burst_value(limit, direction):
@@ -520,7 +514,6 @@ class _TestDscpMarkingQoS(BaseQoSRuleTestCase):
 
 
 class TestDscpMarkingQoSOvs(_TestDscpMarkingQoS, base.BaseFullStackTestCase):
-    scenarios = fullstack_utils.get_ovs_interface_scenarios()
     l2_agent_type = constants.AGENT_TYPE_OVS
 
     def _wait_for_dscp_marking_rule_applied(self, vm, dscp_mark):
@@ -670,11 +663,9 @@ class _TestMinBwQoS(BaseQoSRuleTestCase):
 
 class TestMinBwQoSOvs(_TestMinBwQoS, base.BaseFullStackTestCase):
     l2_agent_type = constants.AGENT_TYPE_OVS
-    direction_scenarios = [
+    scenarios = [
         ('egress', {'direction': constants.EGRESS_DIRECTION})
     ]
-    scenarios = testscenarios.multiply_scenarios(
-        direction_scenarios, fullstack_utils.get_ovs_interface_scenarios())
 
     def _wait_for_min_bw_rule_applied(self, vm, min_bw, direction):
         if direction == constants.EGRESS_DIRECTION:
@@ -685,22 +676,10 @@ class TestMinBwQoSOvs(_TestMinBwQoS, base.BaseFullStackTestCase):
             self.fail('"%s" direction not implemented'
                       % constants.INGRESS_DIRECTION)
 
-    def _find_agent_qos_and_queue(self, vm):
-        # NOTE(ralonsoh): the "_min_bw_qos_id" in vm.bridge is not the same as
-        # the ID in the agent br_int instance. We need first to find the QoS
-        # register and the Queue assigned to vm.neutron_port['id']
-        queue = vm.bridge._find_queue(vm.neutron_port['id'])
-        queue_num = int(queue['external_ids']['queue-num'])
-        qoses = vm.bridge._list_qos()
-        for qos in qoses:
-            qos_queue = qos['queues'].get(queue_num)
-            if qos_queue and qos_queue.uuid == queue['_uuid']:
-                return qos, qos_queue
-
-        self.fail('QoS register not found with queue-num %s' % queue_num)
-
-    def test_min_bw_qos_port_removed(self):
-        """Test if min BW limit config is properly removed when port removed"""
+    def test_bw_limit_qos_port_removed(self):
+        """Test if rate limit config is properly removed when whole port is
+        removed.
+        """
         # Create port with qos policy attached
         vm, qos_policy = self._prepare_vm_with_qos_policy(
             [functools.partial(
@@ -708,15 +687,9 @@ class TestMinBwQoSOvs(_TestMinBwQoS, base.BaseFullStackTestCase):
         self._wait_for_min_bw_rule_applied(
             vm, MIN_BANDWIDTH, self.direction)
 
-        qos, queue = self._find_agent_qos_and_queue(vm)
-        self.assertEqual({'min-rate': str(MIN_BANDWIDTH * 1000)},
-                         queue.other_config)
-        queues = vm.bridge._list_queues(port=vm.neutron_port['id'])
-        self.assertEqual(1, len(queues))
-        self.assertEqual(queue.uuid, queues[0]['_uuid'])
-
         # Delete port with qos policy attached
         vm.destroy()
         self._wait_for_min_bw_rule_removed(vm, self.direction)
-        self.assertEqual([],
-                         vm.bridge._list_queues(port=vm.neutron_port['id']))
+        self.assertIsNone(vm.bridge.find_qos(vm.port.name))
+        self.assertIsNone(vm.bridge.find_queue(vm.port.name,
+                                               ovs_lib.QOS_DEFAULT_QUEUE))
