@@ -15,6 +15,7 @@
 
 import copy
 import functools
+import textwrap
 
 import mock
 import netaddr
@@ -34,10 +35,8 @@ from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
 from neutron.agent.metadata import driver as metadata_driver
-from neutron.common import constants as n_const
 from neutron.common import utils as common_utils
 from neutron.conf.agent import common as agent_config
-from neutron.conf.agent.l3 import config as l3_config
 from neutron.conf import common as common_config
 from neutron.tests.common import l3_test_common
 from neutron.tests.common import net_helpers
@@ -48,39 +47,6 @@ _uuid = uuidutils.generate_uuid
 
 OVS_INTERFACE_DRIVER = 'neutron.agent.linux.interface.OVSInterfaceDriver'
 
-KEEPALIVED_CONFIG = """\
-global_defs {
-    notification_email_from %(email_from)s
-    router_id %(router_id)s
-}
-vrrp_instance VR_1 {
-    state BACKUP
-    interface %(ha_device_name)s
-    virtual_router_id 1
-    priority 50
-    garp_master_delay 60
-    nopreempt
-    advert_int 2
-    track_interface {
-        %(ha_device_name)s
-    }
-    virtual_ipaddress {
-        169.254.0.1/24 dev %(ha_device_name)s
-    }
-    virtual_ipaddress_excluded {
-        %(floating_ip_cidr)s dev %(ex_device_name)s no_track
-        %(external_device_cidr)s dev %(ex_device_name)s no_track
-        %(internal_device_cidr)s dev %(internal_device_name)s no_track
-        %(ex_port_ipv6)s dev %(ex_device_name)s scope link no_track
-        %(int_port_ipv6)s dev %(internal_device_name)s scope link no_track
-    }
-    virtual_routes {
-        0.0.0.0/0 via %(default_gateway_ip)s dev %(ex_device_name)s no_track
-        8.8.8.0/24 via 19.4.4.4 no_track
-        %(extra_subnet_cidr)s dev %(ex_device_name)s scope link no_track
-    }
-}"""
-
 
 def get_ovs_bridge(br_name):
     return ovs_lib.OVSBridge(br_name)
@@ -95,7 +61,6 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         self.mock_plugin_api = mock.patch(
             'neutron.agent.l3.agent.L3PluginApi').start().return_value
         mock.patch('neutron.agent.rpc.PluginReportStateAPI').start()
-        l3_config.register_l3_agent_config_opts(l3_config.OPTS, cfg.CONF)
         self.conf = self._configure_agent('agent1')
         self.agent = neutron_l3_agent.L3NATAgentWithStateReport('agent1',
                                                                 self.conf)
@@ -237,8 +202,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
     def _gateway_check(self, gateway_ip, external_device):
         expected_gateway = gateway_ip
         ip_vers = netaddr.IPAddress(expected_gateway).version
-        existing_gateway = (external_device.route.get_gateway(
-            ip_version=ip_vers).get('gateway'))
+        existing_gateway = external_device.route.get_gateway(
+            ip_version=ip_vers).get('via')
         self.assertEqual(expected_gateway, existing_gateway)
 
     def _assert_ha_device(self, router):
@@ -277,7 +242,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             ra_state = ip_wrapper.netns.execute(['sysctl', '-b',
                 'net.ipv6.conf.%s.accept_ra' % device_name])
             return (
-                enabled == (int(ra_state) != n_const.ACCEPT_RA_DISABLED))
+                enabled == (int(ra_state) != constants.ACCEPT_RA_DISABLED))
 
         common_utils.wait_until_true(_ipv6_accept_ra_state)
 
@@ -349,7 +314,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             # Note(SridharG): enable the assert_gateway for IPv6 once
             # keepalived on Ubuntu14.04 (i.e., check-neutron-dsvm-functional
             # platform) is updated to 1.2.10 (or above).
-            # For more details: https://review.openstack.org/#/c/151284/
+            # For more details: https://review.opendev.org/#/c/151284/
             self._assert_gateway(router, v6_ext_gw_with_sub)
             self.assertTrue(self.floating_ips_configured(router))
             self._assert_snat_chains(router)
@@ -477,7 +442,38 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             router.get_floating_ips()[0]['floating_ip_address'])
         default_gateway_ip = external_port['subnets'][0].get('gateway_ip')
         extra_subnet_cidr = external_port['extra_subnets'][0].get('cidr')
-        return KEEPALIVED_CONFIG % {
+        return textwrap.dedent("""\
+            global_defs {
+                notification_email_from %(email_from)s
+                router_id %(router_id)s
+            }
+            vrrp_instance VR_1 {
+                state BACKUP
+                interface %(ha_device_name)s
+                virtual_router_id 1
+                priority 50
+                garp_master_delay 60
+                nopreempt
+                advert_int 2
+                track_interface {
+                    %(ha_device_name)s
+                }
+                virtual_ipaddress {
+                    169.254.0.1/24 dev %(ha_device_name)s
+                }
+                virtual_ipaddress_excluded {
+                    %(floating_ip_cidr)s dev %(ex_device_name)s
+                    %(external_device_cidr)s dev %(ex_device_name)s
+                    %(internal_device_cidr)s dev %(internal_device_name)s
+                    %(ex_port_ipv6)s dev %(ex_device_name)s scope link
+                    %(int_port_ipv6)s dev %(internal_device_name)s scope link
+                }
+                virtual_routes {
+                    0.0.0.0/0 via %(default_gateway_ip)s dev %(ex_device_name)s
+                    8.8.8.0/24 via 19.4.4.4
+                    %(extra_subnet_cidr)s dev %(ex_device_name)s scope link
+                }
+            }""") % {
             'email_from': keepalived.KEEPALIVED_EMAIL_FROM,
             'router_id': keepalived.KEEPALIVED_ROUTER_ID,
             'ha_device_name': ha_device_name,
@@ -691,8 +687,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             self._assert_ip_address_on_interface(namespace, interface,
                                                  ip_address)
 
-    def _assert_ip_address_not_on_interface(
-        self, namespace, interface, ip_address):
+    def _assert_ip_address_not_on_interface(self,
+                                            namespace, interface, ip_address):
         self.assertNotIn(
             ip_address, self._get_addresses_on_device(namespace, interface))
 
@@ -701,8 +697,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         self.assertIn(
             ip_address, self._get_addresses_on_device(namespace, interface))
 
-    def _assert_ping_reply_from_expected_address(
-        self, ping_result, expected_address):
+    def _assert_ping_reply_from_expected_address(self, ping_result,
+                                                 expected_address):
         ping_results = ping_result.split('\n')
         self.assertGreater(
             len(ping_results), 1,
