@@ -98,7 +98,7 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
         gw_port = router.get_ex_gw_port()
         interface_name = router.get_external_device_name(gw_port['id'])
         device = ip_lib.IPDevice(interface_name, namespace=router.ns_name)
-        self.assertIn('gateway', device.route.get_gateway())
+        self.assertIn('via', device.route.get_gateway())
 
         # Make this copy, so that the agent will think there is change in
         # external gateway port.
@@ -320,7 +320,7 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
         # Verify that the ping replys with fip
         ns_ip_wrapper = ip_lib.IPWrapper(src_machine.namespace)
         result = ns_ip_wrapper.netns.execute(
-            ['ping', '-W', 5, '-c', 1, dst_fip])
+            ['ping', '-c', 1, '-W', 5, dst_fip])
         self._assert_ping_reply_from_expected_address(result, dst_fip)
 
     def _setup_address_scope(self, internal_address_scope1,
