@@ -15,14 +15,13 @@
 import os.path
 
 import mock
+from neutron_lib import fixture as lib_fixtures
 from oslo_config import cfg
 from oslo_utils import fileutils
 import psutil
 
 from neutron.agent.linux import external_process as ep
-from neutron.common import utils as common_utils
 from neutron.tests import base
-from neutron.tests import tools
 
 
 TEST_UUID = 'test-uuid'
@@ -170,24 +169,6 @@ class TestProcessManager(base.BaseTestCase):
                 manager.enable(callback)
                 self.assertFalse(callback.called)
 
-    def test_enable_with_ensure_active(self):
-        def _create_cmd(*args):
-            return ['sleep', 0]
-
-        pm = ep.ProcessManager(self.conf, 'uuid', pid_file='pid_file',
-                               default_cmd_callback=_create_cmd)
-        with mock.patch.object(psutil, 'Process') as mock_psutil_process, \
-                mock.patch.object(ep.ProcessManager, 'pid',
-                                  new_callable=mock.PropertyMock) as mock_pid:
-            mock_pid.return_value = 'pid_value'
-            mock_process = mock.Mock()
-            mock_process.cmdline.side_effect = [[], ['the', 'cmd', 'uuid']]
-            mock_psutil_process.return_value = mock_process
-            try:
-                pm.enable(ensure_active=True)
-            except common_utils.WaitTimeout:
-                self.fail('ProcessManager.enable() raised WaitTimeout')
-
     def test_reload_cfg_without_custom_reload_callback(self):
         with mock.patch.object(ep.ProcessManager, 'disable') as disable:
             manager = ep.ProcessManager(self.conf, 'uuid', namespace='ns')
@@ -312,12 +293,12 @@ class TestProcessManager(base.BaseTestCase):
         self.assertEqual(retval, '/var/path/uuid.pid')
 
     def test_pid(self):
-        self.useFixture(tools.OpenFixture('/var/path/uuid.pid', '5'))
+        self.useFixture(lib_fixtures.OpenFixture('/var/path/uuid.pid', '5'))
         manager = ep.ProcessManager(self.conf, 'uuid')
         self.assertEqual(manager.pid, 5)
 
     def test_pid_no_an_int(self):
-        self.useFixture(tools.OpenFixture('/var/path/uuid.pid', 'foo'))
+        self.useFixture(lib_fixtures.OpenFixture('/var/path/uuid.pid', 'foo'))
         manager = ep.ProcessManager(self.conf, 'uuid')
         self.assertIsNone(manager.pid)
 
