@@ -790,7 +790,7 @@ class TestIpAddrCommand(TestIPCmdBase):
         self.addr_cmd.list(scope='link')
         mock_get_dev_ip.assert_called_once_with('test_ns',
                                                 name=self.addr_cmd.name,
-                                                scope='link')
+                                                scope=253)
 
     @mock.patch.object(ip_lib, 'get_devices_with_ip')
     def test_list_to(self, mock_get_dev_ip):
@@ -841,282 +841,6 @@ class TestIpAddrCommand(TestIPCmdBase):
         self.assertEqual(0, len(retval))
 
 
-class TestIpRouteCommand(TestIPCmdBase):
-    def setUp(self):
-        super(TestIpRouteCommand, self).setUp()
-        self.parent.name = 'eth0'
-        self.command = 'route'
-        self.route_cmd = ip_lib.IpRouteCommand(self.parent)
-        self.ip_version = constants.IP_VERSION_4
-        self.table = 14
-        self.metric = 100
-        self.cidr = '192.168.45.100/24'
-        self.ip = '10.0.0.1'
-        self.gateway = '192.168.45.100'
-        self.test_cases = [{'sample': GATEWAY_SAMPLE1,
-                            'expected': {'gateway': '10.35.19.254',
-                                         'metric': 100}},
-                           {'sample': GATEWAY_SAMPLE2,
-                            'expected': {'gateway': '10.35.19.254',
-                                         'metric': 100}},
-                           {'sample': GATEWAY_SAMPLE3,
-                            'expected': None},
-                           {'sample': GATEWAY_SAMPLE4,
-                            'expected': {'gateway': '10.35.19.254'}},
-                           {'sample': GATEWAY_SAMPLE5,
-                            'expected': {'gateway': '192.168.99.1'}},
-                           {'sample': GATEWAY_SAMPLE6,
-                            'expected': {'gateway': '192.168.99.1',
-                                         'metric': 100}},
-                           {'sample': GATEWAY_SAMPLE7,
-                            'expected': {'metric': 1}}]
-
-    def test_add_gateway(self):
-        self.route_cmd.add_gateway(self.gateway, self.metric, self.table)
-        self._assert_sudo([self.ip_version],
-                          ('replace', 'default',
-                           'via', self.gateway,
-                           'metric', self.metric,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_add_gateway_subtable(self):
-        self.route_cmd.table(self.table).add_gateway(self.gateway, self.metric)
-        self._assert_sudo([self.ip_version],
-                          ('replace', 'default',
-                           'via', self.gateway,
-                           'metric', self.metric,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_del_gateway_success(self):
-        self.route_cmd.delete_gateway(self.gateway, table=self.table)
-        self._assert_sudo([self.ip_version],
-                          ('del', 'default',
-                           'via', self.gateway,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_del_gateway_success_subtable(self):
-        self.route_cmd.table(table=self.table).delete_gateway(self.gateway)
-        self._assert_sudo([self.ip_version],
-                          ('del', 'default',
-                           'via', self.gateway,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_del_gateway_cannot_find_device(self):
-        self.parent._as_root.side_effect = RuntimeError("Cannot find device")
-
-        exc = self.assertRaises(exceptions.DeviceNotFoundError,
-                          self.route_cmd.delete_gateway,
-                          self.gateway, table=self.table)
-        self.assertIn(self.parent.name, str(exc))
-
-    def test_del_gateway_other_error(self):
-        self.parent._as_root.side_effect = RuntimeError()
-
-        self.assertRaises(RuntimeError, self.route_cmd.delete_gateway,
-                          self.gateway, table=self.table)
-
-    def test_get_gateway(self):
-        for test_case in self.test_cases:
-            self.parent._run = mock.Mock(return_value=test_case['sample'])
-            self.assertEqual(self.route_cmd.get_gateway(),
-                             test_case['expected'])
-
-    def test_flush_route_table(self):
-        self.route_cmd.flush(self.ip_version, self.table)
-        self._assert_sudo([self.ip_version], ('flush', 'table', self.table))
-
-    def test_add_route(self):
-        self.route_cmd.add_route(self.cidr, self.ip, self.table)
-        self._assert_sudo([self.ip_version],
-                          ('replace', self.cidr,
-                           'via', self.ip,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_add_route_no_via(self):
-        self.route_cmd.add_route(self.cidr, table=self.table)
-        self._assert_sudo([self.ip_version],
-                          ('replace', self.cidr,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_add_route_with_scope(self):
-        self.route_cmd.add_route(self.cidr, scope='link')
-        self._assert_sudo([self.ip_version],
-                          ('replace', self.cidr,
-                           'dev', self.parent.name,
-                           'scope', 'link'))
-
-    def test_add_route_no_device(self):
-        self.parent._as_root.side_effect = RuntimeError("Cannot find device")
-        self.assertRaises(exceptions.DeviceNotFoundError,
-                          self.route_cmd.add_route,
-                          self.cidr, self.ip, self.table)
-
-    def test_delete_route(self):
-        self.route_cmd.delete_route(self.cidr, self.ip, self.table)
-        self._assert_sudo([self.ip_version],
-                          ('del', self.cidr,
-                           'via', self.ip,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_delete_route_no_via(self):
-        self.route_cmd.delete_route(self.cidr, table=self.table)
-        self._assert_sudo([self.ip_version],
-                          ('del', self.cidr,
-                           'dev', self.parent.name,
-                           'table', self.table))
-
-    def test_delete_route_with_scope(self):
-        self.route_cmd.delete_route(self.cidr, scope='link')
-        self._assert_sudo([self.ip_version],
-                          ('del', self.cidr,
-                           'dev', self.parent.name,
-                           'scope', 'link'))
-
-    def test_delete_route_no_device(self):
-        self.parent._as_root.side_effect = RuntimeError("Cannot find device")
-        self.assertRaises(exceptions.DeviceNotFoundError,
-                          self.route_cmd.delete_route,
-                          self.cidr, self.ip, self.table)
-
-    def test_list_routes(self):
-        self.parent._run.return_value = (
-            "default via 172.124.4.1 dev eth0 metric 100\n"
-            "10.0.0.0/22 dev eth0 scope link\n"
-            "172.24.4.0/24 dev eth0 proto kernel src 172.24.4.2\n")
-        routes = self.route_cmd.table(self.table).list_routes(self.ip_version)
-        self.assertEqual([{'cidr': '0.0.0.0/0',
-                           'dev': 'eth0',
-                           'metric': '100',
-                           'table': 14,
-                           'via': '172.124.4.1'},
-                          {'cidr': '10.0.0.0/22',
-                           'dev': 'eth0',
-                           'scope': 'link',
-                           'table': 14},
-                          {'cidr': '172.24.4.0/24',
-                           'dev': 'eth0',
-                           'proto': 'kernel',
-                           'src': '172.24.4.2',
-                           'table': 14}], routes)
-
-    def test_list_onlink_routes_subtable(self):
-        self.parent._run.return_value = (
-            "10.0.0.0/22\n"
-            "172.24.4.0/24 proto kernel src 172.24.4.2\n")
-        routes = self.route_cmd.table(self.table).list_onlink_routes(
-            self.ip_version)
-        self.assertEqual(['10.0.0.0/22'], [r['cidr'] for r in routes])
-        self._assert_call([self.ip_version],
-                          ('list', 'dev', self.parent.name,
-                           'table', self.table, 'scope', 'link'))
-
-    def test_add_onlink_route_subtable(self):
-        self.route_cmd.table(self.table).add_onlink_route(self.cidr)
-        self._assert_sudo([self.ip_version],
-                          ('replace', self.cidr,
-                           'dev', self.parent.name,
-                           'table', self.table,
-                           'scope', 'link'))
-
-    def test_delete_onlink_route_subtable(self):
-        self.route_cmd.table(self.table).delete_onlink_route(self.cidr)
-        self._assert_sudo([self.ip_version],
-                          ('del', self.cidr,
-                           'dev', self.parent.name,
-                           'table', self.table,
-                           'scope', 'link'))
-
-
-class TestIPv6IpRouteCommand(TestIpRouteCommand):
-    def setUp(self):
-        super(TestIPv6IpRouteCommand, self).setUp()
-        self.ip_version = constants.IP_VERSION_6
-        self.cidr = '2001:db8::/64'
-        self.ip = '2001:db8::100'
-        self.gateway = '2001:db8::1'
-        self.test_cases = [{'sample': IPv6_GATEWAY_SAMPLE1,
-                            'expected':
-                            {'gateway': '2001:470:9:1224:4508:b885:5fb:740b',
-                             'metric': 100}},
-                           {'sample': IPv6_GATEWAY_SAMPLE2,
-                            'expected':
-                            {'gateway': '2001:470:9:1224:4508:b885:5fb:740b',
-                             'metric': 100}},
-                           {'sample': IPv6_GATEWAY_SAMPLE3,
-                            'expected': None},
-                           {'sample': IPv6_GATEWAY_SAMPLE4,
-                            'expected':
-                            {'gateway': 'fe80::dfcc:aaff:feb9:76ce'}},
-                           {'sample': IPv6_GATEWAY_SAMPLE5,
-                            'expected':
-                            {'gateway': '2001:470:9:1224:4508:b885:5fb:740b',
-                             'metric': 1024}}]
-
-    def test_list_routes(self):
-        self.parent._run.return_value = (
-            "default via 2001:db8::1 dev eth0 metric 100\n"
-            "2001:db8::/64 dev eth0 proto kernel src 2001:db8::2\n")
-        routes = self.route_cmd.table(self.table).list_routes(self.ip_version)
-        self.assertEqual([{'cidr': '::/0',
-                           'dev': 'eth0',
-                           'metric': '100',
-                           'table': 14,
-                           'via': '2001:db8::1'},
-                          {'cidr': '2001:db8::/64',
-                           'dev': 'eth0',
-                           'proto': 'kernel',
-                           'src': '2001:db8::2',
-                           'table': 14}], routes)
-
-
-class TestIPRoute(TestIpRouteCommand):
-    """Leverage existing tests for IpRouteCommand for IPRoute
-
-    This test leverages the tests written for IpRouteCommand.  The difference
-    is that the 'dev' argument should not be passed for each of the commands.
-    So, this test removes the dev argument from the expected arguments in each
-    assert.
-    """
-    def setUp(self):
-        super(TestIPRoute, self).setUp()
-        self.parent = ip_lib.IPRoute()
-        self.parent._run = mock.Mock()
-        self.parent._as_root = mock.Mock()
-        self.route_cmd = self.parent.route
-        self.check_dev_args = False
-
-    def _remove_dev_args(self, args):
-        def args_without_dev():
-            previous = None
-            for arg in args:
-                if 'dev' not in (arg, previous):
-                    yield arg
-                previous = arg
-
-        return tuple(arg for arg in args_without_dev())
-
-    def _assert_call(self, options, args):
-        if not self.check_dev_args:
-            args = self._remove_dev_args(args)
-        super(TestIPRoute, self)._assert_call(options, args)
-
-    def _assert_sudo(self, options, args, use_root_namespace=False):
-        if not self.check_dev_args:
-            args = self._remove_dev_args(args)
-        super(TestIPRoute, self)._assert_sudo(options, args)
-
-    def test_del_gateway_cannot_find_device(self):
-        # This test doesn't make sense for this case since dev won't be passed
-        pass
-
-
 class TestIpNetnsCommand(TestIPCmdBase):
     def setUp(self):
         super(TestIpNetnsCommand, self).setUp()
@@ -1212,21 +936,15 @@ class TestDeviceExists(base.BaseTestCase):
 
     def test_ensure_device_is_ready_no_link_address(self):
         with mock.patch.object(
-                priv_lib, 'get_link_attributes') as get_link_attributes, \
-                mock.patch.object(priv_lib, 'set_link_attribute') as \
-                set_link_attribute, \
-                mock.patch.object(priv_lib, 'interface_exists',
-                                  return_value=True):
+            priv_lib, 'get_link_attributes'
+        ) as get_link_attributes, mock.patch.object(
+            priv_lib, 'set_link_attribute'
+        ) as set_link_attribute:
             get_link_attributes.return_value = {}
             self.assertFalse(ip_lib.ensure_device_is_ready("lo"))
             get_link_attributes.assert_called_once_with("lo", None)
             set_link_attribute.assert_not_called()
 
-    def test_ensure_device_is_ready_no_device(self):
-        with mock.patch.object(priv_lib, 'interface_exists',
-                               return_value=False):
-            self.assertFalse(ip_lib.ensure_device_is_ready("lo"))
-
 
 class TestGetRoutingTable(base.BaseTestCase):
     ip_db_interfaces = {
@@ -1896,20 +1614,21 @@ class ParseLinkDeviceTestCase(base.BaseTestCase):
     def test_parse_link_devices(self):
         device = ({'index': 1, 'attrs': [['IFLA_IFNAME', 'int_name']]})
         self.mock_get_ip_addresses.return_value = [
-            {'prefixlen': 24, 'scope': 200, 'attrs': [
+            {'prefixlen': 24, 'scope': 200, 'event': 'RTM_NEWADDR', 'attrs': [
                 ['IFA_ADDRESS', '192.168.10.20'],
                 ['IFA_FLAGS', ifaddrmsg.IFA_F_PERMANENT]]},
-            {'prefixlen': 64, 'scope': 200, 'attrs': [
+            {'prefixlen': 64, 'scope': 200, 'event': 'RTM_DELADDR', 'attrs': [
                 ['IFA_ADDRESS', '2001:db8::1'],
                 ['IFA_FLAGS', ifaddrmsg.IFA_F_PERMANENT]]}]
 
         retval = ip_lib._parse_link_device('namespace', device)
         expected = [{'scope': 'site', 'cidr': '192.168.10.20/24',
                      'dynamic': False, 'dadfailed': False, 'name': 'int_name',
-                     'broadcast': None, 'tentative': False},
+                     'broadcast': None, 'tentative': False, 'event': 'added'},
                     {'scope': 'site', 'cidr': '2001:db8::1/64',
                      'dynamic': False, 'dadfailed': False, 'name': 'int_name',
-                     'broadcast': None, 'tentative': False}]
+                     'broadcast': None, 'tentative': False,
+                     'event': 'removed'}]
         self.assertEqual(expected, retval)
 
 
