@@ -14,7 +14,9 @@
 
 import mock
 
+from neutron_lib.api.definitions import dvr as dvr_apidef
 from neutron_lib.api.definitions import external_net as extnet_apidef
+from neutron_lib.api.definitions import l3_ext_ha_mode
 from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as providernet
@@ -24,6 +26,7 @@ from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.exceptions import l3_ext_ha_mode as l3ha_exc
@@ -39,13 +42,10 @@ import testtools
 
 from neutron.agent.common import utils as agent_utils
 from neutron.api.rpc.handlers import l3_rpc
-from neutron.common import constants as n_const
 from neutron.db import agents_db
-from neutron.db import common_db_mixin
 from neutron.db import l3_agentschedulers_db
 from neutron.db import l3_hamode_db
 from neutron.objects import l3_hamode
-from neutron import quota
 from neutron.scheduler import l3_agent_scheduler
 from neutron.services.revisions import revision_plugin
 from neutron.tests.common import helpers
@@ -56,7 +56,6 @@ _uuid = uuidutils.generate_uuid
 
 
 class FakeL3PluginWithAgents(test_l3.TestL3PluginBaseAttributes,
-                             common_db_mixin.CommonDbMixin,
                              l3_hamode_db.L3_HA_NAT_db_mixin,
                              l3_agentschedulers_db.L3AgentSchedulerDbMixin,
                              agents_db.AgentDbMixin):
@@ -72,10 +71,6 @@ class L3HATestFramework(testlib_api.SqlTestCase):
         notif_p = mock.patch.object(l3_hamode_db.L3_HA_NAT_db_mixin,
                                     '_notify_router_updated')
         self.notif_m = notif_p.start()
-        make_res = mock.patch.object(quota.QuotaEngine, 'make_reservation')
-        self.mock_quota_make_res = make_res.start()
-        commit_res = mock.patch.object(quota.QuotaEngine, 'commit_reservation')
-        self.mock_quota_commit_res = commit_res.start()
         cfg.CONF.set_override('allow_overlapping_ips', True)
 
         self.plugin = FakeL3PluginWithAgents()
@@ -196,56 +191,56 @@ class L3HATestCase(L3HATestFramework):
             self):
         router = self._create_router()
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent2['host'])
         with mock.patch.object(agent_utils, 'is_agent_down',
                                return_value=True):
             self._assert_ha_state_for_agent(router, self.agent1,
-                                            n_const.HA_ROUTER_STATE_UNKNOWN)
+                                            constants.HA_ROUTER_STATE_UNKNOWN)
 
     def test_get_l3_bindings_hosting_router_agents_admin_state_up_is_false(
             self):
         router = self._create_router()
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent2['host'])
         helpers.set_agent_admin_state(self.agent1['id'], admin_state_up=False)
         self._assert_ha_state_for_agent(router, self.agent1,
-                                        n_const.HA_ROUTER_STATE_STANDBY)
+                                        constants.HA_ROUTER_STATE_STANDBY)
 
     def test_get_l3_bindings_hosting_router_agents_admin_state_up_is_true(
             self):
         router = self._create_router()
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent2['host'])
         helpers.set_agent_admin_state(self.agent1['id'], admin_state_up=True)
         self._assert_ha_state_for_agent(router, self.agent1,
-                                        n_const.HA_ROUTER_STATE_ACTIVE)
+                                        constants.HA_ROUTER_STATE_ACTIVE)
 
     def test_get_l3_bindings_hosting_router_with_ha_states_one_dead(self):
         router = self._create_router()
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_STANDBY},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_STANDBY},
             self.agent2['host'])
         with mock.patch.object(agent_utils, 'is_agent_down',
                                return_value=True):
             # With above mock all agents are in dead state
             # hence router state is Unknown overall.
             self._assert_ha_state_for_agent(
-                router, self.agent1, n_const.HA_ROUTER_STATE_UNKNOWN)
+                router, self.agent1, constants.HA_ROUTER_STATE_UNKNOWN)
 
     def test_ha_router_create(self):
         router = self._create_router()
@@ -407,7 +402,8 @@ class L3HATestCase(L3HATestFramework):
     def test_get_ha_sync_data_for_host_with_non_dvr_agent(self):
         with mock.patch.object(self.plugin,
                                '_get_dvr_sync_data') as mock_get_sync:
-            self.plugin.supported_extension_aliases = ['dvr', 'l3-ha']
+            self.plugin.supported_extension_aliases = [
+                dvr_apidef.ALIAS, l3_ext_ha_mode.ALIAS]
             self.plugin.get_ha_sync_data_for_host(self.admin_ctx,
                                                   self.agent1['host'],
                                                   self.agent1)
@@ -416,7 +412,8 @@ class L3HATestCase(L3HATestFramework):
     def test_get_ha_sync_data_for_host_with_dvr_agent(self):
         with mock.patch.object(self.plugin,
                                '_get_dvr_sync_data') as mock_get_sync:
-            self.plugin.supported_extension_aliases = ['dvr', 'l3-ha']
+            self.plugin.supported_extension_aliases = [
+                dvr_apidef.ALIAS, l3_ext_ha_mode.ALIAS]
             self.plugin.get_ha_sync_data_for_host(self.admin_ctx,
                                                   self.agent2['host'],
                                                   self.agent2)
@@ -599,7 +596,7 @@ class L3HATestCase(L3HATestFramework):
 
     def test_add_ha_port_subtransactions_blocked(self):
         ctx = self.admin_ctx
-        with ctx.session.begin():
+        with db_api.CONTEXT_WRITER.using(ctx):
             self.assertRaises(RuntimeError, self.plugin.add_ha_port,
                               ctx, 'id', 'id', 'id')
 
@@ -778,7 +775,7 @@ class L3HATestCase(L3HATestFramework):
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
         for router in routers:
-            self.assertEqual('standby', router[n_const.HA_ROUTER_STATE_KEY])
+            self.assertEqual('standby', router[constants.HA_ROUTER_STATE_KEY])
 
         states = {router1['id']: 'active',
                   router2['id']: 'standby'}
@@ -789,7 +786,7 @@ class L3HATestCase(L3HATestFramework):
             self.admin_ctx, self.agent1['host'], self.agent1)
         for router in routers:
             self.assertEqual(states[router['id']],
-                             router[n_const.HA_ROUTER_STATE_KEY])
+                             router[constants.HA_ROUTER_STATE_KEY])
 
     def test_sync_ha_router_info_ha_interface_port_concurrently_deleted(self):
         ctx = self.admin_ctx
@@ -856,7 +853,7 @@ class L3HATestCase(L3HATestFramework):
                             router2['id']: 'active'})
         routers = self.plugin.get_ha_sync_data_for_host(
             self.admin_ctx, self.agent1['host'], self.agent1)
-        self.assertEqual('active', routers[0][n_const.HA_ROUTER_STATE_KEY])
+        self.assertEqual('active', routers[0][constants.HA_ROUTER_STATE_KEY])
 
     def test_update_routers_states_port_not_found(self):
         router1 = self._create_router()
@@ -1389,10 +1386,10 @@ class L3HAModeDbTestCase(L3HATestFramework):
                                          interface_info)
 
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_STANDBY},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_STANDBY},
             self.agent2['host'])
 
         routers = self.plugin._get_sync_routers(self.admin_ctx,
@@ -1400,10 +1397,10 @@ class L3HAModeDbTestCase(L3HATestFramework):
         self.assertEqual(self.agent1['host'], routers[0]['gw_port_host'])
 
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_STANDBY},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_STANDBY},
             self.agent1['host'])
         self.plugin.update_routers_states(
-            self.admin_ctx, {router['id']: n_const.HA_ROUTER_STATE_ACTIVE},
+            self.admin_ctx, {router['id']: constants.HA_ROUTER_STATE_ACTIVE},
             self.agent2['host'])
         routers = self.plugin._get_sync_routers(self.admin_ctx,
                                                 router_ids=[router['id']])
