@@ -24,7 +24,9 @@ from neutron_lib.db import api as db_api
 from neutron_lib.db import model_query
 from neutron_lib import exceptions as n_exc
 from neutron_lib.objects import exceptions as o_exc
+from neutron_lib.objects.logapi import event_types
 from neutron_lib.objects import utils as obj_utils
+from neutron_lib.tests import tools as lib_test_tools
 from neutron_lib.utils import helpers
 from oslo_db import exception as obj_exc
 from oslo_db.sqlalchemy import utils as db_utils
@@ -41,7 +43,6 @@ from neutron.objects import base
 from neutron.objects import common_types
 from neutron.objects.db import api as obj_db_api
 from neutron.objects import flavor
-from neutron.objects.logapi import event_types
 from neutron.objects import network as net_obj
 from neutron.objects import ports
 from neutron.objects.qos import policy as qos_policy
@@ -452,7 +453,7 @@ def get_random_dscp_mark():
 
 def get_list_of_random_networks(num=10):
     for i in range(5):
-        res = [tools.get_random_ip_network() for i in range(num)]
+        res = [lib_test_tools.get_random_ip_network() for i in range(num)]
         # make sure there are no duplicates
         if len(set(res)) == num:
             return res
@@ -511,13 +512,13 @@ FIELD_TYPE_VALUE_GENERATOR_MAP = {
     common_types.FlowDirectionEnumField: tools.get_random_flow_direction,
     common_types.HARouterEnumField: tools.get_random_ha_states,
     common_types.IpamAllocationStatusEnumField: tools.get_random_ipam_status,
-    common_types.IPNetworkField: tools.get_random_ip_network,
+    common_types.IPNetworkField: lib_test_tools.get_random_ip_network,
     common_types.IPNetworkPrefixLenField: tools.get_random_prefixlen,
     common_types.IPV6ModeEnumField: tools.get_random_ipv6_mode,
     common_types.IPVersionEnumField: tools.get_random_ip_version,
     common_types.IpProtocolEnumField: tools.get_random_ip_protocol,
     common_types.ListOfIPNetworksField: get_list_of_random_networks,
-    common_types.MACAddressField: tools.get_random_EUI,
+    common_types.MACAddressField: lib_test_tools.get_random_EUI,
     common_types.NetworkSegmentRangeNetworkTypeEnumField:
         tools.get_random_network_segment_range_network_type,
     common_types.PortBindingStatusEnumField:
@@ -725,7 +726,6 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
         self.model_map = collections.defaultdict(list)
         self.model_map[self._test_class.db_model] = self.db_objs
         self.pager_map = collections.defaultdict(lambda: None)
-        self.extra_fields_not_in_dict = []
 
         self.get_objects_mock = mock.patch.object(
             obj_db_api, 'get_objects',
@@ -1156,11 +1156,8 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
         for field in self._test_class.obj_extra_fields:
             # field is accessible and cannot be set by any value
             getattr(obj, field)
-            if field in self.extra_fields_not_in_dict:
-                self.assertNotIn(field, obj.to_dict().keys())
-            else:
-                self.assertIn(field, obj.to_dict().keys())
-                self.assertRaises(AttributeError, setattr, obj, field, "1")
+            self.assertIn(field, obj.to_dict().keys())
+            self.assertRaises(AttributeError, setattr, obj, field, "1")
 
     def test_to_dict_makes_primitive_field_value(self):
         obj = self._test_class(self.context, **self.obj_fields[0])
@@ -1211,7 +1208,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
                                '_get_changed_persistent_fields',
                                return_value=fields_to_update):
             with mock.patch.object(obj_db_api, 'get_objects',
-                side_effect=self.fake_get_objects):
+                                   side_effect=self.fake_get_objects):
                 obj = self._test_class(self.context, **self.obj_fields[0])
                 # get new values and fix keys
                 update_mock.return_value = self.db_objs[1]
@@ -1254,9 +1251,8 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
                 with mock.patch.object(base.NeutronDbObject,
                                        '_get_changed_persistent_fields',
                                        return_value=fields_to_update):
-                    with mock.patch.object(
-                        obj_db_api, 'get_objects',
-                        side_effect=self.fake_get_objects):
+                    with mock.patch.object(obj_db_api, 'get_objects',
+                                           side_effect=self.fake_get_objects):
                         obj.update()
                 self._check_equal(self.objs[0], obj)
 
@@ -1630,12 +1626,8 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         self._router.create()
         return self._router['id']
 
-    def _create_test_security_group_id(self, fields=None):
+    def _create_test_security_group_id(self):
         sg_fields = self.get_random_object_fields(securitygroup.SecurityGroup)
-        fields = fields or {}
-        for field, value in ((f, v) for (f, v) in fields.items() if
-                             f in sg_fields):
-            sg_fields[field] = value
         _securitygroup = securitygroup.SecurityGroup(
             self.context, **sg_fields)
         _securitygroup.create()
