@@ -77,18 +77,6 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 priority=65535,
                 table_id=0),
                            active_bundle=None),
-            call._send_msg(ofpp.OFPFlowMod(dp,
-                cookie=self.stamp,
-                instructions=[
-                    ofpp.OFPInstructionActions(
-                        ofp.OFPIT_APPLY_ACTIONS, [
-                            ofpp.OFPActionOutput(ofp.OFPP_NORMAL, 0)
-                        ]),
-                ],
-                match=ofpp.OFPMatch(),
-                priority=3,
-                table_id=constants.TRANSIENT_EGRESS_TABLE),
-                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -195,7 +183,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=20,
+                priority=4,
                 table_id=1),
                            active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
@@ -209,7 +197,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=20,
+                priority=4,
                 table_id=60),
                            active_bundle=None),
         ]
@@ -226,14 +214,14 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
         expected = [
             call.uninstall_flows(
                 strict=True,
-                priority=20,
+                priority=4,
                 table_id=1,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT)),
             call.uninstall_flows(
                 strict=True,
-                priority=20,
+                priority=4,
                 table_id=60,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -265,7 +253,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=20,
+                priority=4,
                 table_id=2),
                            active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
@@ -279,7 +267,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT),
-                priority=20,
+                priority=4,
                 table_id=60),
                            active_bundle=None),
         ]
@@ -296,14 +284,14 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
         expected = [
             call.uninstall_flows(
                 strict=True,
-                priority=20,
+                priority=4,
                 table_id=2,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
                     vlan_vid=vlan_tag | ofp.OFPVID_PRESENT)),
             call.uninstall_flows(
                 strict=True,
-                priority=20,
+                priority=4,
                 table_id=60,
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
