@@ -18,7 +18,9 @@ import contextlib
 import datetime
 
 import mock
+from neutron_lib.api.definitions import l3_ext_ha_mode
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import router_availability_zone
 from neutron_lib import constants
 from neutron_lib import context as n_context
 from neutron_lib.exceptions import l3 as l3_exc
@@ -901,7 +903,7 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
         }
         port = kwargs.get('port')
         plugin = directory.get_plugin()
-        l3plugin = mock.Mock()
+        l3plugin = mock.MagicMock()
         l3plugin.supported_extension_aliases = [
             'router', constants.L3_AGENT_SCHEDULER_EXT_ALIAS,
             constants.L3_DISTRIBUTED_EXT_ALIAS
@@ -984,6 +986,7 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
             'original_port': {
                 portbindings.HOST_ID: 'vm-host',
                 'device_owner': DEVICE_OWNER_COMPUTE,
+                'id': uuidutils.generate_uuid()
             },
             'port': {
                 portbindings.HOST_ID: 'vm-host',
@@ -991,15 +994,18 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
                 portbindings.PROFILE: {'migrating_to': 'vm-host2'},
             },
         }
-        l3plugin = mock.Mock()
+        l3plugin = mock.MagicMock()
         directory.add_plugin(plugin_constants.L3, l3plugin)
-        l3_dvrscheduler_db._notify_l3_agent_port_update(
-            'port', 'after_update', mock.ANY, **kwargs)
-        l3plugin.dvr_handle_new_service_port.assert_called_once_with(
-            self.adminContext, kwargs.get('port'), dest_host='vm-host2')
-        l3plugin.update_arp_entry_for_dvr_service_port.\
-            assert_called_once_with(
-                self.adminContext, kwargs.get('port'))
+        with mock.patch.object(l3plugin, '_get_floatingips_by_port_id',
+                               return_value=[]):
+            l3_dvrscheduler_db._notify_l3_agent_port_update(
+                    'port', 'after_update', mock.ANY, **kwargs)
+            l3plugin.dvr_handle_new_service_port.assert_called_once_with(
+                    self.adminContext, kwargs.get('port'),
+                    dest_host='vm-host2', router_id=None)
+            l3plugin.update_arp_entry_for_dvr_service_port.\
+                assert_called_once_with(
+                        self.adminContext, kwargs.get('port'))
 
     def test__notify_l3_agent_update_port_no_action(self):
         kwargs = {
@@ -1113,17 +1119,22 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
         router_to_remove = [{'agent_id': 'foo_agent',
                              'router_id': 'foo_id',
                              'host': 'vm-host1'}]
-        self._test__notify_l3_agent_port_binding_change(router_to_remove, fip)
+        self._test__notify_l3_agent_port_binding_change(
+                router_to_remove, fip,
+                router_id=fip['router_id'])
 
     def test__notify_l3_agent_port_binding_change_with_fip(self):
         fip = {'router_id': 'router_id'}
-        self._test__notify_l3_agent_port_binding_change(None, fip)
+        self._test__notify_l3_agent_port_binding_change(
+                None, fip, router_id=fip['router_id'])
 
     def test__notify_l3_agent_port_binding_change_fip_dvr(self):
         fip = {'router_id': 'router_id'}
         is_distributed = True
-        self._test__notify_l3_agent_port_binding_change(None,
-                                                        fip, is_distributed)
+        self._test__notify_l3_agent_port_binding_change(
+                None,
+                fip, is_distributed,
+                router_id=fip['router_id'])
 
     def test__notify_l3_agent_port_binding_change_fip_dvr_rmrt(self):
         fip = {'router_id': 'router_id'}
@@ -1131,8 +1142,9 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
                              'router_id': 'foo_id',
                              'host': 'vm-host1'}]
         is_distributed = True
-        self._test__notify_l3_agent_port_binding_change(router_to_remove,
-                                                        fip, is_distributed)
+        self._test__notify_l3_agent_port_binding_change(
+                router_to_remove, fip, is_distributed,
+                router_id=fip['router_id'])
 
     def test__notify_l3_agent_port_binding_change_fip_dvr_on_rmrt(self):
         fip = {'router_id': 'foo_id'}
@@ -1140,13 +1152,15 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
                              'router_id': 'foo_id',
                              'host': 'vm-host1'}]
         is_distributed = True
-        self._test__notify_l3_agent_port_binding_change(router_to_remove,
-                                                        fip, is_distributed)
+        self._test__notify_l3_agent_port_binding_change(
+                router_to_remove, fip, is_distributed,
+                router_id=fip['router_id'])
 
     def _test__notify_l3_agent_port_binding_change(self,
                                                    routers_to_remove=None,
                                                    fip=None,
-                                                   is_distributed=False):
+                                                   is_distributed=False,
+                                                   router_id=None):
         source_host = 'vm-host1'
         kwargs = {
             'context': self.adminContext,
@@ -1178,13 +1192,14 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
                     1,
                     l3plugin.delete_arp_entry_for_dvr_service_port.call_count)
             if fip and is_distributed and not (routers_to_remove and
-                fip['router_id'] is routers_to_remove[0]['router_id']):
+                    fip['router_id'] is routers_to_remove[0]['router_id']):
                 (l3plugin.l3_rpc_notifier.routers_updated_on_host.
                  assert_called_once_with(mock.ANY, ['router_id'], source_host))
             self.assertEqual(
                 1, l3plugin.update_arp_entry_for_dvr_service_port.call_count)
             l3plugin.dvr_handle_new_service_port.assert_called_once_with(
-                self.adminContext, kwargs.get('port'), dest_host=None)
+                self.adminContext, kwargs.get('port'),
+                dest_host=None, router_id=router_id)
 
     def test__notify_l3_agent_update_port_removing_routers(self):
         port_id = 'fake-port'
@@ -1445,7 +1460,8 @@ class L3DvrSchedulerTestCase(L3SchedulerBaseMixin,
 class L3HAPlugin(db_v2.NeutronDbPluginV2,
                  l3_hamode_db.L3_HA_NAT_db_mixin,
                  l3_hascheduler_db.L3_HA_scheduler_db_mixin):
-    supported_extension_aliases = ["l3-ha", "router_availability_zone"]
+    supported_extension_aliases = [l3_ext_ha_mode.ALIAS,
+                                   router_availability_zone.ALIAS]
 
     @classmethod
     def get_plugin_type(cls):
