@@ -23,8 +23,8 @@ import time
 
 import eventlet.wsgi
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as exception
-from neutron_lib import worker as neutron_worker
 from oslo_config import cfg
 import oslo_i18n
 from oslo_log import log as logging
@@ -41,9 +41,8 @@ import webob.exc
 
 from neutron._i18n import _
 from neutron.common import config
-from neutron.common import exceptions as n_exc
 from neutron.conf import wsgi as wsgi_config
-from neutron.db import api
+from neutron import worker as neutron_worker
 
 CONF = cfg.CONF
 wsgi_config.register_socket_opts()
@@ -59,19 +58,20 @@ def encode_body(body):
     return encodeutils.to_utf8(body)
 
 
-class WorkerService(neutron_worker.BaseWorker):
+class WorkerService(neutron_worker.NeutronBaseWorker):
     """Wraps a worker to be handled by ProcessLauncher"""
-    def __init__(self, service, application, disable_ssl=False,
+    def __init__(self, service, application, set_proctitle, disable_ssl=False,
                  worker_process_count=0):
-        super(WorkerService, self).__init__(worker_process_count)
+        super(WorkerService, self).__init__(worker_process_count,
+                                            set_proctitle)
 
         self._service = service
         self._application = application
         self._disable_ssl = disable_ssl
         self._server = None
 
-    def start(self):
-        super(WorkerService, self).start()
+    def start(self, desc=None):
+        super(WorkerService, self).start(desc=desc)
         # When api worker is stopped it kills the eventlet wsgi server which
         # internally closes the wsgi server socket object. This server socket
         # object becomes not usable which leads to "Bad file descriptor"
@@ -163,7 +163,7 @@ class Server(object):
 
         return sock
 
-    def start(self, application, port, host='0.0.0.0', workers=0):
+    def start(self, application, port, host='0.0.0.0', workers=0, desc=None):
         """Run a WSGI server with the given application."""
         self._host = host
         self._port = port
@@ -175,25 +175,27 @@ class Server(object):
 
         self._launch(application, workers)
 
-    def _launch(self, application, workers=0):
-        service = WorkerService(self, application, self.disable_ssl, workers)
+    def _launch(self, application, workers=0, desc=None):
+        set_proctitle = "off" if desc is None else CONF.setproctitle
+        service = WorkerService(self, application, set_proctitle,
+                                self.disable_ssl, workers)
         if workers < 1:
             # The API service should run in the current process.
             self._server = service
             # Dump the initial option values
             cfg.CONF.log_opt_values(LOG, logging.DEBUG)
-            service.start()
+            service.start(desc=desc)
             systemd.notify_once()
         else:
             # dispose the whole pool before os.fork, otherwise there will
             # be shared DB connections in child processes which may cause
             # DB errors.
-            api.context_manager.dispose_pool()
+            db_api.get_context_manager().dispose_pool()
             # The API service runs in a number of child processes.
             # Minimize the cost of checking for child exit by extending the
             # wait interval past the default of 0.01s.
-            self._server = common_service.ProcessLauncher(cfg.CONF,
-                                                          wait_interval=1.0)
+            self._server = common_service.ProcessLauncher(
+                cfg.CONF, wait_interval=1.0, restart_method='mutate')
             self._server.launch_service(service,
                                         workers=service.worker_process_count)
 
@@ -242,15 +244,17 @@ class Request(wsgi.Request):
             if _format in ['json']:
                 return 'application/{0}'.format(_format)
 
-        #Then look up content header
+        # Then look up content header
         type_from_header = self.get_content_type()
         if type_from_header:
             return type_from_header
         ctypes = ['application/json']
 
-        #Finally search in Accept-* headers
-        bm = self.accept.best_match(ctypes)
-        return bm or 'application/json'
+        # Finally search in Accept-* headers
+        acceptable = self.accept.acceptable_offers(ctypes)
+        if acceptable:
+            return acceptable[0][0]
+        return 'application/json'
 
     def get_content_type(self):
         allowed_types = ("application/json",)
@@ -271,7 +275,11 @@ class Request(wsgi.Request):
         if not self.accept_language:
             return None
         all_languages = oslo_i18n.get_available_languages('neutron')
-        return self.accept_language.best_match(all_languages)
+        best_match = self.accept_language.lookup(all_languages,
+                                                 default='fake_LANG')
+        if best_match == 'fake_LANG':
+            best_match = None
+        return best_match
 
     @property
     def context(self):
@@ -379,7 +387,7 @@ class JSONDeserializer(TextDeserializer):
             return jsonutils.loads(datastring)
         except ValueError:
             msg = _("Cannot understand JSON")
-            raise n_exc.MalformedRequestBody(reason=msg)
+            raise exception.MalformedRequestBody(reason=msg)
 
     def default(self, datastring):
         return {'body': self._from_json(datastring)}
@@ -592,7 +600,7 @@ class Resource(Application):
             msg = _("Unsupported Content-Type")
             LOG.exception("InvalidContentType: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
-        except n_exc.MalformedRequestBody:
+        except exception.MalformedRequestBody:
             msg = _("Malformed request body")
             LOG.exception("MalformedRequestBody: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
@@ -629,7 +637,7 @@ class Resource(Application):
 
         controller_method = getattr(self.controller, action)
         try:
-            #NOTE(salvatore-orlando): the controller method must have
+            # NOTE(salvatore-orlando): the controller method must have
             # an argument whose name is 'request'
             return controller_method(request=request, **action_args)
         except TypeError:
